@@ -4,12 +4,12 @@ from docutils.nodes import reference
 from docutils.parsers.rst.roles import set_classes
 
 
-# adapted from
-# https://doughellmann.com/blog/2010/05/09/defining-custom-roles-in-sphinx/
-
-
 def gh_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
-    """Link to a GitHub issue."""
+    """Link to a GitHub issue.
+
+    adapted from
+    https://doughellmann.com/blog/2010/05/09/defining-custom-roles-in-sphinx/
+    """
     try:
         # issue/PR mode (issues/PR-num will redirect to pull/PR-num)
         int(text)
