@@ -1,21 +1,25 @@
+# -*- coding: utf-8 -*-
 """
 .. _ex-read-neo:
 
+===============================================
 How to use data in neural ensemble (NEO) format
-================================================
+===============================================
 
 This example shows how to create an MNE-Python `~mne.io.Raw` object from data
-in the `neural ensemble <http://neuralensemble.org/neo/>`__ format. For general
+in the `neural ensemble <https://neo.readthedocs.io>`__ format. For general
 information on creating MNE-Python's data objects from NumPy arrays, see
-:ref:`tut_creating_data_structures`.
+:ref:`tut-creating-data-structures`.
 """
 
+# %%
+
 import neo
 import mne
 
-###############################################################################
+# %%
 # This example uses NEO's ``ExampleIO`` object for creating fake data. The data
-# will be all zeros, so the plot won't be very interesting; but it should
+# will be all zeros, so the plot won't be very interesting, but it should
 # demonstrate the steps to using NEO data. For actual data and different file
 # formats, consult the NEO documentation.
 
