@@ -53,7 +53,7 @@ data = np.sum((data.astype(int) & eog_event_id) == eog_event_id, axis=0)
 # %%
 # Plot EOG artifact distribution
 fig, ax = plt.subplots()
-ax.stem(1e3 * epochs.times, data, use_line_collection=True)
+ax.stem(1e3 * epochs.times, data)
 ax.set(xlabel='Times (ms)',
        ylabel='Blink counts (from %s trials)' % len(epochs))
 fig.tight_layout()
