@@ -54,13 +54,13 @@ events = mne.find_events(raw, stim_channel='STI 014')
 events[:, 2] = 1
 raw.plot(events=events)
 
-topo_kwargs = dict(times=[0, 0.1, 0.2], ch_type='mag', vmin=-500, vmax=500,
-                   time_unit='s')
+topo_kwargs = dict(times=[0, 0.1, 0.2], ch_type='mag', vlim=(-500, 500))
 
 # %%
 # First, take the average of stationary data (bilateral auditory patterns).
 evoked_stat = mne.Epochs(raw_stat, events, 1, -0.2, 0.8).average()
-evoked_stat.plot_topomap(title='Stationary', **topo_kwargs)
+fig = evoked_stat.plot_topomap(**topo_kwargs)
+fig.suptitle('Stationary')
 
 # %%
 # Second, take a naive average, which averages across epochs that have been
@@ -68,18 +68,19 @@ evoked_stat.plot_topomap(title='Stationary', **topo_kwargs)
 # spatially smearing the activity.
 epochs = mne.Epochs(raw, events, 1, -0.2, 0.8)
 evoked = epochs.average()
-evoked.plot_topomap(title='Moving: naive average', **topo_kwargs)
+fig = evoked.plot_topomap(**topo_kwargs)
+fig.suptitle('Moving: naive average')
 
 # %%
 # Third, use raw movement compensation (restores pattern).
 raw_sss = maxwell_filter(raw, head_pos=head_pos)
 evoked_raw_mc = mne.Epochs(raw_sss, events, 1, -0.2, 0.8).average()
-evoked_raw_mc.plot_topomap(title='Moving: movement compensated (raw)',
-                           **topo_kwargs)
+fig = evoked_raw_mc.plot_topomap(**topo_kwargs)
+fig.suptitle('Moving: movement compensated (raw)')
 
 # %%
 # Fourth, use evoked movement compensation. For these data, which contain
 # very large rotations, it does not as cleanly restore the pattern.
 evoked_evo_mc = mne.epochs.average_movements(epochs, head_pos=head_pos)
-evoked_evo_mc.plot_topomap(title='Moving: movement compensated (evoked)',
-                           **topo_kwargs)
+fig = evoked_evo_mc.plot_topomap(**topo_kwargs)
+fig.suptitle('Moving: movement compensated (evoked)')
