@@ -2,7 +2,8 @@
 
 from ._lcmv import (make_lcmv, apply_lcmv, apply_lcmv_epochs, apply_lcmv_raw,
                     apply_lcmv_cov)
-from ._dics import make_dics, apply_dics, apply_dics_epochs, apply_dics_csd
+from ._dics import (make_dics, apply_dics, apply_dics_epochs,
+                    apply_dics_tfr_epochs, apply_dics_csd)
 from ._rap_music import rap_music
 from ._compute_beamformer import Beamformer, read_beamformer
 from .resolution_matrix import make_lcmv_resolution_matrix
