@@ -4,7 +4,7 @@
 #          Roman Goj <roman.goj@gmail.com>
 #          Britta Westner <britta.wstnr@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from copy import deepcopy
 
@@ -16,11 +16,10 @@ from ..io.proj import make_projector, Projection
 from ..minimum_norm.inverse import _get_vertno, _prepare_forward
 from ..source_space import label_src_vertno_sel
 from ..utils import (verbose, check_fname, _reg_pinv, _check_option, logger,
-                     _pl, _check_src_normal, check_version, _sym_mat_pow, warn)
+                     _pl, _check_src_normal, check_version, _sym_mat_pow, warn,
+                     _import_h5io_funcs)
 from ..time_frequency.csd import CrossSpectralDensity
 
-from ..externals.h5io import read_hdf5, write_hdf5
-
 
 def _check_proj_match(proj, filters):
     """Check whether SSP projections in data and spatial filter match."""
@@ -57,6 +56,8 @@ def _prepare_beamformer_input(info, forward, label=None, pick_ori=None,
 
     if loose is None:
         loose = 0. if is_fixed_orient(forward) else 1.
+    # TODO: Deduplicate with _check_one_ch_type, should not be necessary
+    # (DICS hits this code path, LCMV does not)
     if noise_cov is None:
         noise_cov = make_ad_hoc_cov(info, std=1.)
     forward, info_picked, gain, _, orient_prior, _, trace_GRGT, noise_cov, \
@@ -330,7 +331,7 @@ def _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
     if weight_norm is not None:
         # Three different ways to calculate the normalization factors here.
         # Only matters when in vector mode, as otherwise n_orient == 1 and
-        # they are all equivalent. Sekihara 2008 says to use
+        # they are all equivalent.
         #
         # In MNE < 0.21, we just used the Frobenius matrix norm:
         #
@@ -386,7 +387,6 @@ def _compute_beamformer(G, Cm, reg, n_orient, weight_norm, pick_ori,
     return W, max_power_ori
 
 
-# TODO: Eventually we can @jit() this to make it faster
 def _compute_power(Cm, W, n_orient):
     """Use beamformer filters to compute source power.
 
@@ -404,10 +404,9 @@ def _compute_power(Cm, W, n_orient):
     """
     n_sources = W.shape[0] // n_orient
 
-    source_power = np.zeros(n_sources)
-    for k in range(n_sources):
-        Wk = W[n_orient * k: n_orient * k + n_orient]
-        source_power[k] = np.trace(Wk @ Cm @ Wk.conj().T).real
+    Wk = W.reshape(n_sources, n_orient, W.shape[1])
+    source_power = np.trace((Wk @ Cm @ Wk.conj().transpose(0, 2, 1)).real,
+                            axis1=1, axis2=2)
 
     return source_power
 
@@ -462,6 +461,8 @@ class Beamformer(dict):
         %(overwrite)s
         %(verbose)s
         """
+        _, write_hdf5 = _import_h5io_funcs()
+
         ending = '-%s.h5' % (self['kind'].lower(),)
         check_fname(fname, self['kind'], (ending,))
         csd_orig = None
@@ -488,6 +489,7 @@ def read_beamformer(fname):
     filter : instance of Beamformer
         The beamformer filter.
     """
+    read_hdf5, _ = _import_h5io_funcs()
     beamformer = read_hdf5(fname, title='mnepython')
     if 'csd' in beamformer:
         beamformer['csd'] = CrossSpectralDensity(**beamformer['csd'])
