@@ -5,7 +5,7 @@ Examples
 --------
 .. code-block:: console
 
-    $ mne compute_proj_ecg -i sample_audvis_raw.fif -c "MEG 1531" \
+    $ mne compute_proj_ecg -i sample_audvis_raw.fif -c "MEG 1531" -a \
                            --l-freq 1 --h-freq 100 \
                            --rej-grad 3000 --rej-mag 4000 --rej-eeg 100
 
@@ -61,7 +61,7 @@ def run():
                       default=True)
     parser.add_option("-a", "--average", dest="average", action="store_true",
                       help="Compute SSP after averaging",
-                      default=False)  # XXX: change to default=True in 0.17
+                      default=False)
     parser.add_option("--proj", dest="proj",
                       help="Use SSP projections from a fif file.",
                       default=None)
@@ -84,11 +84,11 @@ def run():
                       "in fT (peak to peak amplitude)",
                       default=3000)
     parser.add_option("--rej-eeg", dest="rej_eeg", type="float",
-                      help="EEG rejection parameter in uV "
+                      help="EEG rejection parameter in µV "
                       "(peak to peak amplitude)",
                       default=50)
     parser.add_option("--rej-eog", dest="rej_eog", type="float",
-                      help="EOG rejection parameter in uV "
+                      help="EOG rejection parameter in µV "
                       "(peak to peak amplitude)",
                       default=250)
     parser.add_option("--avg-ref", dest="avg_ref", action="store_true",
@@ -182,7 +182,7 @@ def run():
     else:
         raw_event = raw
 
-    flat = None  # XXX : not exposed to the user
+    flat = None
     projs, events = mne.preprocessing.compute_proj_ecg(
         raw, raw_event, tmin, tmax, n_grad, n_mag, n_eeg, l_freq, h_freq,
         average, filter_length, n_jobs, ch_name, reject, flat, bads, avg_ref,
@@ -208,4 +208,5 @@ def run():
     print("Writing ECG events in %s" % ecg_event_fname)
     mne.write_events(ecg_event_fname, events)
 
+
 mne.utils.run_command_if_main()
