@@ -158,7 +158,7 @@ def test_make_scalp_surfaces(tmp_path, monkeypatch):
     dense_fname = op.join(subj_dir, 'sample-head-dense.fif')
     medium_fname = op.join(subj_dir, 'sample-head-medium.fif')
     with ArgvSetter(cmd, disable_stdout=False, disable_stderr=False):
-        monkeypatch.delenv('FREESURFER_HOME', None)
+        monkeypatch.delenv('FREESURFER_HOME')
         with pytest.raises(RuntimeError, match='The FreeSurfer environ'):
             mne_make_scalp_surfaces.run()
         shutil.copy(op.join(surf_path, 'lh.seghead'), surf_path_new)
@@ -307,8 +307,7 @@ def test_flash_bem(tmp_path):
 
     # Test synthesize flash5 with MEF flash5 and flash30 default locations
     flash5_img = convert_flash_mris(
-        subject="sample", subjects_dir=tempdir, convert=False,
-        unwarp=False
+        subject="sample", subjects_dir=tempdir, unwarp=False
     )
     assert flash5_img == (flash_path / "parameter_maps" / "flash5.mgz")
     assert flash5_img.exists()
