@@ -27,7 +27,7 @@ from mne.fixes import has_numba, _compare_version
 from mne.io import read_raw_fif, read_raw_ctf, read_raw_nirx, read_raw_snirf
 from mne.stats import cluster_level
 from mne.utils import (_pl, _assert_no_instances, numerics, Bunch,
-                       _check_qt_version, _TempDir)
+                       _check_qt_version, _TempDir, check_version)
 
 # data from sample dataset
 from mne.viz._figure import use_browser_backend
@@ -119,6 +119,20 @@ def pytest_configure(config):
     ignore:.*elementwise comparison failed; returning scalar in.*:FutureWarning
     # numba with NumPy dev
     ignore:`np.MachAr` is deprecated.*:DeprecationWarning
+    # matplotlib 3.6 and pyvista/nilearn
+    ignore:.*cmap function will be deprecated.*:
+    # joblib hasn't updated to avoid distutils
+    ignore:.*distutils package is deprecated.*:DeprecationWarning
+    # nbclient
+    ignore:Passing a schema to Validator\.iter_errors is deprecated.*:
+    ignore:Unclosed context <zmq.asyncio.Context.*:ResourceWarning
+    ignore:Jupyter is migrating its paths.*:DeprecationWarning
+    ignore:Widget\..* is deprecated\.:DeprecationWarning
+    # hopefully temporary https://github.com/matplotlib/matplotlib/pull/24455#issuecomment-1319318629
+    ignore:The circles attribute was deprecated in Matplotlib.*:
+    # PySide6
+    ignore:Enum value .* is marked as deprecated:DeprecationWarning
+    ignore:Function.*is marked as deprecated, please check the documentation.*:DeprecationWarning
     """.format(first_kind)  # noqa: E501
     for warning_line in warning_lines.split('\n'):
         warning_line = warning_line.strip()
@@ -422,11 +436,11 @@ def _check_pyqtgraph(request):
             pytest.skip(f'mne_qt_browser {ver} requires PyQt5, API is {api}')
 
 
-@pytest.mark.pgtest
 @pytest.fixture
 def pg_backend(request, garbage_collect):
     """Use for pyqtgraph-specific test-functions."""
     _check_pyqtgraph(request)
+    from mne_qt_browser._pg_figure import MNEQtBrowser
     with use_browser_backend('qt') as backend:
         backend._close_all()
         yield backend
@@ -434,6 +448,9 @@ def pg_backend(request, garbage_collect):
         # This shouldn't be necessary, but let's make sure nothing is stale
         import mne_qt_browser
         mne_qt_browser._browser_instances.clear()
+        if check_version('mne_qt_browser', min_version='0.4'):
+            _assert_no_instances(
+                MNEQtBrowser, f'Closure of {request.node.name}')
 
 
 @pytest.fixture(params=[
@@ -507,6 +524,7 @@ def _check_skip_backend(name):
     from mne.viz.backends.tests._utils import (has_pyvista,
                                                has_imageio_ffmpeg,
                                                has_pyvistaqt)
+    from mne.viz.backends._utils import _notebook_vtk_works
     if name in ('pyvistaqt', 'notebook'):
         if not has_pyvista():
             pytest.skip("Test skipped, requires pyvista.")
@@ -516,16 +534,18 @@ def _check_skip_backend(name):
         pytest.skip("Test skipped, requires Qt.")
     if name == 'pyvistaqt' and not has_pyvistaqt():
         pytest.skip("Test skipped, requires pyvistaqt")
+    if name == 'notebook' and not _notebook_vtk_works():
+        pytest.skip("Test skipped, requires working notebook vtk")
 
 
 @pytest.fixture(scope='session')
 def pixel_ratio():
     """Get the pixel ratio."""
     from mne.viz.backends.tests._utils import has_pyvista
+    # _check_qt_version will init an app for us, so no need for us to do it
     if not has_pyvista() or not _check_qt_version():
         return 1.
-    from qtpy.QtWidgets import QApplication, QMainWindow
-    _ = QApplication.instance() or QApplication([])
+    from qtpy.QtWidgets import QMainWindow
     window = QMainWindow()
     ratio = float(window.devicePixelRatio())
     window.close()
@@ -598,17 +618,26 @@ def _fwd_subvolume(_evoked_cov_sphere):
         evoked.info, fname_trans, src_vol, sphere, mindist=5.0)
 
 
+@pytest.fixture
+def fwd_volume_small(_fwd_subvolume):
+    """Provide a small volumetric source space."""
+    return _fwd_subvolume.copy()
+
+
 @pytest.fixture(scope='session')
 def _all_src_types_fwd(_fwd_surf, _fwd_subvolume):
     """Create all three forward types (surf, vol, mixed)."""
-    fwds = dict(surface=_fwd_surf, volume=_fwd_subvolume)
+    fwds = dict(
+        surface=_fwd_surf.copy(),
+        volume=_fwd_subvolume.copy())
     with pytest.raises(RuntimeError,
                        match='Invalid source space with kinds'):
         fwds['volume']['src'] + fwds['surface']['src']
 
     # mixed (4)
     fwd = fwds['surface'].copy()
-    f2 = fwds['volume']
+    f2 = fwds['volume'].copy()
+    del _fwd_surf, _fwd_subvolume
     for keys, axis in [(('source_rr',), 0),
                        (('source_nn',), 0),
                        (('sol', 'data'), 1),
@@ -759,6 +788,9 @@ def brain_gc(request):
     assert len(bad) == 0, 'VTK objects linger:\n' + '\n'.join(bad)
 
 
+_files = list()
+
+
 def pytest_sessionfinish(session, exitstatus):
     """Handle the end of the session."""
     n = session.config.option.durations
@@ -770,7 +802,6 @@ def pytest_sessionfinish(session, exitstatus):
     except ImportError:
         print('Module-level timings require pytest-harvest')
         return
-    from py.io import TerminalWriter
     # get the number to print
     res = pytest_harvest.get_session_synthesis_dct(session)
     files = dict()
@@ -787,12 +818,17 @@ def pytest_sessionfinish(session, exitstatus):
         files[file_key] = files.get(file_key, 0) + val['pytest_duration_s']
     files = sorted(list(files.items()), key=lambda x: x[1])[::-1]
     # print
-    files = files[:n]
-    if len(files):
-        writer = TerminalWriter()
-        writer.line()  # newline
-        writer.sep('=', f'slowest {n} test module{_pl(n)}')
-        names, timings = zip(*files)
+    _files[:] = files[:n]
+
+
+def pytest_terminal_summary(terminalreporter, exitstatus, config):
+    """Print the module-level timings."""
+    writer = terminalreporter
+    n = len(_files)
+    if n:
+        writer.line('')  # newline
+        writer.write_sep('=', f'slowest {n} test module{_pl(n)}')
+        names, timings = zip(*_files)
         timings = [f'{timing:0.2f}s total' for timing in timings]
         rjust = max(len(timing) for timing in timings)
         timings = [timing.rjust(rjust) for timing in timings]
@@ -859,7 +895,10 @@ def _nbclient():
 }""", as_version=4)
     client = NotebookClient(nb, km=km)
     yield client
-    client._cleanup_kernel()
+    try:
+        client._cleanup_kernel()
+    except Exception:
+        pass
 
 
 @pytest.fixture(scope='function')
@@ -906,5 +945,8 @@ def pytest_runtest_call(item):
 def nirx_snirf(request):
     """Return a (raw_nirx, raw_snirf) matched pair."""
     pytest.importorskip('h5py')
+    skipper = request.param[2].marks[0].mark
+    if skipper.args[0]:  # will skip
+        pytest.skip(skipper.kwargs['reason'])
     return (read_raw_nirx(request.param[0], preload=True),
             read_raw_snirf(request.param[1], preload=True))
