@@ -4,16 +4,17 @@
 # License: BSD Style.
 
 from ...utils import verbose
-from ..utils import (_data_path, _data_path_doc,
+from ..utils import (_data_path_doc, _download_mne_dataset,
                      _get_version, _version_doc)
 
 
 @verbose
 def data_path(path=None, force_update=False, update_path=False,
-              download=True, verbose=None):  # noqa: D103
-    return _data_path(path=path, force_update=force_update,
-                      update_path=update_path, name='fake',
-                      download=download)
+              download=True, *, verbose=None):  # noqa: D103
+    return _download_mne_dataset(
+        name='fake', processor='untar', path=path,
+        force_update=force_update, update_path=update_path,
+        download=download)
 
 
 data_path.__doc__ = _data_path_doc.format(name='fake',
