@@ -1,16 +1,9 @@
 # Authors: Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
-
-from functools import partial
-
+# License: BSD-3-Clause
 from ...utils import verbose
-from ..utils import (has_dataset, _data_path, _get_version, _version_doc,
-                     _data_path_doc_accept)
-
-has_brainstorm_data = partial(has_dataset,
-                              name='brainstorm.bst_phantom_elekta')
-
+from ..utils import (_get_version, _version_doc,
+                     _data_path_doc_accept, _download_mne_dataset)
 
 _description = u"""
 URL: http://neuroimage.usc.edu/brainstorm/Tutorials/PhantomElekta
@@ -18,13 +11,12 @@ URL: http://neuroimage.usc.edu/brainstorm/Tutorials/PhantomElekta
 
 
 @verbose
-def data_path(path=None, force_update=False, update_path=True, download=True,
-              *, accept=False, verbose=None):  # noqa: D103
-    return _data_path(path=path, force_update=force_update,
-                      update_path=update_path, name='brainstorm',
-                      download=download,
-                      archive_name='bst_phantom_elekta.tar.gz',
-                      accept=accept)
+def data_path(path=None, force_update=False, update_path=True,
+              download=True, accept=False, *, verbose=None):  # noqa: D103
+    return _download_mne_dataset(
+        name='bst_phantom_elekta', processor='nested_untar', path=path,
+        force_update=force_update, update_path=update_path,
+        download=download, accept=accept)
 
 
 _data_path_doc = _data_path_doc_accept.format(
@@ -36,7 +28,7 @@ data_path.__doc__ = _data_path_doc
 
 
 def get_version():  # noqa: D103
-    return _get_version('brainstorm.bst_phantom_elekta')
+    return _get_version('bst_phantom_elekta')
 
 
 get_version.__doc__ = _version_doc.format(name='brainstorm')
