@@ -1,16 +1,11 @@
 # Authors: Mainak Jas <mainak.jas@telecom-paristech.fr>
 #
-# License: BSD (3-clause)
-
-from functools import partial
-
+# License: BSD-3-Clause
 from ...utils import verbose
-from ..utils import (has_dataset, _data_path, _get_version, _version_doc,
-                     _data_path_doc_accept)
+from ..utils import (_get_version, _version_doc,
+                     _data_path_doc_accept, _download_mne_dataset)
 
-has_brainstorm_data = partial(has_dataset, name='brainstorm.bst_resting')
-
-_description = u"""
+_description = """
 URL: http://neuroimage.usc.edu/brainstorm/DatasetResting
     - One subject
     - Two runs of 10 min of resting state recordings
@@ -19,12 +14,12 @@ URL: http://neuroimage.usc.edu/brainstorm/DatasetResting
 
 
 @verbose
-def data_path(path=None, force_update=False, update_path=True, download=True,
-              *, accept=False, verbose=None):  # noqa: D103
-    return _data_path(path=path, force_update=force_update,
-                      update_path=update_path, name='brainstorm',
-                      download=download, archive_name='bst_resting.tar.gz',
-                      accept=accept)
+def data_path(path=None, force_update=False, update_path=True,
+              download=True, accept=False, *, verbose=None):  # noqa: D103
+    return _download_mne_dataset(
+        name='bst_resting', processor='nested_untar', path=path,
+        force_update=force_update, update_path=update_path,
+        download=download, accept=accept)
 
 
 _data_path_doc = _data_path_doc_accept.format(
@@ -35,7 +30,7 @@ data_path.__doc__ = _data_path_doc
 
 
 def get_version():  # noqa: D103
-    return _get_version('brainstorm.bst_resting')
+    return _get_version('bst_resting')
 
 
 get_version.__doc__ = _version_doc.format(name='brainstorm')
