@@ -3,22 +3,18 @@
 #          Eric Larson <larson.eric.d@gmail.com>
 # License: BSD Style.
 
-from functools import partial
-
 from ...utils import verbose
-from ..utils import (has_dataset, _data_path, _data_path_doc,
-                     _get_version, _version_doc)
-
-
-has_opm_data = partial(has_dataset, name='opm')
+from ..utils import (_data_path_doc, _get_version, _version_doc,
+                     _download_mne_dataset)
 
 
 @verbose
-def data_path(path=None, force_update=False, update_path=True, download=True,
-              verbose=None):  # noqa: D103
-    return _data_path(path=path, force_update=force_update,
-                      update_path=update_path, name='opm',
-                      download=download)
+def data_path(path=None, force_update=False, update_path=True,
+              download=True, *, verbose=None):  # noqa: D103
+    return _download_mne_dataset(
+        name='opm', processor='untar', path=path,
+        force_update=force_update, update_path=update_path,
+        download=download)
 
 
 data_path.__doc__ = _data_path_doc.format(name='opm',
