@@ -8,8 +8,8 @@ import numpy as np
 import pytest
 
 from numpy.testing import assert_array_equal
+import pooch
 
-import mne
 from mne.utils import requires_good_network
 from mne.utils import requires_pandas, requires_version
 from mne.datasets.sleep_physionet import age, temazepam
@@ -20,9 +20,9 @@ from mne.datasets.sleep_physionet._utils import TEMAZEPAM_SLEEP_RECORDS
 
 
 @pytest.fixture(scope='session')
-def physionet_tmpdir(tmpdir_factory):
+def physionet_tmpdir(tmp_path_factory):
     """Fixture exposing a temporary directory for testing."""
-    return str(tmpdir_factory.mktemp('physionet_files'))
+    return str(tmp_path_factory.mktemp('physionet_files'))
 
 
 class _FakeFetch:
@@ -60,12 +60,13 @@ def _check_mocked_function_calls(mocked_func, call_fname_hash_pairs,
     # Check it has been called with the right parameters in the right
     # order.
     for idx, current in enumerate(call_fname_hash_pairs):
-        call_args, call_kwargs = mocked_func.call_args_list[idx]
-        assert call_args[0] == _get_expected_url(current['name'])
-        assert call_args[1] == _get_expected_path(base_path, current['name'])
-        assert call_kwargs['hash_'] == current['hash']
-        assert call_kwargs['hash_type'] == 'sha1'
-        assert call_kwargs['print_destination'] is False
+        _, call_kwargs = mocked_func.call_args_list[idx]
+        hash_type, hash = call_kwargs['known_hash'].split(':')
+        assert call_kwargs['url'] == _get_expected_url(current['name'])
+        assert op.join(call_kwargs['path'], call_kwargs['fname']) == \
+            _get_expected_path(base_path, current['name'])
+        assert hash == current['hash']
+        assert hash_type == 'sha1'
 
 
 @pytest.mark.timeout(60)
@@ -73,10 +74,10 @@ def _check_mocked_function_calls(mocked_func, call_fname_hash_pairs,
 @requires_good_network
 @requires_pandas
 @requires_version('xlrd', '0.9')
-def test_run_update_age_records(tmpdir):
+def test_run_update_age_records(tmp_path):
     """Test Sleep Physionet URL handling."""
     import pandas as pd
-    fname = op.join(str(tmpdir), "records.csv")
+    fname = op.join(str(tmp_path), "records.csv")
     _update_sleep_age_records(fname)
     data = pd.read_csv(fname)
     pd.testing.assert_frame_equal(data, pd.read_csv(AGE_SLEEP_RECORDS))
@@ -86,18 +87,19 @@ def test_run_update_age_records(tmpdir):
 def test_sleep_physionet_age_missing_subjects(physionet_tmpdir, subject,
                                               download_is_error):
     """Test handling of missing subjects in Sleep Physionet age fetcher."""
-    params = {'path': physionet_tmpdir, 'update_path': False}
-
     with pytest.raises(
             ValueError, match='This dataset contains subjects 0 to 82'):
         age.fetch_data(
-            subjects=[subject], recording=[1], on_missing='raise', **params)
+            subjects=[subject], recording=[1], on_missing='raise',
+            path=physionet_tmpdir)
     with pytest.warns(RuntimeWarning,
                       match='This dataset contains subjects 0 to 82'):
         age.fetch_data(
-            subjects=[subject], recording=[1], on_missing='warn', **params)
+            subjects=[subject], recording=[1], on_missing='warn',
+            path=physionet_tmpdir)
     paths = age.fetch_data(
-        subjects=[subject], recording=[1], on_missing='ignore', **params)
+        subjects=[subject], recording=[1], on_missing='ignore',
+        path=physionet_tmpdir)
     assert paths == []
 
 
@@ -105,42 +107,40 @@ def test_sleep_physionet_age_missing_subjects(physionet_tmpdir, subject,
 def test_sleep_physionet_age_missing_recordings(physionet_tmpdir, subject,
                                                 recording, download_is_error):
     """Test handling of missing recordings in Sleep Physionet age fetcher."""
-    params = {'path': physionet_tmpdir, 'update_path': False}
-
     with pytest.raises(
             ValueError, match=f'Requested recording {recording} for subject'):
         age.fetch_data(subjects=[subject], recording=[recording],
-                       on_missing='raise', **params)
+                       on_missing='raise', path=physionet_tmpdir)
     with pytest.warns(RuntimeWarning,
                       match=f'Requested recording {recording} for subject'):
         age.fetch_data(subjects=[subject], recording=[recording],
-                       on_missing='warn', **params)
+                       on_missing='warn', path=physionet_tmpdir)
     paths = age.fetch_data(subjects=[subject], recording=[recording],
-                           on_missing='ignore', **params)
+                           on_missing='ignore', path=physionet_tmpdir)
     assert paths == []
 
 
 def test_sleep_physionet_age(physionet_tmpdir, monkeypatch, download_is_error):
     """Test Sleep Physionet URL handling."""
     # check download_is_error patching
-    params = {'path': physionet_tmpdir, 'update_path': False}
     with pytest.raises(AssertionError, match='Test should not download'):
-        age.fetch_data(subjects=[0], recording=[1], **params)
+        age.fetch_data(subjects=[0], recording=[1], path=physionet_tmpdir)
     # then patch
     my_func = _FakeFetch()
-    monkeypatch.setattr(
-        mne.datasets.sleep_physionet._utils, '_fetch_file', my_func)
+    monkeypatch.setattr(pooch, 'retrieve', my_func)
 
-    paths = age.fetch_data(subjects=[0], recording=[1], **params)
+    paths = age.fetch_data(subjects=[0], recording=[1], path=physionet_tmpdir)
     assert_array_equal(_keep_basename_only(paths),
                        [['SC4001E0-PSG.edf', 'SC4001EC-Hypnogram.edf']])
 
-    paths = age.fetch_data(subjects=[0, 1], recording=[1], **params)
+    paths = age.fetch_data(subjects=[0, 1], recording=[1],
+                           path=physionet_tmpdir)
     assert_array_equal(_keep_basename_only(paths),
                        [['SC4001E0-PSG.edf', 'SC4001EC-Hypnogram.edf'],
                         ['SC4011E0-PSG.edf', 'SC4011EH-Hypnogram.edf']])
 
-    paths = age.fetch_data(subjects=[0], recording=[1, 2], **params)
+    paths = age.fetch_data(subjects=[0], recording=[1, 2],
+                           path=physionet_tmpdir)
     assert_array_equal(_keep_basename_only(paths),
                        [['SC4001E0-PSG.edf', 'SC4001EC-Hypnogram.edf'],
                         ['SC4002E0-PSG.edf', 'SC4002EC-Hypnogram.edf']])
@@ -174,10 +174,10 @@ def test_sleep_physionet_age(physionet_tmpdir, monkeypatch, download_is_error):
 @requires_good_network
 @requires_pandas
 @requires_version('xlrd', '0.9')
-def test_run_update_temazepam_records(tmpdir):
+def test_run_update_temazepam_records(tmp_path):
     """Test Sleep Physionet URL handling."""
     import pandas as pd
-    fname = op.join(str(tmpdir), "records.csv")
+    fname = op.join(str(tmp_path), "records.csv")
     _update_sleep_temazepam_records(fname)
     data = pd.read_csv(fname)
 
@@ -188,12 +188,9 @@ def test_run_update_temazepam_records(tmpdir):
 def test_sleep_physionet_temazepam(physionet_tmpdir, monkeypatch):
     """Test Sleep Physionet URL handling."""
     my_func = _FakeFetch()
-    monkeypatch.setattr(
-        mne.datasets.sleep_physionet._utils, '_fetch_file', my_func)
+    monkeypatch.setattr(pooch, 'retrieve', my_func)
 
-    params = {'path': physionet_tmpdir, 'update_path': False}
-
-    paths = temazepam.fetch_data(subjects=[0], **params)
+    paths = temazepam.fetch_data(subjects=[0], path=physionet_tmpdir)
     assert_array_equal(_keep_basename_only(paths),
                        [['ST7011J0-PSG.edf', 'ST7011JP-Hypnogram.edf']])
 
@@ -207,4 +204,4 @@ def test_sleep_physionet_temazepam(physionet_tmpdir, monkeypatch):
 
     with pytest.raises(
             ValueError, match='This dataset contains subjects 0 to 21'):
-        paths = temazepam.fetch_data(subjects=[22], **params)
+        paths = temazepam.fetch_data(subjects=[22], path=physionet_tmpdir)
