@@ -5,19 +5,20 @@
 from functools import partial
 
 from ...utils import verbose, get_config
-from ..utils import (has_dataset, _data_path, _data_path_doc,
-                     _get_version, _version_doc)
+from ..utils import (has_dataset, _data_path_doc, _get_version,
+                     _version_doc, _download_mne_dataset)
 
 
 has_spm_data = partial(has_dataset, name='spm')
 
 
 @verbose
-def data_path(path=None, force_update=False, update_path=True, download=True,
-              verbose=None):  # noqa: D103
-    return _data_path(path=path, force_update=force_update,
-                      update_path=update_path, name='spm',
-                      download=download)
+def data_path(path=None, force_update=False, update_path=True,
+              download=True, *, verbose=None):  # noqa: D103
+    return _download_mne_dataset(
+        name='spm', processor='untar', path=path,
+        force_update=force_update, update_path=update_path,
+        download=download)
 
 
 data_path.__doc__ = _data_path_doc.format(name='spm',
