@@ -14,7 +14,7 @@ from mne.datasets import (testing, fetch_infant_template, fetch_phantom,
 from mne.datasets._fsaverage.base import _set_montage_coreg_path
 from mne.datasets._infant import base as infant_base
 from mne.datasets._phantom import base as phantom_base
-from mne.datasets.utils import _manifest_check_download, _mne_path
+from mne.datasets.utils import _manifest_check_download
 
 from mne.utils import (requires_good_network,
                        get_subjects_dir, ArgvSetter, _pl, use_log_level,
@@ -74,6 +74,11 @@ def test_downloads(tmp_path, monkeypatch, capsys):
     # that we're hitting (https://github.com/pytest-dev/pytest/issues/5997)
     # now that we use pooch
     with capsys.disabled():
+        with pytest.raises(RuntimeError, match='Do not download .* in tests'):
+            path = datasets._fake.data_path(update_path=False, **kwargs)
+        monkeypatch.setattr(
+            datasets.utils,
+            '_MODULES_TO_ENSURE_DOWNLOAD_IS_FALSE_IN_TESTS', ())
         path = datasets._fake.data_path(update_path=False, **kwargs)
     assert op.isdir(path)
     assert op.isfile(op.join(path, 'bar'))
@@ -266,15 +271,3 @@ def test_fetch_uncompressed_file(tmp_path):
         hash=None)
     fetch_dataset(dataset_dict, path=None, force_update=True)
     assert (tmp_path / 'foo' / 'LICENSE.foo').is_file()
-
-
-def test_mne_path():
-    """Test our Path wrapping."""
-    path = _mne_path("")
-    assert str(path) == '.'
-    with pytest.deprecated_call(match=r'pathlib\.Path object'):
-        assert path + 'me' == f'.{op.sep}me'
-    assert str(path / 'me') == 'me'
-    assert str('me' / path) == 'me'
-    with pytest.raises(TypeError, match='can only concatenate str'):
-        'me' + path  # our paths were absolute, so this should raise an error
