@@ -9,9 +9,11 @@
 # License: BSD Style.
 
 from collections import OrderedDict
+import importlib
+import inspect
 import os
 import os.path as op
-from pathlib import WindowsPath, PosixPath
+from pathlib import Path
 import sys
 import zipfile
 import tempfile
@@ -20,7 +22,7 @@ import numpy as np
 
 from .config import _hcp_mmp_license_text, MNE_DATASETS
 from ..label import read_labels_from_annot, Label, write_labels_to_annot
-from ..utils import (get_config, set_config, logger, _validate_type, warn,
+from ..utils import (get_config, set_config, logger, _validate_type,
                      verbose, get_subjects_dir, _pl, _safe_input)
 from ..utils.docs import docdict, _docformat
 
@@ -101,7 +103,7 @@ def _get_path(path, key, name):
                    f"not exist. Either create this directory manually and try "
                    f"again, or set MNE_DATA to an existing directory.")
             raise FileNotFoundError(msg)
-        return _mne_path(path)
+        return Path(path)
     # 4. ~/mne_data (but use a fake home during testing so we don't
     #    unnecessarily create ~/mne_data)
     logger.info('Using default location ~/mne_data for %s...' % name)
@@ -117,7 +119,7 @@ def _get_path(path, key, name):
                           "argument to data_path() where user has "
                           "write permissions, for ex:data_path"
                           "('/home/xyz/me2/')" % (path))
-    return _mne_path(path)
+    return Path(path)
 
 
 def _do_path_update(path, update_path, key, name):
@@ -142,12 +144,59 @@ def _do_path_update(path, update_path, key, name):
     return path
 
 
+# This is meant to be semi-public: let packages like mne-bids use it to make
+# sure they don't accidentally set download=True in their tests, too
+_MODULES_TO_ENSURE_DOWNLOAD_IS_FALSE_IN_TESTS = ('mne',)
+
+
+def _check_in_testing_and_raise(name, download):
+    """Check if we're in an MNE test and raise an error if download!=False."""
+    root_dirs = [
+        importlib.import_module(ns)
+        for ns in _MODULES_TO_ENSURE_DOWNLOAD_IS_FALSE_IN_TESTS]
+    root_dirs = [str(Path(ns.__file__).parent) for ns in root_dirs]
+    check = False
+    func = None
+    frame = inspect.currentframe()
+    try:
+        # First, traverse out of the data_path() call
+        while frame:
+            if frame.f_code.co_name in ('data_path', 'load_data'):
+                func = frame.f_code.co_name
+                frame = frame.f_back.f_back  # out of verbose decorator
+                break
+            frame = frame.f_back
+        # Next, see what the caller was
+        while frame:
+            fname = frame.f_code.co_filename
+            if fname is not None:
+                fname = Path(fname)
+                # in mne namespace, and
+                # (can't use is_relative_to here until 3.9)
+                if any(str(fname).startswith(rd) for rd in root_dirs) and (
+                        # in tests/*.py
+                        fname.parent.stem == 'tests' or
+                        # or in a conftest.py
+                        fname.stem == 'conftest.py'):
+                    check = True
+                    break
+            frame = frame.f_back
+    finally:
+        del frame
+    if check and download is not False:
+        raise RuntimeError(
+            f'Do not download dataset {repr(name)} in tests, pass '
+            f'{func}(download=False) to prevent accidental downloads')
+
+
 def _download_mne_dataset(name, processor, path, force_update,
                           update_path, download, accept=False):
     """Aux function for downloading internal MNE datasets."""
     import pooch
     from mne.datasets._fetch import fetch_dataset
 
+    _check_in_testing_and_raise(name, download)
+
     # import pooch library for handling the dataset downloading
     dataset_params = MNE_DATASETS[name]
     dataset_params['dataset_name'] = name
@@ -524,33 +573,3 @@ def _manifest_check_download(manifest_path, destination, url, hash_):
                     ff.extract(name, path=destination)
         logger.info('Successfully extracted %d file%s'
                     % (len(need), _pl(need)))
-
-
-# Adapted from pathlib.Path.__new__
-def _mne_path(path):
-    klass = MNEWindowsPath if os.name == 'nt' else MNEPosixPath
-    out = klass._from_parts((path,))
-    if not out._flavour.is_supported:
-        raise NotImplementedError("cannot instantiate %r on your system"
-                                  % (klass.__name__,))
-    return out
-
-
-class _PathAdd:
-
-    def __add__(self, other):
-        if isinstance(other, str):
-            warn('data_path functions now return pathlib.Path objects which '
-                 'do not natively support the plus (+) operator, switch to '
-                 'using forward slash (/) instead. Support for plus will be '
-                 'removed in 1.2.', DeprecationWarning)
-            return f'{str(self)}{op.sep}{other}'
-        raise NotImplementedError
-
-
-class MNEWindowsPath(_PathAdd, WindowsPath):  # noqa: D101
-    pass
-
-
-class MNEPosixPath(_PathAdd, PosixPath):  # noqa: D101
-    pass
