@@ -5,7 +5,7 @@
 #          Clemens Brunner <clemens.brunner@gmail.com>
 #          Jean-Remi King <jeanremi.king@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import copy as cp
 
@@ -14,8 +14,16 @@ import numpy as np
 from .base import BaseEstimator
 from .mixin import TransformerMixin
 from ..cov import _regularized_covariance
+from ..defaults import (_BORDER_DEFAULT, _EXTRAPOLATE_DEFAULT,
+                        _INTERPOLATION_DEFAULT)
 from ..fixes import pinv
-from ..utils import fill_doc, _check_option, _validate_type
+from ..utils import fill_doc, _check_option, _validate_type, copy_doc, warn
+from ..viz.utils import _warn_deprecated_vmin_vmax
+
+# TODO ↓↓↓↓↓ remove after 1.3 release
+_TITLE_WARNING_MSG = (
+    'The "title" parameter is deprecated and will be removed in version 1.4. '
+    'Use "fig.suptitle()" instead.')
 
 
 @fill_doc
@@ -62,7 +70,7 @@ class CSP(TransformerMixin, BaseEstimator):
         Parameters to pass to :func:`mne.compute_covariance`.
 
         .. versionadded:: 0.16
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.17
     component_order : 'mutual_info' | 'alternate' (default 'mutual_info')
@@ -231,15 +239,21 @@ class CSP(TransformerMixin, BaseEstimator):
                 X /= self.std_
         return X
 
+    @copy_doc(TransformerMixin.fit_transform)
+    def fit_transform(self, X, y, **fit_params):  # noqa: D102
+        return super().fit_transform(X, y=y, **fit_params)
+
     @fill_doc
-    def plot_patterns(self, info, components=None, ch_type=None,
-                      vmin=None, vmax=None, cmap='RdBu_r', sensors=True,
-                      colorbar=True, scalings=None, units='a.u.', res=64,
-                      size=1, cbar_fmt='%3.1f', name_format='CSP%01d',
-                      show=True, show_names=False, title=None, mask=None,
-                      mask_params=None, outlines='head', contours=6,
-                      image_interp='bilinear', average=None,
-                      sphere=None):
+    def plot_patterns(
+            self, info, components=None, *, average=None, ch_type=None,
+            scalings=None, sensors=True, show_names=False, mask=None,
+            mask_params=None, contours=6, outlines='head', sphere=None,
+            image_interp=_INTERPOLATION_DEFAULT,
+            extrapolate=_EXTRAPOLATE_DEFAULT, border=_BORDER_DEFAULT, res=64,
+            size=1, cmap='RdBu_r', vlim=(None, None), vmin=None, vmax=None,
+            cnorm=None, colorbar=True, cbar_fmt='%3.1f', units=None,
+            axes=None, name_format='CSP%01d', title=None, nrows=1,
+            ncols='auto', show=True):
         """Plot topographic patterns of components.
 
         The patterns explain how the measured data was generated from the
@@ -247,93 +261,59 @@ class CSP(TransformerMixin, BaseEstimator):
 
         Parameters
         ----------
-        info : instance of Info
-            Info dictionary of the epochs used for fitting.
-            If not possible, consider using ``create_info``.
+        %(info_not_none)s Used for fitting. If not available, consider using
+            :func:`mne.create_info`.
         components : float | array of float | None
-           The patterns to plot. If None, n_components will be shown.
-        ch_type : 'mag' | 'grad' | 'planar1' | 'planar2' | 'eeg' | None
-            The channel type to plot. For 'grad', the gradiometers are
-            collected in pairs and the RMS for each pair is plotted.
-            If None, then first available channel type from order given
-            above is used. Defaults to None.
-        vmin : float | callable
-            The value specifying the lower bound of the color range.
-            If None, and vmax is None, -vmax is used. Else np.min(data).
-            If callable, the output equals vmin(data).
-        vmax : float | callable
-            The value specifying the upper bound of the color range.
-            If None, the maximum absolute value is used. If vmin is None,
-            but vmax is not, default np.min(data).
-            If callable, the output equals vmax(data).
-        cmap : matplotlib colormap | (colormap, bool) | 'interactive' | None
-            Colormap to use. If tuple, the first value indicates the colormap
-            to use and the second value is a boolean defining interactivity. In
-            interactive mode the colors are adjustable by clicking and dragging
-            the colorbar with left and right mouse button. Left mouse button
-            moves the scale up and down and right mouse button adjusts the
-            range. Hitting space bar resets the range. Up and down arrows can
-            be used to change the colormap. If None, 'Reds' is used for all
-            positive data, otherwise defaults to 'RdBu_r'. If 'interactive',
-            translates to (None, True). Defaults to 'RdBu_r'.
-
-            .. warning::  Interactive mode works smoothly only for a small
-                amount of topomaps.
-        sensors : bool | str
-            Add markers for sensor locations to the plot. Accepts matplotlib
-            plot format string (e.g., 'r+' for red plusses). If True,
-            a circle will be used (via .add_artist). Defaults to True.
-        colorbar : bool
-            Plot a colorbar.
+           The patterns to plot. If ``None``, all components will be shown.
+        %(average_plot_evoked_topomap)s
+        %(ch_type_topomap)s
         scalings : dict | float | None
             The scalings of the channel types to be applied for plotting.
             If None, defaults to ``dict(eeg=1e6, grad=1e13, mag=1e15)``.
-        units : dict | str | None
-            The unit of the channel type used for colorbar label. If
-            scale is None the unit is automatically determined.
-        res : int
-            The resolution of the topomap image (n pixels along each side).
-        size : float
-            Side length per topomap in inches.
-        cbar_fmt : str
-            String format for colorbar values.
+        %(sensors_topomap)s
+        %(show_names_topomap)s
+        %(mask_patterns_topomap)s
+        %(mask_params_topomap)s
+        %(contours_topomap)s
+        %(outlines_topomap)s
+        %(sphere_topomap_auto)s
+        %(image_interp_topomap)s
+        %(extrapolate_topomap)s
+
+            .. versionadded:: 1.3
+        %(border_topomap)s
+
+            .. versionadded:: 1.3
+        %(res_topomap)s
+        %(size_topomap)s
+        %(cmap_topomap)s
+        %(vlim_plot_topomap)s
+
+            .. versionadded:: 1.3
+        %(vmin_vmax_topomap)s
+
+            .. deprecated:: v1.4
+               The ``vmin`` and ``vmax`` parameters will be removed in version
+               1.4. Please use the ``vlim`` parameter instead.
+        %(cnorm)s
+
+            .. versionadded:: 1.3
+        %(colorbar_topomap)s
+        %(cbar_fmt_topomap)s
+        %(units_topomap)s
+        %(axes_evoked_plot_topomap)s
         name_format : str
             String format for topomap values. Defaults to "CSP%%01d".
-        show : bool
-            Show figure if True.
-        show_names : bool | callable
-            If True, show channel names on top of the map. If a callable is
-            passed, channel names will be formatted using the callable; e.g.,
-            to delete the prefix 'MEG ' from all channel names, pass the
-            function lambda x: x.replace('MEG ', ''). If ``mask`` is not None,
-            only significant sensors will be shown.
-        title : str | None
-            Title. If None (default), no title is displayed.
-        mask : ndarray of bool, shape (n_channels, n_times) | None
-            The channels to be marked as significant at a given time point.
-            Indices set to `True` will be considered. Defaults to None.
-        mask_params : dict | None
-            Additional plotting parameters for plotting significant sensors.
-            Default (None) equals::
+        %(title_none)s
 
-                dict(marker='o', markerfacecolor='w', markeredgecolor='k',
-                     linewidth=0, markersize=4)
-        %(topomap_outlines)s
-        contours : int | array of float
-            The number of contour lines to draw. If 0, no contours will be
-            drawn. When an integer, matplotlib ticker locator is used to find
-            suitable values for the contour thresholds (may sometimes be
-            inaccurate, use array for accuracy). If an array, the values
-            represent the levels for the contours. Defaults to 6.
-        image_interp : str
-            The image interpolation to be used.
-            All matplotlib options are accepted.
-        average : float | None
-            The time window around a given time to be used for averaging
-            (seconds). For example, 0.01 would translate into window that
-            starts 5 ms before and ends 5 ms after a given time point.
-            Defaults to None, which means no averaging.
-        %(topomap_sphere_auto)s
+            .. deprecated:: v1.4
+               The ``title`` parameter will be removed in version 1.4. Please
+               use :meth:`fig.suptitle()<matplotlib.figure.Figure.suptitle>`
+               instead.
+        %(nrows_ncols_topomap)s
+
+            .. versionadded:: 1.3
+        %(show)s
 
         Returns
         -------
@@ -341,33 +321,47 @@ class CSP(TransformerMixin, BaseEstimator):
            The figure.
         """
         from .. import EvokedArray
+
+        vlim = _warn_deprecated_vmin_vmax(vlim, vmin, vmax, '1.4')
+
+        if units is None:
+            units = 'AU'
         if components is None:
             components = np.arange(self.n_components)
 
         # set sampling frequency to have 1 component per time point
         info = cp.deepcopy(info)
-        info['sfreq'] = 1.
+        with info._unlock():
+            info['sfreq'] = 1.
         # create an evoked
         patterns = EvokedArray(self.patterns_.T, info, tmin=0)
         # the call plot_topomap
-        return patterns.plot_topomap(
-            times=components, ch_type=ch_type,
-            vmin=vmin, vmax=vmax, cmap=cmap, colorbar=colorbar, res=res,
-            cbar_fmt=cbar_fmt, sensors=sensors,
-            scalings=scalings, units=units, time_unit='s',
-            time_format=name_format, size=size, show_names=show_names,
-            title=title, mask_params=mask_params, mask=mask, outlines=outlines,
-            contours=contours, image_interp=image_interp, show=show,
-            average=average, sphere=sphere)
+        fig = patterns.plot_topomap(
+            times=components, average=average, ch_type=ch_type,
+            scalings=scalings, sensors=sensors, show_names=show_names,
+            mask=mask, mask_params=mask_params, contours=contours,
+            outlines=outlines, sphere=sphere, image_interp=image_interp,
+            extrapolate=extrapolate, border=border, res=res, size=size,
+            cmap=cmap, vlim=vlim, cnorm=cnorm, colorbar=colorbar,
+            cbar_fmt=cbar_fmt, units=units, axes=axes, time_format=name_format,
+            nrows=nrows, ncols=ncols, show=show)
+
+        if title is not None:
+            warn(_TITLE_WARNING_MSG, FutureWarning)
+            fig.suptitle(title)
+        return fig
 
     @fill_doc
-    def plot_filters(self, info, components=None, ch_type=None,
-                     vmin=None, vmax=None, cmap='RdBu_r', sensors=True,
-                     colorbar=True, scalings=None, units='a.u.', res=64,
-                     size=1, cbar_fmt='%3.1f', name_format='CSP%01d',
-                     show=True, show_names=False, title=None, mask=None,
-                     mask_params=None, outlines='head', contours=6,
-                     image_interp='bilinear', average=None):
+    def plot_filters(
+            self, info, components=None, *, average=None, ch_type=None,
+            scalings=None, sensors=True, show_names=False, mask=None,
+            mask_params=None, contours=6, outlines='head', sphere=None,
+            image_interp=_INTERPOLATION_DEFAULT,
+            extrapolate=_EXTRAPOLATE_DEFAULT, border=_BORDER_DEFAULT, res=64,
+            size=1, cmap='RdBu_r', vlim=(None, None), vmin=None, vmax=None,
+            cnorm=None, colorbar=True, cbar_fmt='%3.1f', units=None,
+            axes=None, name_format='CSP%01d', title=None, nrows=1,
+            ncols='auto', show=True):
         """Plot topographic filters of components.
 
         The filters are used to extract discriminant neural sources from
@@ -375,92 +369,59 @@ class CSP(TransformerMixin, BaseEstimator):
 
         Parameters
         ----------
-        info : instance of Info
-            Info dictionary of the epochs used for fitting.
-            If not possible, consider using ``create_info``.
+        %(info_not_none)s Used for fitting. If not available, consider using
+            :func:`mne.create_info`.
         components : float | array of float | None
-           The patterns to plot. If None, n_components will be shown.
-        ch_type : 'mag' | 'grad' | 'planar1' | 'planar2' | 'eeg' | None
-            The channel type to plot. For 'grad', the gradiometers are
-            collected in pairs and the RMS for each pair is plotted.
-            If None, then first available channel type from order given
-            above is used. Defaults to None.
-        vmin : float | callable
-            The value specifying the lower bound of the color range.
-            If None, and vmax is None, -vmax is used. Else np.min(data).
-            If callable, the output equals vmin(data).
-        vmax : float | callable
-            The value specifying the upper bound of the color range.
-            If None, the maximum absolute value is used. If vmin is None,
-            but vmax is not, defaults to np.min(data).
-            If callable, the output equals vmax(data).
-        cmap : matplotlib colormap | (colormap, bool) | 'interactive' | None
-            Colormap to use. If tuple, the first value indicates the colormap
-            to use and the second value is a boolean defining interactivity. In
-            interactive mode the colors are adjustable by clicking and dragging
-            the colorbar with left and right mouse button. Left mouse button
-            moves the scale up and down and right mouse button adjusts the
-            range. Hitting space bar resets the range. Up and down arrows can
-            be used to change the colormap. If None, 'Reds' is used for all
-            positive data, otherwise defaults to 'RdBu_r'. If 'interactive',
-            translates to (None, True). Defaults to 'RdBu_r'.
-
-            .. warning::  Interactive mode works smoothly only for a small
-                amount of topomaps.
-        sensors : bool | str
-            Add markers for sensor locations to the plot. Accepts matplotlib
-            plot format string (e.g., 'r+' for red plusses). If True,
-            a circle will be used (via .add_artist). Defaults to True.
-        colorbar : bool
-            Plot a colorbar.
+           The patterns to plot. If ``None``, all components will be shown.
+        %(average_plot_evoked_topomap)s
+        %(ch_type_topomap)s
         scalings : dict | float | None
             The scalings of the channel types to be applied for plotting.
             If None, defaults to ``dict(eeg=1e6, grad=1e13, mag=1e15)``.
-        units : dict | str | None
-            The unit of the channel type used for colorbar label. If
-            scale is None the unit is automatically determined.
-        res : int
-            The resolution of the topomap image (n pixels along each side).
-        size : float
-            Side length per topomap in inches.
-        cbar_fmt : str
-            String format for colorbar values.
+        %(sensors_topomap)s
+        %(show_names_topomap)s
+        %(mask_patterns_topomap)s
+        %(mask_params_topomap)s
+        %(contours_topomap)s
+        %(outlines_topomap)s
+        %(sphere_topomap_auto)s
+        %(image_interp_topomap)s
+        %(extrapolate_topomap)s
+
+            .. versionadded:: 1.3
+        %(border_topomap)s
+
+            .. versionadded:: 1.3
+        %(res_topomap)s
+        %(size_topomap)s
+        %(cmap_topomap)s
+        %(vlim_plot_topomap_psd)s
+
+            .. versionadded:: 1.3
+        %(vmin_vmax_topomap)s
+
+            .. deprecated:: v1.4
+               The ``vmin`` and ``vmax`` parameters will be removed in version
+               1.4. Please use the ``vlim`` parameter instead.
+        %(cnorm)s
+
+            .. versionadded:: 1.3
+        %(colorbar_topomap)s
+        %(cbar_fmt_topomap)s
+        %(units_topomap)s
+        %(axes_evoked_plot_topomap)s
         name_format : str
             String format for topomap values. Defaults to "CSP%%01d".
-        show : bool
-            Show figure if True.
-        show_names : bool | callable
-            If True, show channel names on top of the map. If a callable is
-            passed, channel names will be formatted using the callable; e.g.,
-            to delete the prefix 'MEG ' from all channel names, pass the
-            function lambda x: x.replace('MEG ', ''). If ``mask`` is not None,
-            only significant sensors will be shown.
-        title : str | None
-            Title. If None (default), no title is displayed.
-        mask : ndarray of bool, shape (n_channels, n_times) | None
-            The channels to be marked as significant at a given time point.
-            Indices set to `True` will be considered. Defaults to None.
-        mask_params : dict | None
-            Additional plotting parameters for plotting significant sensors.
-            Default (None) equals::
+        %(title_none)s
 
-                dict(marker='o', markerfacecolor='w', markeredgecolor='k',
-                     linewidth=0, markersize=4)
-        %(topomap_outlines)s
-        contours : int | array of float
-            The number of contour lines to draw. If 0, no contours will be
-            drawn. When an integer, matplotlib ticker locator is used to find
-            suitable values for the contour thresholds (may sometimes be
-            inaccurate, use array for accuracy). If an array, the values
-            represent the levels for the contours. Defaults to 6.
-        image_interp : str
-            The image interpolation to be used.
-            All matplotlib options are accepted.
-        average : float | None
-            The time window around a given time to be used for averaging
-            (seconds). For example, 0.01 would translate into window that
-            starts 5 ms before and ends 5 ms after a given time point.
-            Defaults to None, which means no averaging.
+            .. deprecated:: v1.4
+               The ``title`` parameter will be removed in version 1.4. Please
+               use :meth:`fig.suptitle()<matplotlib.figure.Figure.suptitle>`
+               instead.
+        %(nrows_ncols_topomap)s
+
+            .. versionadded:: 1.3
+        %(show)s
 
         Returns
         -------
@@ -468,23 +429,34 @@ class CSP(TransformerMixin, BaseEstimator):
            The figure.
         """
         from .. import EvokedArray
+
+        vlim = _warn_deprecated_vmin_vmax(vlim, vmin, vmax, '1.4')
+
+        if units is None:
+            units = 'AU'
         if components is None:
             components = np.arange(self.n_components)
 
         # set sampling frequency to have 1 component per time point
         info = cp.deepcopy(info)
-        info['sfreq'] = 1.
+        with info._unlock():
+            info['sfreq'] = 1.
         # create an evoked
         filters = EvokedArray(self.filters_.T, info, tmin=0)
         # the call plot_topomap
-        return filters.plot_topomap(
-            times=components, ch_type=ch_type, vmin=vmin,
-            vmax=vmax, cmap=cmap, colorbar=colorbar, res=res,
-            cbar_fmt=cbar_fmt, sensors=sensors, scalings=scalings, units=units,
-            time_unit='s', time_format=name_format, size=size,
-            show_names=show_names, title=title, mask_params=mask_params,
-            mask=mask, outlines=outlines, contours=contours,
-            image_interp=image_interp, show=show, average=average)
+        fig = filters.plot_topomap(
+            times=components, average=average, ch_type=ch_type,
+            scalings=scalings, sensors=sensors, show_names=show_names,
+            mask=mask, mask_params=mask_params, contours=contours,
+            outlines=outlines, sphere=sphere, image_interp=image_interp,
+            extrapolate=extrapolate, border=border, res=res, size=size,
+            cmap=cmap, vlim=vlim, cnorm=cnorm, colorbar=colorbar,
+            cbar_fmt=cbar_fmt, units=units, axes=axes, time_format=name_format,
+            nrows=nrows, ncols=ncols, show=show)
+        if title is not None:
+            warn(_TITLE_WARNING_MSG, FutureWarning)
+            fig.suptitle(title)
+        return fig
 
     def _compute_covariance_matrices(self, X, y):
         _, n_channels, _ = X.shape
@@ -709,7 +681,7 @@ class SPoC(CSP):
         Parameters to pass to :func:`mne.compute_covariance`.
 
         .. versionadded:: 0.16
-    %(rank_None)s
+    %(rank_none)s
 
         .. versionadded:: 0.17
 
