@@ -4,6 +4,7 @@
 #
 # License: BSD-3-Clause
 
+from contextlib import nullcontext
 from datetime import datetime, timezone
 from mne.io import RawArray
 from mne.io.meas_info import create_info
@@ -19,11 +20,12 @@ from mne import (read_epochs_eeglab, Epochs, read_evokeds, read_evokeds_mff,
                  Annotations)
 from mne.datasets import testing, misc
 from mne.export import export_evokeds, export_evokeds_mff
+from mne.fixes import _compare_version
 from mne.io import (read_raw_fif, read_raw_eeglab, read_raw_edf,
                     read_raw_brainvision)
 from mne.utils import (_check_eeglabio_installed, requires_version,
                        object_diff, _check_edflib_installed, _resource_path,
-                       _check_pybv_installed)
+                       _check_pybv_installed, _record_warnings)
 from mne.tests.test_epochs import _get_data
 
 fname_evoked = _resource_path('mne.io.tests.data', 'test-ave.fif')
@@ -86,8 +88,11 @@ def test_export_raw_eeglab(tmp_path):
     raw.export(temp_fname)
     raw.drop_channels([ch for ch in ['epoc']
                        if ch in raw.ch_names])
-    raw_read = read_raw_eeglab(temp_fname, preload=True)
+
+    with pytest.warns(RuntimeWarning, match='is above the 99th percentile'):
+        raw_read = read_raw_eeglab(temp_fname, preload=True, montage_units='m')
     assert raw.ch_names == raw_read.ch_names
+
     cart_coords = np.array([d['loc'][:3] for d in raw.info['chs']])  # just xyz
     cart_coords_read = np.array([d['loc'][:3] for d in raw_read.info['chs']])
     assert_allclose(cart_coords, cart_coords_read)
@@ -151,6 +156,11 @@ def test_double_export_edf(tmp_path):
     read_ch_types = raw_read.get_channel_types()
     assert_array_equal(orig_ch_types, read_ch_types)
 
+    # check handling of missing subject metadata
+    del info['subject_info']['sex']
+    raw_2 = RawArray(data, info)
+    raw_2.export(temp_fname, add_ch_type=True, overwrite=True)
+
 
 @pytest.mark.skipif(not _check_edflib_installed(strict=False),
                     reason='edflib-python not installed')
@@ -340,17 +350,25 @@ def test_export_raw_edf(tmp_path, dataset, format):
         raw.times, raw_read.times[:orig_raw_len], rtol=0, atol=1e-5)
 
 
+@pytest.mark.xfail(reason='eeglabio (usage?) bugs that should be fixed')
 @requires_version('pymatreader')
 @pytest.mark.skipif(not _check_eeglabio_installed(strict=False),
                     reason='eeglabio not installed')
 @pytest.mark.parametrize('preload', (True, False))
 def test_export_epochs_eeglab(tmp_path, preload):
     """Test saving an Epochs instance to EEGLAB's set format."""
+    import eeglabio
     raw, events = _get_data()[:2]
     raw.load_data()
     epochs = Epochs(raw, events, preload=preload)
     temp_fname = op.join(str(tmp_path), 'test.set')
-    epochs.export(temp_fname)
+    # TODO: eeglabio 0.2 warns about invalid events
+    if _compare_version(eeglabio.__version__, '==', '0.0.2-1'):
+        ctx = _record_warnings
+    else:
+        ctx = nullcontext
+    with ctx():
+        epochs.export(temp_fname)
     epochs.drop_channels([ch for ch in ['epoc', 'STI 014']
                           if ch in epochs.ch_names])
     epochs_read = read_epochs_eeglab(temp_fname)
@@ -369,10 +387,12 @@ def test_export_epochs_eeglab(tmp_path, preload):
     # test overwrite
     with pytest.raises(FileExistsError, match='Destination file exists'):
         epochs.export(temp_fname, overwrite=False)
-    epochs.export(temp_fname, overwrite=True)
+    with ctx():
+        epochs.export(temp_fname, overwrite=True)
 
     # test pathlib.Path files
-    epochs.export(Path(temp_fname), overwrite=True)
+    with ctx():
+        epochs.export(Path(temp_fname), overwrite=True)
 
     # test warning with unapplied projectors
     epochs = Epochs(raw, events, preload=preload, proj=False)
