@@ -12,24 +12,54 @@ at which the fix is no longer needed.
 #          Lars Buitinck <L.J.Buitinck@uva.nl>
 # License: BSD
 
-from distutils.version import LooseVersion
-import functools
 import inspect
 from math import log
+from pprint import pprint
+from io import StringIO
 import os
-from pathlib import Path
 import warnings
 
 import numpy as np
 
 
+###############################################################################
+# distutils
+
+# distutils has been deprecated since Python 3.10 and is scheduled for removal
+# from the standard library with the release of Python 3.12. For version
+# comparisons, we use setuptools's `parse_version` if available.
+
+def _compare_version(version_a, operator, version_b):
+    """Compare two version strings via a user-specified operator.
+
+    Parameters
+    ----------
+    version_a : str
+        First version string.
+    operator : '==' | '>' | '<' | '>=' | '<='
+        Operator to compare ``version_a`` and ``version_b`` in the form of
+        ``version_a operator version_b``.
+    version_b : str
+        Second version string.
+
+    Returns
+    -------
+    bool
+        The result of the version comparison.
+    """
+    from packaging.version import parse
+    with warnings.catch_warnings(record=True):
+        warnings.simplefilter('ignore')
+        return eval(f'parse("{version_a}") {operator} parse("{version_b}")')
+
+
 ###############################################################################
 # Misc
 
 def _median_complex(data, axis):
     """Compute marginal median on complex data safely.
 
-    XXX: Can be removed when numpy introduces a fix.
+    Can be removed when numpy introduces a fix.
     See: https://github.com/scipy/scipy/pull/12676/.
     """
     # np.median must be passed real arrays for the desired result
@@ -41,21 +71,6 @@ def _median_complex(data, axis):
     return data
 
 
-# helpers to get function arguments
-def _get_args(function, varargs=False):
-    params = inspect.signature(function).parameters
-    args = [key for key, param in params.items()
-            if param.kind not in (param.VAR_POSITIONAL, param.VAR_KEYWORD)]
-    if varargs:
-        varargs = [param.name for param in params.values()
-                   if param.kind == param.VAR_POSITIONAL]
-        if len(varargs) == 0:
-            varargs = None
-        return args, varargs
-    else:
-        return args
-
-
 def _safe_svd(A, **kwargs):
     """Wrapper to get around the SVD did not converge error of death"""
     # Intel has a bug with their GESVD driver:
@@ -69,12 +84,9 @@ def _safe_svd(A, **kwargs):
         return linalg.svd(A, **kwargs)
     except np.linalg.LinAlgError as exp:
         from .utils import warn
-        if 'lapack_driver' in _get_args(linalg.svd):
-            warn('SVD error (%s), attempting to use GESVD instead of GESDD'
-                 % (exp,))
-            return linalg.svd(A, lapack_driver='gesvd', **kwargs)
-        else:
-            raise
+        warn('SVD error (%s), attempting to use GESVD instead of GESDD'
+                % (exp,))
+        return linalg.svd(A, lapack_driver='gesvd', **kwargs)
 
 
 def _csc_matrix_cast(x):
@@ -147,7 +159,7 @@ def _read_geometry(filepath, read_metadata=False, read_stamp=False):
         else:
             raise ValueError("File does not appear to be a Freesurfer surface")
 
-    coords = coords.astype(np.float64)  # XXX: due to mayavi bug on mac 32bits
+    coords = coords.astype(np.float64)
 
     ret = (coords, faces)
     if read_metadata:
@@ -160,27 +172,6 @@ def _read_geometry(filepath, read_metadata=False, read_stamp=False):
     return ret
 
 
-###############################################################################
-# Triaging FFT functions to get fast pocketfft (SciPy 1.4)
-
-@functools.lru_cache(None)
-def _import_fft(name):
-    single = False
-    if not isinstance(name, tuple):
-        name = (name,)
-        single = True
-    try:
-        from scipy.fft import rfft  # noqa analysis:ignore
-    except ImportError:
-        from numpy import fft  # noqa
-    else:
-        from scipy import fft  # noqa
-    out = [getattr(fft, n) for n in name]
-    if single:
-        out = out[0]
-    return out
-
-
 ###############################################################################
 # NumPy Generator (NumPy 1.17)
 
@@ -450,10 +441,11 @@ class BaseEstimator(object):
         return self
 
     def __repr__(self):
-        from sklearn.base import _pprint
+        params = StringIO()
+        pprint(self.get_params(deep=False), params)
+        params.seek(0)
         class_name = self.__class__.__name__
-        return '%s(%s)' % (class_name, _pprint(self.get_params(deep=False),
-                                               offset=len(class_name),),)
+        return '%s(%s)' % (class_name, params.read().strip())
 
     # __getstate__ and __setstate__ are omitted because they only contain
     # conditionals that are not satisfied by our objects (e.g.,
@@ -892,9 +884,9 @@ def svd(a, hermitian=False):
         sgn = np.sign(s)
         s = np.abs(s)
         sidx = np.argsort(s)[..., ::-1]
-        sgn = take_along_axis(sgn, sidx, axis=-1)
-        s = take_along_axis(s, sidx, axis=-1)
-        u = take_along_axis(u, sidx[..., None, :], axis=-1)
+        sgn = np.take_along_axis(sgn, sidx, axis=-1)
+        s = np.take_along_axis(s, sidx, axis=-1)
+        u = np.take_along_axis(u, sidx[..., None, :], axis=-1)
         # singular values are unsigned, move the sign into v
         vt = (u * sgn[..., np.newaxis, :]).swapaxes(-2, -1).conj()
         np.abs(s, out=s)
@@ -903,58 +895,6 @@ def svd(a, hermitian=False):
         return np.linalg.svd(a)
 
 
-###############################################################################
-# NumPy einsum backward compat (allow "optimize" arg and fix 1.14.0 bug)
-# XXX eventually we should hand-tune our `einsum` calls given our array sizes!
-
-def einsum(*args, **kwargs):
-    if 'optimize' not in kwargs:
-        kwargs['optimize'] = False
-    return np.einsum(*args, **kwargs)
-
-
-try:
-    from numpy import take_along_axis
-except ImportError:  # NumPy < 1.15
-    def take_along_axis(arr, indices, axis):
-        # normalize inputs
-        if axis is None:
-            arr = arr.flat
-            arr_shape = (len(arr),)  # flatiter has no .shape
-            axis = 0
-        else:
-            # there is a NumPy function for this, but rather than copy our
-            # internal uses should be correct, so just normalize quickly
-            if axis < 0:
-                axis += arr.ndim
-            assert 0 <= axis < arr.ndim
-            arr_shape = arr.shape
-
-        # use the fancy index
-        return arr[_make_along_axis_idx(arr_shape, indices, axis)]
-
-    def _make_along_axis_idx(arr_shape, indices, axis):
-        # compute dimensions to iterate over
-        if not np.issubdtype(indices.dtype, np.integer):
-            raise IndexError('`indices` must be an integer array')
-        if len(arr_shape) != indices.ndim:
-            raise ValueError(
-                "`indices` and `arr` must have the same number of dimensions")
-        shape_ones = (1,) * indices.ndim
-        dest_dims = list(range(axis)) + [None] + list(range(axis+1, indices.ndim))
-
-        # build a fancy index, consisting of orthogonal aranges, with the
-        # requested index inserted at the right location
-        fancy_index = []
-        for dim, n in zip(dest_dims, arr_shape):
-            if dim is None:
-                fancy_index.append(indices)
-            else:
-                ind_shape = shape_ones[:dim] + (-1,) + shape_ones[dim+1:]
-                fancy_index.append(np.arange(n).reshape(ind_shape))
-
-        return tuple(fancy_index)
-
 ###############################################################################
 # From nilearn
 
@@ -980,7 +920,7 @@ def _crop_colorbar(cbar, cbar_vmin, cbar_vmax):
     # _outline was removed in
     # https://github.com/matplotlib/matplotlib/commit/03a542e875eba091a027046d5ec652daa8be6863
     # so we use the code from there
-    if LooseVersion(matplotlib.__version__) >= LooseVersion("3.2.0"):
+    if _compare_version(matplotlib.__version__, '>=', '3.2.0'):
         cbar.ax.set_ylim(cbar_vmin, cbar_vmax)
         X = cbar._mesh()[0]
         X = np.array([X[0], X[-1]])
@@ -1005,31 +945,20 @@ def _crop_colorbar(cbar, cbar_vmin, cbar_vmax):
     cbar.update_ticks()
 
 
-###############################################################################
-# Matplotlib
-
-def _get_status(checks):
-    """Deal with old MPL to get check box statuses."""
-    try:
-        return list(checks.get_status())
-    except AttributeError:
-        return [x[0].get_visible() for x in checks.lines]
-
-
 ###############################################################################
 # Numba (optional requirement)
 
 # Here we choose different defaults to speed things up by default
 try:
     import numba
-    if LooseVersion(numba.__version__) < LooseVersion('0.40'):
+    if _compare_version(numba.__version__, '<', '0.48'):
         raise ImportError
     prange = numba.prange
     def jit(nopython=True, nogil=True, fastmath=True, cache=True,
             **kwargs):  # noqa
         return numba.jit(nopython=nopython, nogil=nogil, fastmath=fastmath,
                          cache=cache, **kwargs)
-except ImportError:
+except Exception:  # could be ImportError, SystemError, etc.
     has_numba = False
 else:
     has_numba = (os.getenv('MNE_USE_NUMBA', 'true').lower() == 'true')
@@ -1073,16 +1002,23 @@ else:
 
 
 ###############################################################################
-# Added in Python 3.7 (remove when we drop support for 3.6)
+# Matplotlib
 
-try:
-    from contextlib import nullcontext
-except ImportError:
-    from contextlib import contextmanager
-
-    @contextmanager
-    def nullcontext():
-        yield
+# workaround: plt.close() doesn't spawn close_event on Agg backend
+# https://github.com/matplotlib/matplotlib/issues/18609
+# scheduled to be fixed by MPL 3.6
+def _close_event(fig):
+    """Force calling of the MPL figure close event."""
+    from .utils import logger
+    from matplotlib import backend_bases
+    try:
+        fig.canvas.callbacks.process(
+            'close_event', backend_bases.CloseEvent(
+                name='close_event', canvas=fig.canvas))
+        logger.debug(f'Called {fig!r}.canvas.close_event()')
+    except ValueError:  # old mpl with Qt
+        logger.debug(f'Calling {fig!r}.canvas.close_event() failed')
+        pass  # pragma: no cover
 
 
 def _is_last_row(ax):
@@ -1093,12 +1029,18 @@ def _is_last_row(ax):
     return ax.get_subplotspec().is_last_row()
 
 
+def _sharex(ax1, ax2):
+    if hasattr(ax1.axes, 'sharex'):
+        ax1.axes.sharex(ax2)
+    else:
+        ax1.get_shared_x_axes().join(ax1, ax2)
+
+
 ###############################################################################
 # SciPy deprecation of pinv + pinvh rcond (never worked properly anyway) in 1.7
 
 def pinvh(a, rtol=None):
     """Compute a pseudo-inverse of a Hermitian matrix."""
-    from scipy.linalg.decomp import _asarray_validated
     s, u = np.linalg.eigh(a)
     del a
     if rtol is None:
