@@ -15,6 +15,7 @@ from mne.inverse_sparse.mxne_optim import (mixed_norm_solver,
                                            norm_epsilon_inf, norm_epsilon,
                                            _Phi, _PhiT, dgap_l21l1)
 from mne.time_frequency._stft import stft_norm2
+from mne.utils import catch_logging, _record_warnings
 
 
 def _generate_tf_data():
@@ -46,47 +47,31 @@ def test_l21_mxne():
     M = np.dot(G, X)
 
     args = (M, G, alpha, 1000, 1e-8)
-    with pytest.warns(None):  # CD
-        X_hat_prox, active_set, _ = mixed_norm_solver(
-            *args, active_set_size=None,
-            debias=True, solver='prox')
-    assert_array_equal(np.where(active_set)[0], [0, 4])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_cd, active_set, _, gap_cd = mixed_norm_solver(
             *args, active_set_size=None,
             debias=True, solver='cd', return_gap=True)
     assert_array_less(gap_cd, 1e-8)
     assert_array_equal(np.where(active_set)[0], [0, 4])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, E, gap_bcd = mixed_norm_solver(
             M, G, alpha, maxit=1000, tol=1e-8, active_set_size=None,
             debias=True, solver='bcd', return_gap=True)
     assert_array_less(gap_bcd, 9.6e-9)
     assert_array_equal(np.where(active_set)[0], [0, 4])
-    assert_allclose(X_hat_prox, X_hat_cd, rtol=1e-2)
-    assert_allclose(X_hat_prox, X_hat_bcd, rtol=1e-2)
     assert_allclose(X_hat_bcd, X_hat_cd, rtol=1e-2)
 
-    with pytest.warns(None):  # CD
-        X_hat_prox, active_set, _ = mixed_norm_solver(
-            *args, active_set_size=2, debias=True, solver='prox')
-    assert_array_equal(np.where(active_set)[0], [0, 4])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_cd, active_set, _ = mixed_norm_solver(
             *args, active_set_size=2, debias=True, solver='cd')
     assert_array_equal(np.where(active_set)[0], [0, 4])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, _ = mixed_norm_solver(
             *args, active_set_size=2, debias=True, solver='bcd')
     assert_array_equal(np.where(active_set)[0], [0, 4])
     assert_allclose(X_hat_bcd, X_hat_cd, rtol=1e-2)
-    assert_allclose(X_hat_bcd, X_hat_prox, rtol=1e-2)
 
-    with pytest.warns(None):  # CD
-        X_hat_prox, active_set, _ = mixed_norm_solver(
-            *args, active_set_size=2, debias=True, n_orient=2, solver='prox')
-    assert_array_equal(np.where(active_set)[0], [0, 1, 4, 5])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, _ = mixed_norm_solver(
             *args, active_set_size=2, debias=True, n_orient=2, solver='bcd')
     assert_array_equal(np.where(active_set)[0], [0, 1, 4, 5])
@@ -96,24 +81,40 @@ def test_l21_mxne():
         X_hat_cd, active_set, _ = mixed_norm_solver(
             *args, active_set_size=2, debias=True, n_orient=2, solver='cd')
     assert_array_equal(np.where(active_set)[0], [0, 1, 4, 5])
-    assert_allclose(X_hat_bcd, X_hat_prox, rtol=1e-2)
     assert_allclose(X_hat_bcd, X_hat_cd, rtol=1e-2)
 
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, _ = mixed_norm_solver(
             *args, active_set_size=2, debias=True, n_orient=5, solver='bcd')
     assert_array_equal(np.where(active_set)[0], [0, 1, 2, 3, 4])
-    with pytest.warns(None):  # CD
-        X_hat_prox, active_set, _ = mixed_norm_solver(
-            *args, active_set_size=2, debias=True, n_orient=5, solver='prox')
-    assert_array_equal(np.where(active_set)[0], [0, 1, 2, 3, 4])
     with pytest.warns(RuntimeWarning, match='descent'):
         X_hat_cd, active_set, _ = mixed_norm_solver(
             *args, active_set_size=2, debias=True, n_orient=5, solver='cd')
 
     assert_array_equal(np.where(active_set)[0], [0, 1, 2, 3, 4])
-    assert_array_equal(X_hat_bcd, X_hat_cd)
-    assert_allclose(X_hat_bcd, X_hat_prox, rtol=1e-2)
+    assert_allclose(X_hat_bcd, X_hat_cd)
+
+
+@pytest.mark.slowtest
+def test_non_convergence():
+    """Test non-convergence of MxNE solver to catch unexpected bugs."""
+    n, p, t, alpha = 30, 40, 20, 1.
+    rng = np.random.RandomState(0)
+    G = rng.randn(n, p)
+    G /= np.std(G, axis=0)[None, :]
+    X = np.zeros((p, t))
+    X[0] = 3
+    X[4] = -2
+    M = np.dot(G, X)
+
+    # Impossible to converge with only 1 iteration and tol 1e-12
+    # In case of non-convegence, we test that no error is returned.
+    args = (M, G, alpha, 1, 1e-12)
+    with catch_logging() as log:
+        mixed_norm_solver(*args, active_set_size=None, debias=True,
+                          solver='bcd', verbose=True)
+    log = log.getvalue()
+    assert 'Convergence reached' not in log
 
 
 def test_tf_mxne():
@@ -123,7 +124,7 @@ def test_tf_mxne():
 
     M, G, active_set = _generate_tf_data()
 
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_tf, active_set_hat_tf, E, gap_tfmxne = tf_mixed_norm_solver(
             M, G, alpha_space, alpha_time, maxit=200, tol=1e-8, verbose=True,
             n_orient=1, tstep=4, wsize=32, return_gap=True)
@@ -251,40 +252,29 @@ def test_iterative_reweighted_mxne():
     X[4] = -2
     M = np.dot(G, X)
 
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_l21, _, _ = mixed_norm_solver(
             M, G, alpha, maxit=1000, tol=1e-8, verbose=False, n_orient=1,
             active_set_size=None, debias=False, solver='bcd')
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, _ = iterative_mixed_norm_solver(
             M, G, alpha, 1, maxit=1000, tol=1e-8, active_set_size=None,
             debias=False, solver='bcd')
-    with pytest.warns(None):  # CD
-        X_hat_prox, active_set, _ = iterative_mixed_norm_solver(
-            M, G, alpha, 1, maxit=1000, tol=1e-8, active_set_size=None,
-            debias=False, solver='prox')
     assert_allclose(X_hat_bcd, X_hat_l21, rtol=1e-3)
-    assert_allclose(X_hat_prox, X_hat_l21, rtol=1e-3)
 
-    with pytest.warns(None):  # CD
-        X_hat_prox, active_set, _ = iterative_mixed_norm_solver(
-            M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=None,
-            debias=True, solver='prox')
-    assert_array_equal(np.where(active_set)[0], [0, 4])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, _ = iterative_mixed_norm_solver(
             M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=2,
             debias=True, solver='bcd')
     assert_array_equal(np.where(active_set)[0], [0, 4])
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_cd, active_set, _ = iterative_mixed_norm_solver(
             M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=None,
             debias=True, solver='cd')
     assert_array_equal(np.where(active_set)[0], [0, 4])
-    assert_array_almost_equal(X_hat_prox, X_hat_cd, 5)
     assert_array_almost_equal(X_hat_bcd, X_hat_cd, 5)
 
-    with pytest.warns(None):  # CD
+    with _record_warnings():  # CD
         X_hat_bcd, active_set, _ = iterative_mixed_norm_solver(
             M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=2,
             debias=True, n_orient=2, solver='bcd')
@@ -295,7 +285,7 @@ def test_iterative_reweighted_mxne():
             M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=2,
             debias=True, n_orient=2, solver='cd')
     assert_array_equal(np.where(active_set)[0], [0, 1, 4, 5])
-    assert_array_equal(X_hat_bcd, X_hat_cd, 5)
+    assert_allclose(X_hat_bcd, X_hat_cd)
 
     X_hat_bcd, active_set, _ = iterative_mixed_norm_solver(
         M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=2, debias=True,
@@ -306,7 +296,7 @@ def test_iterative_reweighted_mxne():
             M, G, alpha, 5, maxit=1000, tol=1e-8, active_set_size=2,
             debias=True, n_orient=5, solver='cd')
     assert_array_equal(np.where(active_set)[0], [0, 1, 2, 3, 4])
-    assert_array_equal(X_hat_bcd, X_hat_cd, 5)
+    assert_allclose(X_hat_bcd, X_hat_cd)
 
 
 @pytest.mark.slowtest
