@@ -202,6 +202,8 @@ FIFF.FIFFV_EXCI_CH      = 920  # flux excitation channel used to be a stimulus c
 FIFF.FIFFV_DIPOLE_WAVE  = 1000  # Dipole time curve (xplotter/xfit)
 FIFF.FIFFV_GOODNESS_FIT = 1001  # Goodness of fit (xplotter/xfit)
 FIFF.FIFFV_FNIRS_CH     = 1100  # Functional near-infrared spectroscopy
+FIFF.FIFFV_TEMPERATURE_CH = 1200  # Functional near-infrared spectroscopy
+FIFF.FIFFV_GALVANIC_CH  = 1300  # Galvanic skin response
 _ch_kind_named = {key: key for key in (
     FIFF.FIFFV_BIO_CH,
     FIFF.FIFFV_MEG_CH,
@@ -223,6 +225,8 @@ _ch_kind_named = {key: key for key in (
     FIFF.FIFFV_DIPOLE_WAVE,
     FIFF.FIFFV_GOODNESS_FIT,
     FIFF.FIFFV_FNIRS_CH,
+    FIFF.FIFFV_GALVANIC_CH,
+    FIFF.FIFFV_TEMPERATURE_CH,
 )}
 
 #
@@ -764,10 +768,6 @@ FWD.COIL_ACCURACY_POINT          = 0
 FWD.COIL_ACCURACY_NORMAL         = 1
 FWD.COIL_ACCURACY_ACCURATE       = 2
 
-FWD.BEM_UNKNOWN                  = -1
-FWD.BEM_CONSTANT_COLL            = 1
-FWD.BEM_LINEAR_COLL              = 2
-
 FWD.BEM_IP_APPROACH_LIMIT        = 0.1
 
 FWD.BEM_LIN_FIELD_SIMPLE         = 1
@@ -839,7 +839,7 @@ FIFF.FIFF_UNIT_C   = 106  # coulomb
 FIFF.FIFF_UNIT_V   = 107  # volt
 FIFF.FIFF_UNIT_F   = 108  # farad
 FIFF.FIFF_UNIT_OHM = 109  # ohm
-FIFF.FIFF_UNIT_MHO = 110  # one per ohm
+FIFF.FIFF_UNIT_S   = 110  # Siemens (same as Moh, what fiff-constants calls it)
 FIFF.FIFF_UNIT_WB  = 111  # weber
 FIFF.FIFF_UNIT_T   = 112  # tesla
 FIFF.FIFF_UNIT_H   = 113  # Henry
@@ -861,7 +861,7 @@ _ch_unit_named = {key: key for key in(
     FIFF.FIFF_UNIT_CD, FIFF.FIFF_UNIT_MOL_M3, FIFF.FIFF_UNIT_HZ,
     FIFF.FIFF_UNIT_N, FIFF.FIFF_UNIT_PA, FIFF.FIFF_UNIT_J, FIFF.FIFF_UNIT_W,
     FIFF.FIFF_UNIT_C, FIFF.FIFF_UNIT_V, FIFF.FIFF_UNIT_F, FIFF.FIFF_UNIT_OHM,
-    FIFF.FIFF_UNIT_MHO, FIFF.FIFF_UNIT_WB, FIFF.FIFF_UNIT_T, FIFF.FIFF_UNIT_H,
+    FIFF.FIFF_UNIT_S, FIFF.FIFF_UNIT_WB, FIFF.FIFF_UNIT_T, FIFF.FIFF_UNIT_H,
     FIFF.FIFF_UNIT_CEL, FIFF.FIFF_UNIT_LM, FIFF.FIFF_UNIT_LX,
     FIFF.FIFF_UNIT_V_M2, FIFF.FIFF_UNIT_T_M, FIFF.FIFF_UNIT_AM,
     FIFF.FIFF_UNIT_AM_M2, FIFF.FIFF_UNIT_AM_M3,
