@@ -1,6 +1,6 @@
 # Author: Joan Massich <mailsik@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import numpy as np
 import os.path as op
@@ -13,7 +13,7 @@ from .info import _convert_time
 
 def _get_markers(fname):
     def consume(fid, predicate):  # just a consumer to move around conveniently
-        while(predicate(fid.readline())):
+        while predicate(fid.readline()):
             pass
 
     def parse_marker(string):  # XXX: there should be a nicer way to do that
@@ -23,17 +23,18 @@ def _get_markers(fname):
 
     markers = dict()
     with open(fname) as fid:
-        consume(fid, lambda l: not l.startswith('NUMBER OF MARKERS:'))
+        consume(fid, lambda line: not line.startswith('NUMBER OF MARKERS:'))
         num_of_markers = int(fid.readline())
 
         for _ in range(num_of_markers):
-            consume(fid, lambda l: not l.startswith('NAME:'))
+            consume(fid, lambda line: not line.startswith('NAME:'))
             label = fid.readline().strip('\n')
 
-            consume(fid, lambda l: not l.startswith('NUMBER OF SAMPLES:'))
+            consume(
+                fid, lambda line: not line.startswith('NUMBER OF SAMPLES:'))
             n_markers = int(fid.readline())
 
-            consume(fid, lambda l: not l.startswith('LIST OF SAMPLES:'))
+            consume(fid, lambda line: not line.startswith('LIST OF SAMPLES:'))
             next(fid)  # skip the samples header
             markers[label] = [
                 parse_marker(next(fid)) for _ in range(n_markers)
