@@ -3,14 +3,17 @@
 # Authors: Dirk Gütlin <dirk.guetlin@stud.sbg.ac.at>
 #
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
-import os.path as op
 from collections import namedtuple
-import re
-import numpy as np
 from datetime import datetime, timezone
+import os.path as op
+from pathlib import Path
+import re
 
+import numpy as np
+
+from .._digitization import _make_dig_points
 from ..base import BaseRaw
 from ..meas_info import create_info
 from ..tag import _coil_trans_to_loc
@@ -21,8 +24,7 @@ from ...surface import _normal_orth
 from ...transforms import (apply_trans, Transform, get_ras_to_neuromag_trans,
                            combine_transforms, invert_transform,
                            _angle_between_quats, rot_to_quat)
-from ...utils import (check_fname, check_version, logger, verbose, warn,
-                      _check_fname)
+from ...utils import check_fname, logger, verbose, _check_fname
 from ...annotations import Annotations
 
 FILE_EXTENSIONS = {
@@ -30,14 +32,16 @@ FILE_EXTENSIONS = {
         "info": ".dap",
         "data": ".dat",
         "labels": ".rs3",
-        "events": ".cef",
+        "events_cef": ".cef",
+        "events_ceo": ".ceo",
         "hpi": ".hpi",
     },
     "Curry 8": {
         "info": ".cdt.dpa",
         "data": ".cdt",
         "labels": ".cdt.dpa",
-        "events": ".cdt.cef",
+        "events_cef": ".cdt.cef",
+        "events_ceo": ".cdt.ceo",
         "hpi": ".cdt.hpi",
     }
 }
@@ -63,16 +67,23 @@ def _get_curry_file_structure(fname, required=()):
     """Store paths to a dict and check for required files."""
     _msg = "The following required files cannot be found: {0}.\nPlease make " \
            "sure all required files are located in the same directory as {1}."
-    fname = _check_fname(fname, 'read', True, 'fname')
+    fname = Path(_check_fname(fname, 'read', True, 'fname'))
 
     # we don't use os.path.splitext to also handle extensions like .cdt.dpa
-    fname_base, ext = fname.split(".", maxsplit=1)
+    # this won't handle a dot in the filename, but it should handle it in
+    # the parent directories
+    fname_base = fname.name.split('.', maxsplit=1)[0]
+    ext = fname.name[len(fname_base):]
+    fname_base = str(fname)
+    fname_base = fname_base[:len(fname_base) - len(ext)]
+    del fname
     version = _get_curry_version(ext)
     my_curry = dict()
-    for key in ('info', 'data', 'labels', 'events', 'hpi'):
+    for key in ('info', 'data', 'labels', 'events_cef', 'events_ceo', 'hpi'):
         fname = fname_base + FILE_EXTENSIONS[version][key]
         if op.isfile(fname):
-            my_curry[key] = fname
+            _key = 'events' if key.startswith('events') else key
+            my_curry[_key] = fname
 
     missing = [field for field in required if field not in my_curry]
     if missing:
@@ -224,6 +235,7 @@ def _read_curry_info(curry_paths):
     assert len(labels) == len(sensors) == len(normals)
 
     all_chans = list()
+    dig_ch_pos = dict()
     for key in ["meg", "eeg", "misc"]:
         chanidx_is_explicit = (len(curry_params.chanidx_in_file["CHAN_IN_FILE"
                                    + CHANTYPES[key]]) > 0)    # channel index
@@ -257,6 +269,7 @@ def _read_curry_info(curry_paths):
                 ch['loc'] = loc
                 # XXX need to check/ensure this
                 ch['coord_frame'] = FIFF.FIFFV_COORD_HEAD
+                dig_ch_pos[chan] = loc[:3]
             elif key == 'meg':
                 pos = np.array(sensors["SENSORS" + CHANTYPES[key]][ind], float)
                 pos /= 1000.  # to meters
@@ -272,6 +285,9 @@ def _read_curry_info(curry_paths):
                 ch['loc'] = _coil_trans_to_loc(trans)
                 ch['coord_frame'] = FIFF.FIFFV_COORD_DEVICE
             all_chans.append(ch)
+    dig = _make_dig_points(
+        dig_ch_pos=dig_ch_pos, coord_frame='head', add_missing_fiducials=True)
+    del dig_ch_pos
 
     ch_count = len(all_chans)
     assert (ch_count == curry_params.n_chans)  # ensure that we have assembled
@@ -286,7 +302,9 @@ def _read_curry_info(curry_paths):
 
     ch_names = [chan["ch_name"] for chan in all_chans]
     info = create_info(ch_names, curry_params.sfreq)
-    info['meas_date'] = curry_params.dt_start          # for Git issue #8398
+    with info._unlock():
+        info['meas_date'] = curry_params.dt_start  # for Git issue #8398
+        info['dig'] = dig
     _make_trans_dig(curry_paths, info, curry_dev_dev_t)
 
     for ind, ch_dict in enumerate(info["chs"]):
@@ -321,7 +339,8 @@ def _make_trans_dig(curry_paths, info, curry_dev_dev_t):
     key = 'LM_REMARKS' + CHANTYPES['meg']
     remarks = _read_curry_lines(label_fname, [key])[key]
     assert len(remarks) == len(lm)
-    info['dig'] = list()
+    with info._unlock():
+        info['dig'] = list()
     cards = dict()
     for remark, r in zip(remarks, lm):
         kind = ident = None
@@ -336,7 +355,8 @@ def _make_trans_dig(curry_paths, info, curry_dev_dev_t):
             info['dig'].append(dict(
                 kind=kind, ident=ident, r=r,
                 coord_frame=FIFF.FIFFV_COORD_UNKNOWN))
-    info['dig'].sort(key=lambda x: (x['kind'], x['ident']))
+    with info._unlock():
+        info['dig'].sort(key=lambda x: (x['kind'], x['ident']))
     has_cards = len(cards) == 3
     has_hpi = 'hpi' in curry_paths
     if has_cards and has_hpi:  # have all three
@@ -360,11 +380,12 @@ def _make_trans_dig(curry_paths, info, curry_dev_dev_t):
                 *(cards[key] for key in (FIFF.FIFFV_POINT_NASION,
                                          FIFF.FIFFV_POINT_LPA,
                                          FIFF.FIFFV_POINT_RPA))))
-        info['dev_head_t'] = combine_transforms(
-            invert_transform(unknown_dev_t), unknown_head_t, 'meg', 'head')
-        for d in info['dig']:
-            d.update(coord_frame=FIFF.FIFFV_COORD_HEAD,
-                     r=apply_trans(unknown_head_t, d['r']))
+        with info._unlock():
+            info['dev_head_t'] = combine_transforms(
+                invert_transform(unknown_dev_t), unknown_head_t, 'meg', 'head')
+            for d in info['dig']:
+                d.update(coord_frame=FIFF.FIFFV_COORD_HEAD,
+                         r=apply_trans(unknown_head_t, d['r']))
     else:
         if has_cards:
             no_msg += ' (no .hpi file found)'
@@ -386,7 +407,7 @@ def _first_hpi(fname):
             break
         else:
             raise RuntimeError('Could not find valid HPI in %s' % (fname,))
-    # t is the first enttry
+    # t is the first entry
     assert hpi.ndim == 1
     hpi = hpi[1:]
     hpi.shape = (-1, 5)
@@ -408,8 +429,8 @@ def _read_events_curry(fname):
     events : ndarray, shape (n_events, 3)
         The array of events.
     """
-    check_fname(fname, 'curry event', ('.cef', '.cdt.cef'),
-                endings_err=('.cef', '.cdt.cef'))
+    check_fname(fname, 'curry event', ('.cef', '.ceo', '.cdt.cef', '.cdt.ceo'),
+                endings_err=('.cef', '.ceo', '.cdt.cef', '.cdt.ceo'))
 
     events_dict = _read_curry_lines(fname, ["NUMBER_LIST"])
     # The first 3 column seem to contain the event information
@@ -457,7 +478,7 @@ def read_raw_curry(fname, preload=False, verbose=None):
     Parameters
     ----------
     fname : str
-        Path to a curry file with extensions .dat, .dap, .rs3, .cdt, cdt.dpa,
+        Path to a curry file with extensions .dat, .dap, .rs3, .cdt, .cdt.dpa,
         .cdt.cef or .cef.
     %(preload)s
     %(verbose)s
@@ -476,7 +497,7 @@ class RawCurry(BaseRaw):
     Parameters
     ----------
     fname : str
-        Path to a curry file with extensions .dat, .dap, .rs3, .cdt, cdt.dpa,
+        Path to a curry file with extensions .dat, .dap, .rs3, .cdt, .cdt.dpa,
         .cdt.cef or .cef.
     %(preload)s
     %(verbose)s
@@ -518,16 +539,10 @@ class RawCurry(BaseRaw):
         if self._raw_extras[fi]['is_ascii']:
             if isinstance(idx, slice):
                 idx = np.arange(idx.start, idx.stop)
-            kwargs = dict(skiprows=start, usecols=idx)
-            if check_version("numpy", "1.16.0"):
-                kwargs['max_rows'] = stop - start
-            else:
-                warn("Data reading might take longer for ASCII files. Update "
-                     "numpy to version 1.16.0 or greater for more efficient "
-                     "data reading.")
-            block = np.loadtxt(self._filenames[0], **kwargs)[:stop - start].T
-            data_view = data[:, :block.shape[1]]
-            _mult_cal_one(data_view, block, idx, cals, mult)
+            block = np.loadtxt(
+                self._filenames[0], skiprows=start, max_rows=stop - start,
+                ndmin=2).T
+            _mult_cal_one(data, block, idx, cals, mult)
 
         else:
             _read_segments_file(
