@@ -7,6 +7,7 @@
 
 from datetime import datetime, timezone
 import os.path as op
+from pathlib import Path
 from shutil import copyfile
 
 import pytest
@@ -22,6 +23,7 @@ from mne.io.constants import FIFF
 from mne.io.edf import read_raw_bdf
 from mne.io.bti import read_raw_bti
 from mne.io.curry import read_raw_curry
+from mne.io.tests.test_raw import _test_raw_reader
 from mne.utils import check_version, catch_logging, _record_warnings
 from mne.annotations import read_annotations
 from mne.io.curry.curry import (_get_curry_version, _get_curry_file_structure,
@@ -98,6 +100,18 @@ def test_read_raw_curry(fname, tol, preload, bdf_curry_ref):
     assert raw.info['dev_head_t'] is None
 
 
+@testing.requires_testing_data
+@pytest.mark.parametrize('fname', [
+    pytest.param(curry7_bdf_file, id='curry 7'),
+    pytest.param(curry8_bdf_file, id='curry 8'),
+    pytest.param(curry7_bdf_ascii_file, id='curry 7 ascii'),
+    pytest.param(curry8_bdf_ascii_file, id='curry 8 ascii'),
+])
+def test_read_raw_curry_test_raw(fname):
+    """Test read_raw_curry with _test_raw_reader."""
+    _test_raw_reader(read_raw_curry, fname=fname)
+
+
 # These values taken from a different recording but allow us to test
 # using our existing filres
 
@@ -478,3 +492,22 @@ def test_meas_date(fname, expected_meas_date):
     # If the information is not valid, raw.info['meas_date'] should be None
     raw = read_raw_curry(fname, preload=False)
     assert raw.info['meas_date'] == expected_meas_date
+
+
+@testing.requires_testing_data
+@pytest.mark.parametrize('fname, others', [
+    pytest.param(curry7_rfDC_file, ('.dap', '.rs3'), id='curry7'),
+    pytest.param(curry8_rfDC_file, ('.cdt.dpa',), id='curry8'),
+])
+def test_dot_names(fname, others, tmp_path):
+    """Test that dots are parsed properly (e.g., in paths)."""
+    my_path = tmp_path / 'dot.dot.dot'
+    my_path.mkdir()
+    my_path = my_path / Path(fname).parts[-1]
+    fname = Path(fname)
+    copyfile(fname, my_path)
+    for ext in others:
+        this_fname = fname.with_suffix(ext)
+        to_fname = my_path.with_suffix(ext)
+        copyfile(this_fname, to_fname)
+    read_raw_curry(my_path)
