@@ -17,9 +17,9 @@ import re
 
 import numpy as np
 
-from ...utils import verbose, logger, warn
+from ...utils import verbose, logger, warn, _validate_type
 from ..utils import _blk_read_lims, _mult_cal_one
-from ..base import BaseRaw
+from ..base import BaseRaw, _get_scaling
 from ..meas_info import _empty_info, _unique_channel_names
 from ..constants import FIFF
 from ...filter import resample
@@ -38,6 +38,7 @@ CH_TYPE_MAPPING = {
     'EMG': FIFF.FIFFV_EMG_CH,
     'BIO': FIFF.FIFFV_BIO_CH,
     'RESP': FIFF.FIFFV_RESP_CH,
+    'TEMP': FIFF.FIFFV_TEMPERATURE_CH,
     'MISC': FIFF.FIFFV_MISC_CH,
     'SAO2': FIFF.FIFFV_BIO_CH,
 }
@@ -83,6 +84,8 @@ class RawEDF(BaseRaw):
 
         .. versionadded:: 1.1
     %(preload)s
+    %(units_edf_bdf_io)s
+    %(encoding_edf)s
     %(verbose)s
 
     See Also
@@ -132,7 +135,7 @@ class RawEDF(BaseRaw):
     @verbose
     def __init__(self, input_fname, eog=None, misc=None, stim_channel='auto',
                  exclude=(), infer_types=False, preload=False, include=None,
-                 verbose=None):
+                 units=None, encoding='utf8', *, verbose=None):
         logger.info('Extracting EDF parameters from {}...'.format(input_fname))
         input_fname = os.path.abspath(input_fname)
         info, edf_info, orig_units = _get_info(input_fname, stim_channel, eog,
@@ -140,6 +143,26 @@ class RawEDF(BaseRaw):
                                                preload, include)
         logger.info('Creating raw.info structure...')
 
+        _validate_type(units, (str, None, dict), 'units')
+        if units is None:
+            units = dict()
+        elif isinstance(units, str):
+            units = {ch_name: units for ch_name in info['ch_names']}
+
+        for k, (this_ch, this_unit) in enumerate(orig_units.items()):
+            if this_ch not in units:
+                continue
+            if this_unit not in ("", units[this_ch]):
+                raise ValueError(
+                    f'Unit for channel {this_ch} is present in the file as '
+                    f'{repr(this_unit)}, cannot overwrite it with the units '
+                    f'argument {repr(units[this_ch])}.')
+            if this_unit == "":
+                orig_units[this_ch] = units[this_ch]
+                ch_type = edf_info["ch_types"][k]
+                scaling = _get_scaling(ch_type.lower(), orig_units[this_ch])
+                edf_info["units"][k] /= scaling
+
         # Raw attributes
         last_samps = [edf_info['nsamples'] - 1]
         super().__init__(info, preload, filenames=[input_fname],
@@ -155,7 +178,10 @@ class RawEDF(BaseRaw):
             tal_data = self._read_segment_file(
                 np.empty((0, self.n_times)), idx, 0, 0, int(self.n_times),
                 np.ones((len(idx), 1)), None)
-            onset, duration, desc = _read_annotations_edf(tal_data[0])
+            onset, duration, desc = _read_annotations_edf(
+                tal_data[0],
+                encoding=encoding,
+            )
 
         self.set_annotations(Annotations(onset=onset, duration=duration,
                                          description=desc, orig_time=None))
@@ -1282,7 +1308,7 @@ def _find_tal_idx(ch_names):
 @fill_doc
 def read_raw_edf(input_fname, eog=None, misc=None, stim_channel='auto',
                  exclude=(), infer_types=False, include=None, preload=False,
-                 verbose=None):
+                 units=None, encoding='utf8', *, verbose=None):
     """Reader function for EDF or EDF+ files.
 
     Parameters
@@ -1322,6 +1348,8 @@ def read_raw_edf(input_fname, eog=None, misc=None, stim_channel='auto',
 
         .. versionadded:: 1.1
     %(preload)s
+    %(units_edf_bdf_io)s
+    %(encoding_edf)s
     %(verbose)s
 
     Returns
@@ -1384,13 +1412,13 @@ def read_raw_edf(input_fname, eog=None, misc=None, stim_channel='auto',
     return RawEDF(input_fname=input_fname, eog=eog, misc=misc,
                   stim_channel=stim_channel, exclude=exclude,
                   infer_types=infer_types, preload=preload, include=include,
-                  verbose=verbose)
+                  units=units, encoding=encoding, verbose=verbose)
 
 
 @fill_doc
 def read_raw_bdf(input_fname, eog=None, misc=None, stim_channel='auto',
                  exclude=(), infer_types=False, include=None, preload=False,
-                 verbose=None):
+                 units=None, encoding='utf8', *, verbose=None):
     """Reader function for BDF files.
 
     Parameters
@@ -1430,6 +1458,8 @@ def read_raw_bdf(input_fname, eog=None, misc=None, stim_channel='auto',
 
         .. versionadded:: 1.1
     %(preload)s
+    %(units_edf_bdf_io)s
+    %(encoding_edf)s
     %(verbose)s
 
     Returns
@@ -1485,7 +1515,7 @@ def read_raw_bdf(input_fname, eog=None, misc=None, stim_channel='auto',
     return RawEDF(input_fname=input_fname, eog=eog, misc=misc,
                   stim_channel=stim_channel, exclude=exclude,
                   infer_types=infer_types, preload=preload, include=include,
-                  verbose=verbose)
+                  units=units, encoding=encoding, verbose=verbose)
 
 
 @fill_doc
@@ -1545,13 +1575,15 @@ def read_raw_gdf(input_fname, eog=None, misc=None, stim_channel='auto',
                   include=include, verbose=verbose)
 
 
-def _read_annotations_edf(annotations):
+@fill_doc
+def _read_annotations_edf(annotations, encoding='utf8'):
     """Annotation File Reader.
 
     Parameters
     ----------
     annotations : ndarray (n_chans, n_samples) | str
         Channel data in EDF+ TAL format or path to annotation file.
+    %(encoding_edf)s
 
     Returns
     -------
@@ -1566,8 +1598,9 @@ def _read_annotations_edf(annotations):
     """
     pat = '([+-]\\d+\\.?\\d*)(\x15(\\d+\\.?\\d*))?(\x14.*?)\x14\x00'
     if isinstance(annotations, str):
-        with open(annotations, encoding='latin-1') as annot_file:
-            triggers = re.findall(pat, annot_file.read())
+        with open(annotations, "rb") as annot_file:
+            triggers = re.findall(pat.encode(), annot_file.read())
+            triggers = [tuple(map(lambda x: x.decode(), t)) for t in triggers]
     else:
         tals = bytearray()
         annotations = np.atleast_2d(annotations)
@@ -1586,8 +1619,13 @@ def _read_annotations_edf(annotations):
                 # Exploit np vectorized processing
                 tals.extend(np.uint8([this_chan % 256, this_chan // 256])
                             .flatten('F'))
-
-        triggers = re.findall(pat, tals.decode('utf8'))
+        try:
+            triggers = re.findall(pat, tals.decode(encoding))
+        except UnicodeDecodeError as e:
+            raise Exception(
+                "Encountered invalid byte in at least one annotations channel."
+                " You might want to try setting \"encoding='latin1'\"."
+            ) from e
 
     events = []
     offset = 0.
@@ -1611,11 +1649,6 @@ def _read_annotations_edf(annotations):
     return zip(*events) if events else (list(), list(), list())
 
 
-def _get_edf_default_event_id(descriptions):
-    mapping = {a: n for n, a in enumerate(sorted(set(descriptions)), start=1)}
-    return mapping
-
-
 def _get_annotations_gdf(edf_info, sfreq):
     onset, duration, desc = list(), list(), list()
     events = edf_info.get('events', None)
