@@ -96,8 +96,8 @@ class RawSNIRF(BaseRaw):
                      "MNE does not support this feature. "
                      "Only the first dataset will be processed.")
 
-            manafacturer = _get_metadata_str(dat, "ManufacturerName")
-            if (optode_frame == "unknown") & (manafacturer == "Gowerlabs"):
+            manufacturer = _get_metadata_str(dat, "ManufacturerName")
+            if (optode_frame == "unknown") & (manufacturer == "Gowerlabs"):
                 optode_frame = "head"
 
             snirf_data_type = np.array(dat.get('nirs/data1/measurementList1'
@@ -333,10 +333,15 @@ class RawSNIRF(BaseRaw):
                     else:
                         extra_ps[f'EEG{len(extra_ps) + 1:03d}'] = \
                             diglocs[idx, :3]
+                add_missing_fiducials = (
+                    coord_frame == FIFF.FIFFV_COORD_HEAD and
+                    lpa is None and rpa is None and nasion is None
+                )
                 dig = _make_dig_points(
                     nasion=nasion, lpa=lpa, rpa=rpa, hpi=hpi,
                     dig_ch_pos=extra_ps,
-                    coord_frame=_frame_to_str[coord_frame])
+                    coord_frame=_frame_to_str[coord_frame],
+                    add_missing_fiducials=add_missing_fiducials)
             else:
                 ch_locs = [info['chs'][idx]['loc'][0:3]
                            for idx in range(len(channels))]
