@@ -1,8 +1,9 @@
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
+from contextlib import contextmanager
 from gzip import GzipFile
 import os.path as op
 import re
@@ -333,6 +334,14 @@ def start_file(fname, id_=None):
     return fid
 
 
+@contextmanager
+def start_and_end_file(fname, id_=None):
+    """Start and (if successfully written) close the file."""
+    with start_file(fname, id_=id_) as fid:
+        yield fid
+        end_file(fid)  # we only hit this line if the yield does not err
+
+
 def check_fiff_length(fid, close=True):
     """Ensure our file hasn't grown too large to work properly."""
     if fid.tell() > 2147483648:  # 2 ** 31, FIFF uses signed 32-bit locations
