@@ -2,9 +2,10 @@
 
 from .inverse import (InverseOperator, read_inverse_operator, apply_inverse,
                       apply_inverse_raw, make_inverse_operator,
-                      apply_inverse_epochs, write_inverse_operator,
-                      compute_rank_inverse, prepare_inverse_operator,
-                      estimate_snr, apply_inverse_cov, INVERSE_METHODS)
+                      apply_inverse_epochs, apply_inverse_tfr_epochs,
+                      write_inverse_operator, compute_rank_inverse,
+                      prepare_inverse_operator, estimate_snr,
+                      apply_inverse_cov, INVERSE_METHODS)
 from .time_frequency import (source_band_induced_power, source_induced_power,
                              compute_source_psd, compute_source_psd_epochs)
 from .resolution_matrix import (make_inverse_resolution_matrix,
