@@ -2,26 +2,27 @@
 #            Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #            Eric Larson <larson.eric.d@gmail.com>
 
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
+import copy
 import os.path as op
 import warnings
-import copy
+
 import numpy as np
 
 from .fixes import _get_img_fdata
+from .morph_map import read_morph_map
 from .parallel import parallel_func
 from .source_estimate import (
     _BaseSurfaceSourceEstimate, _BaseVolSourceEstimate, _BaseSourceEstimate,
     _get_ico_tris)
 from .source_space import SourceSpaces, _ensure_src, _grid_interp
-from .surface import read_morph_map, mesh_edges, read_surface, _compute_nearest
-from .transforms import _angle_between_quats, rot_to_quat
+from .surface import mesh_edges, read_surface, _compute_nearest
 from .utils import (logger, verbose, check_version, get_subjects_dir,
                     warn as warn_, fill_doc, _check_option, _validate_type,
-                    BunchConst, wrapped_stdout, _check_fname, warn,
-                    _ensure_int, ProgressBar, use_log_level)
-from .externals.h5io import read_hdf5, write_hdf5
+                    BunchConst, _check_fname, warn, _custom_lru_cache,
+                    _ensure_int, ProgressBar, use_log_level,
+                    _import_h5io_funcs)
 
 
 @verbose
@@ -30,7 +31,7 @@ def compute_source_morph(src, subject_from=None, subject_to='fsaverage',
                          niter_affine=(100, 100, 10), niter_sdr=(5, 5, 3),
                          spacing=5, smooth=None, warn=True, xhemi=False,
                          sparse=False, src_to=None, precompute=False,
-                         verbose=False):
+                         verbose=None):
     """Create a SourceMorph from one subject to another.
 
     Method is based on spherical morphing by FreeSurfer for surface
@@ -292,15 +293,17 @@ _SOURCE_MORPH_ATTRIBUTES = [  # used in writing
     'subject_from', 'subject_to', 'kind', 'zooms', 'niter_affine', 'niter_sdr',
     'spacing', 'smooth', 'xhemi', 'morph_mat', 'vertices_to',
     'shape', 'affine', 'pre_affine', 'sdr_morph', 'src_data',
-    'vol_morph_mat', 'verbose']
+    'vol_morph_mat']
 
 
 @fill_doc
-class SourceMorph(object):
+class SourceMorph:
     """Morph source space data from one subject to another.
 
-    .. note:: This class should not be instantiated directly.
-              Use :func:`mne.compute_source_morph` instead.
+    .. note::
+        This class should not be instantiated directly via
+        ``mne.SourceMorph(...)``. Instead, use one of the functions
+        listed in the See Also section below.
 
     Parameters
     ----------
@@ -351,6 +354,11 @@ class SourceMorph(object):
         was used.
     %(verbose)s
 
+    See Also
+    --------
+    compute_source_morph
+    read_source_morph
+
     Notes
     -----
     .. versionadded:: 0.17
@@ -360,11 +368,12 @@ class SourceMorph(object):
     .. footbibliography::
     """
 
+    @verbose
     def __init__(self, subject_from, subject_to, kind, zooms,
                  niter_affine, niter_sdr, spacing, smooth, xhemi,
                  morph_mat, vertices_to, shape,
                  affine, pre_affine, sdr_morph, src_data,
-                 vol_morph_mat, verbose=None):
+                 vol_morph_mat, *, verbose=None):
         # universal
         self.subject_from = subject_from
         self.subject_to = subject_to
@@ -387,7 +396,6 @@ class SourceMorph(object):
         # used by both
         self.src_data = src_data
         self.vol_morph_mat = vol_morph_mat
-        self.verbose = verbose
         # compute vertices_to here (partly for backward compat and no src
         # provided)
         if vertices_to is None or len(vertices_to) == 0 and kind == 'volume':
@@ -432,7 +440,7 @@ class SourceMorph(object):
         mri_space : bool | None
             Whether the image to world registration should be in mri space. The
             default (None) is mri_space=mri_resolution.
-        %(verbose_meth)s
+        %(verbose)s
 
         Returns
         -------
@@ -473,7 +481,7 @@ class SourceMorph(object):
 
         Parameters
         ----------
-        %(verbose_meth)s
+        %(verbose)s
 
         Returns
         -------
@@ -646,8 +654,9 @@ class SourceMorph(object):
             The stem of the file name. '-morph.h5' will be added if fname does
             not end with '.h5'.
         %(overwrite)s
-        %(verbose_meth)s
+        %(verbose)s
         """
+        _, write_hdf5 = _import_h5io_funcs()
         fname = _check_fname(fname, overwrite=overwrite, must_exist=False)
         if not fname.endswith('.h5'):
             fname = '%s-morph.h5' % fname
@@ -663,8 +672,10 @@ _slicers = list()
 
 
 def _debug_img(data, affine, title, shape=None):
-    # XXX uncomment these lines for debugging help with volume morph
+    # Uncomment these lines for debugging help with volume morph:
+    #
     # import nibabel as nib
+    # from scipy import sparse
     # if sparse.issparse(data):
     #     data = data.toarray()
     # data = np.asarray(data)
@@ -741,6 +752,7 @@ def read_source_morph(fname):
     source_morph : instance of SourceMorph
         The loaded morph.
     """
+    read_hdf5, _ = _import_h5io_funcs()
     vals = read_hdf5(fname)
     if vals['pre_affine'] is not None:  # reconstruct
         from dipy.align.imaffine import AffineMap
@@ -1003,94 +1015,20 @@ def _interpolate_data(stc, morph, mri_resolution, mri_space, output):
 ###############################################################################
 # Morph for VolSourceEstimate
 
-def _compute_r2(a, b):
-    return 100 * (a.ravel() @ b.ravel()) / \
-        (np.linalg.norm(a) * np.linalg.norm(b))
-
-
 def _compute_morph_sdr(mri_from, mri_to, niter_affine, niter_sdr, zooms):
     """Get a matrix that morphs data from one subject to another."""
-    with np.testing.suppress_warnings():
-        from dipy.align import imaffine, imwarp, metrics, transforms
-    from dipy.align.reslice import reslice
-
-    logger.info('Computing nonlinear Symmetric Diffeomorphic Registration...')
-
-    # reslice mri_from to zooms
-    mri_from_orig = mri_from
-    mri_from, mri_from_affine = reslice(
-        _get_img_fdata(mri_from_orig), mri_from_orig.affine,
-        mri_from_orig.header.get_zooms()[:3], zooms)
-
-    # reslice mri_to to zooms
-    mri_to, affine = reslice(
-        _get_img_fdata(mri_to), mri_to.affine,
-        mri_to.header.get_zooms()[:3], zooms)
-
-    mri_to /= mri_to.max()
-    mri_from /= mri_from.max()  # normalize
-
-    # compute center of mass
-    c_of_mass = imaffine.transform_centers_of_mass(
-        mri_to, affine, mri_from, mri_from_affine)
-
-    # set up Affine Registration
-    affreg = imaffine.AffineRegistration(
-        metric=imaffine.MutualInformationMetric(nbins=32),
-        level_iters=list(niter_affine),
-        sigmas=[3.0, 1.0, 0.0],
-        factors=[4, 2, 1])
-
-    # translation
-    logger.info('Optimizing translation:')
-    with wrapped_stdout(indent='    ', cull_newlines=True):
-        translation = affreg.optimize(
-            mri_to, mri_from, transforms.TranslationTransform3D(), None,
-            affine, mri_from_affine, starting_affine=c_of_mass.affine)
-
-    # rigid body transform (translation + rotation)
-    logger.info('Optimizing rigid-body:')
-    with wrapped_stdout(indent='    ', cull_newlines=True):
-        rigid = affreg.optimize(
-            mri_to, mri_from, transforms.RigidTransform3D(), None,
-            affine, mri_from_affine, starting_affine=translation.affine)
-    mri_from_to = rigid.transform(mri_from)
-    dist = np.linalg.norm(rigid.affine[:3, 3])
-    angle = np.rad2deg(_angle_between_quats(
-        np.zeros(3), rot_to_quat(rigid.affine[:3, :3])))
-
-    logger.info(f'    Translation: {dist:6.1f} mm')
-    logger.info(f'    Rotation:    {angle:6.1f}°')
-    logger.info(f'    R²:          {_compute_r2(mri_to, mri_from_to):6.1f}%')
-
-    # affine transform (translation + rotation + scaling)
-    logger.info('Optimizing full affine:')
-    with wrapped_stdout(indent='    ', cull_newlines=True):
-        pre_affine = affreg.optimize(
-            mri_to, mri_from, transforms.AffineTransform3D(), None,
-            affine, mri_from_affine, starting_affine=rigid.affine)
-    mri_from_to = pre_affine.transform(mri_from)
-    logger.info(f'    R²:          {_compute_r2(mri_to, mri_from_to):6.1f}%')
-
-    # SDR
-    shape = tuple(pre_affine.domain_shape)
-    if len(niter_sdr):
-        sdr = imwarp.SymmetricDiffeomorphicRegistration(
-            metrics.CCMetric(3), list(niter_sdr))
-        logger.info('Optimizing SDR:')
-        with wrapped_stdout(indent='    ', cull_newlines=True):
-            sdr_morph = sdr.optimize(mri_to, pre_affine.transform(mri_from))
-        assert shape == tuple(sdr_morph.domain_shape)  # should be tuple of int
-        mri_from_to = sdr_morph.transform(mri_from_to)
-    else:
-        sdr_morph = None
-
-    logger.info(f'    R²:          {_compute_r2(mri_to, mri_from_to):6.1f}%')
-    _debug_img(mri_from_orig.dataobj, mri_from_orig.affine, 'From')
-    _debug_img(mri_from, affine, 'From-reslice')
-    _debug_img(mri_from_to, affine, 'From-reslice')
-    _debug_img(mri_to, affine, 'To-reslice')
-    return shape, zooms, affine, pre_affine, sdr_morph
+    from .transforms import _compute_volume_registration
+    from dipy.align.imaffine import AffineMap
+    pipeline = 'all' if niter_sdr else 'affines'
+    niter = dict(translation=niter_affine, rigid=niter_affine,
+                 affine=niter_affine,
+                 sdr=niter_sdr if niter_sdr else (1,))
+    pre_affine, sdr_morph, to_shape, to_affine, from_shape, from_affine = \
+        _compute_volume_registration(
+            mri_from, mri_to, zooms=zooms, niter=niter, pipeline=pipeline)
+    pre_affine = AffineMap(
+        pre_affine, to_shape, to_affine, from_shape, from_affine)
+    return to_shape, zooms, to_affine, pre_affine, sdr_morph
 
 
 def _compute_morph_matrix(subject_from, subject_to, vertices_from, vertices_to,
@@ -1135,18 +1073,29 @@ def _compute_morph_matrix(subject_from, subject_to, vertices_from, vertices_to,
 
 def _hemi_morph(tris, vertices_to, vertices_from, smooth, maps, warn):
     from scipy import sparse
+    _validate_type(smooth, (str, None, 'int-like'), 'smoothing steps')
     if len(vertices_from) == 0:
         return sparse.csr_matrix((len(vertices_to), 0))
     e = mesh_edges(tris)
     e.data[e.data == 2] = 1
     n_vertices = e.shape[0]
-    e = e + sparse.eye(n_vertices)
+    e += sparse.eye(n_vertices, format='csr')
     if isinstance(smooth, str):
         _check_option('smooth', smooth, ('nearest',),
                       extra=' when used as a string.')
         mm = _surf_nearest(vertices_from, e).tocsr()
+    elif smooth == 0:
+        mm = sparse.csc_matrix(
+            (np.ones(len(vertices_from)),  # data, indices, indptr
+                vertices_from,
+                np.arange(len(vertices_from) + 1)),
+            shape=(e.shape[0], len(vertices_from))).tocsr()
     else:
-        mm = _surf_upsampling_mat(vertices_from, e, smooth, warn=warn)
+        mm, n_missing, n_iter = _surf_upsampling_mat(vertices_from, e, smooth)
+        if n_missing and warn:
+            warn_(f'{n_missing}/{e.shape[0]} vertices not included in '
+                  'smoothing, consider increasing the number of steps')
+        logger.info(f'    {n_iter} smooth iterations done.')
     assert mm.shape == (n_vertices, len(vertices_from))
     if maps is not None:
         mm = maps[vertices_to] * mm
@@ -1157,7 +1106,7 @@ def _hemi_morph(tris, vertices_to, vertices_from, smooth, maps, warn):
 
 
 @verbose
-def grade_to_vertices(subject, grade, subjects_dir=None, n_jobs=1,
+def grade_to_vertices(subject, grade, subjects_dir=None, n_jobs=None,
                       verbose=None):
     """Convert a grade to source space vertices for a given subject.
 
@@ -1230,6 +1179,8 @@ def grade_to_vertices(subject, grade, subjects_dir=None, n_jobs=1,
     return vertices
 
 
+# Takes ~20 ms to hash, ~100 ms to compute (5x speedup)
+@_custom_lru_cache(20)
 def _surf_nearest(vertices, adj_mat):
     from scipy import sparse
     from scipy.sparse.csgraph import dijkstra
@@ -1257,7 +1208,12 @@ def _csr_row_norm(data, row_norm):
     data.data /= np.where(row_norm, row_norm, 1).repeat(np.diff(data.indptr))
 
 
-def _surf_upsampling_mat(idx_from, e, smooth, warn=True):
+# upsamplers are generally not very big (< 1 MB), and users might have a lot
+# For 5 smoothing steps for example:
+# smoothing_steps=5 takes ~20 ms to hash, ~100 ms to compute (5x speedup)
+# smoothing_steps=None takes ~20 ms to hash, ~400 ms to compute (20x speedup)
+@_custom_lru_cache(20)
+def _surf_upsampling_mat(idx_from, e, smooth):
     """Upsample data on a subject's surface given mesh edges."""
     # we're in CSR format and it's to==from
     from scipy import sparse
@@ -1270,9 +1226,9 @@ def _surf_upsampling_mat(idx_from, e, smooth, warn=True):
     _validate_type(smooth, ('int-like', str, None), 'smoothing steps')
     if smooth is not None:  # number of steps
         smooth = _ensure_int(smooth, 'smoothing steps')
-        if smooth < 1:
+        if smooth <= 0:  # == 0 is handled in a shortcut above
             raise ValueError(
-                'The number of smoothing operations has to be at least 1, got '
+                'The number of smoothing operations has to be at least 0, got '
                 f'{smooth}')
         smooth = smooth - 1
     # idx will gradually expand from idx_from -> np.arange(n_tot)
@@ -1302,11 +1258,9 @@ def _surf_upsampling_mat(idx_from, e, smooth, warn=True):
         if k == smooth or (smooth is None and len(idx) == n_tot):
             break  # last iteration / done
     assert data.shape == (n_tot, len(idx_from))
-    if len(idx) != n_tot and warn:
-        warn_(f'{n_tot-len(idx)}/{n_tot} vertices not included in smoothing, '
-              'consider increasing the number of steps')
-    logger.info(f'    {k + 1} smooth iterations done.')
-    return data
+    n_missing = n_tot - len(idx)
+    n_iter = k + 1
+    return data, n_missing, n_iter
 
 
 def _sparse_argmax_nnz_row(csr_mat):
