@@ -1,7 +1,5 @@
 import os.path as op
 
-import pytest
-
 from mne import Annotations
 from mne.io import read_raw_fif
 from mne.preprocessing.eog import find_eog_events
@@ -30,10 +28,6 @@ def test_find_eog():
     assert len(events_thr) == 5
 
     # test different ways to specify the EOG channel(s)
-    with pytest.warns(DeprecationWarning, match='pass a list of channels'):
-        events = find_eog_events(raw, ch_name='EEG 060,EOG 061')
-    assert len(events) == 4
-
     events = find_eog_events(raw, ch_name=None)
     assert len(events) == 4
 
