@@ -244,10 +244,11 @@ def test_ica_noop(n_components, n_pca_components, tmp_path):
     # and with I/O
     fname = tmp_path / 'temp-ica.fif'
     ica.save(fname)
-    ica = read_ica(fname)
-    raw_new = ica.apply(raw.copy())
+    ica_new = read_ica(fname)
+    raw_new = ica_new.apply(raw.copy())
     assert_allclose(raw.get_data(), raw_new.get_data(), err_msg='I/O failure')
-    _assert_ica_attributes(ica)
+    _assert_ica_attributes(ica_new)
+    assert ica.reject_ == ica_new.reject_
 
 
 @requires_sklearn
@@ -415,15 +416,20 @@ def test_ica_reset(method):
         'pca_mean_',
         'n_iter_'
     )
+
+    ica = ICA(n_components=3, method=method, max_iter=1)
+    assert ica.current_fit == 'unfitted'
     with pytest.warns(UserWarning, match='did not converge'):
-        ica = ICA(
-            n_components=3, method=method, max_iter=1).fit(raw, picks=picks)
+        ica.fit(raw, picks=picks)
 
     assert (all(hasattr(ica, attr) for attr in run_time_attrs))
     assert ica.labels_ is not None
+    assert ica.current_fit == 'raw'
+
     ica._reset()
     assert (not any(hasattr(ica, attr) for attr in run_time_attrs))
     assert ica.labels_ is not None
+    assert ica.current_fit == 'unfitted'
 
 
 @requires_sklearn
@@ -485,8 +491,6 @@ def test_ica_core(method, n_components, noise_cov, n_pca_components,
     assert 'raw data decomposition' in repr_
     assert f'{ica.n_components_} ICA components' in repr_
     assert 'Available PCA components' in repr_html_
-    assert 'Explained variance' in repr_html_
-
     assert ('mag' in ica)  # should now work without error
 
     # test re-fit
@@ -944,6 +948,95 @@ def test_ica_additional(method, tmp_path, short_raw_epochs):
     ica.fit(raw_, picks=picks, reject_by_annotation=True)
 
 
+@requires_sklearn
+def test_get_explained_variance_ratio(tmp_path, short_raw_epochs):
+    """Test ICA.get_explained_variance_ratio()."""
+    raw, epochs, _ = short_raw_epochs
+    ica = ICA(max_iter=1)
+
+    # Unfitted ICA should raise an exception
+    with pytest.raises(ValueError, match='ICA must be fitted first'):
+        ica.get_explained_variance_ratio(epochs)
+
+    with pytest.warns(RuntimeWarning, match='were baseline-corrected'):
+        ica.fit(epochs)
+
+    # components = int, ch_type = None
+    explained_var_comp_0 = ica.get_explained_variance_ratio(
+        epochs, components=0
+    )
+    # components = int, ch_type = str
+    explained_var_comp_0_eeg = ica.get_explained_variance_ratio(
+        epochs, components=0, ch_type='eeg'
+    )
+    # components = int, ch_type = list of str
+    explained_var_comp_0_eeg_mag = ica.get_explained_variance_ratio(
+        epochs, components=0, ch_type=['eeg', 'mag']
+    )
+    # components = list of int, single element, ch_type = None
+    explained_var_comp_1 = ica.get_explained_variance_ratio(
+        epochs, components=[1]
+    )
+    # components = list of int, multiple elements, ch_type = None
+    explained_var_comps_01 = ica.get_explained_variance_ratio(
+        epochs, components=[0, 1]
+    )
+    # components = None, i.e., all components, ch_type = None
+    explained_var_comps_all = ica.get_explained_variance_ratio(
+        epochs, components=None
+    )
+
+    assert 'grad' in explained_var_comp_0
+    assert 'mag' in explained_var_comp_0
+    assert 'eeg' in explained_var_comp_0
+
+    assert len(explained_var_comp_0_eeg) == 1
+    assert 'eeg' in explained_var_comp_0_eeg
+
+    assert 'mag' in explained_var_comp_0_eeg_mag
+    assert 'eeg' in explained_var_comp_0_eeg_mag
+    assert 'grad' not in explained_var_comp_0_eeg_mag
+
+    assert round(explained_var_comp_0['grad'], 4) == 0.1784
+    assert round(explained_var_comp_0['mag'], 4) == 0.0259
+    assert round(explained_var_comp_0['eeg'], 4) == 0.0229
+
+    assert np.isclose(
+        explained_var_comp_0['eeg'],
+        explained_var_comp_0_eeg['eeg']
+    )
+    assert np.isclose(
+        explained_var_comp_0['mag'],
+        explained_var_comp_0_eeg_mag['mag']
+    )
+    assert np.isclose(
+        explained_var_comp_0['eeg'],
+        explained_var_comp_0_eeg_mag['eeg']
+    )
+
+    assert round(explained_var_comp_1['eeg'], 4) == 0.0231
+    assert round(explained_var_comps_01['eeg'], 4) == 0.0459
+    assert (
+        explained_var_comps_all['grad'] ==
+        explained_var_comps_all['mag'] ==
+        explained_var_comps_all['eeg'] ==
+        1
+    )
+
+    # Test Raw
+    ica.get_explained_variance_ratio(raw)
+    # Test Evoked
+    evoked = epochs.average()
+    ica.get_explained_variance_ratio(evoked)
+    # Test Evoked without baseline correction
+    evoked.baseline = None
+    ica.get_explained_variance_ratio(evoked)
+
+    # Test invalid ch_type
+    with pytest.raises(ValueError, match='only the following channel types'):
+        ica.get_explained_variance_ratio(raw, ch_type='foobar')
+
+
 @requires_sklearn
 @pytest.mark.slowtest
 @pytest.mark.parametrize('method, cov', [
@@ -1084,7 +1177,7 @@ def test_fit_methods(method, tmp_path):
         ('flat', dict(eeg=1e-6))
     )
 )
-def test_fit_params_epochs_vs_raw(param_name, param_val):
+def test_fit_params_epochs_vs_raw(param_name, param_val, tmp_path):
     """Check that we get a warning when passing parameters that get ignored."""
     method = 'infomax'
     n_components = 3
@@ -1092,12 +1185,20 @@ def test_fit_params_epochs_vs_raw(param_name, param_val):
 
     raw = read_raw_fif(raw_fname).pick_types(meg=False, eeg=True)
     events = read_events(event_name)
-    epochs = Epochs(raw, events=events)
+    reject = param_val if param_name == 'reject' else None
+    epochs = Epochs(raw, events=events, reject=reject)
     ica = ICA(n_components=n_components, max_iter=max_iter, method=method)
 
     fit_params = {param_name: param_val}
     with pytest.warns(RuntimeWarning, match='parameters.*will be ignored'):
         ica.fit(inst=epochs, **fit_params)
+    assert ica.reject_ == reject
+    _assert_ica_attributes(ica)
+    tmp_fname = tmp_path / 'test-ica.fif'
+    ica.save(tmp_fname)
+    ica = read_ica(tmp_fname)
+    assert ica.reject_ == reject
+    _assert_ica_attributes(ica)
 
 
 @requires_sklearn
@@ -1327,6 +1428,20 @@ def test_ica_labels():
     assert ica.labels_['muscle'] == [0]
     assert_allclose(scores, [0.56, 0.01, 0.03, 0.00], atol=0.03)
 
+    events = np.array([[6000, 0, 0], [8000, 0, 0]])
+    epochs = Epochs(raw, events=events, baseline=None, preload=True)
+    # move up threhsold more noise because less data
+    scores = ica.find_bads_muscle(epochs, threshold=0.8)[1]
+    assert 'muscle' in ica.labels_
+    assert ica.labels_['muscle'] == [0]
+    assert_allclose(scores, [0.81, 0.14, 0.37, 0.05], atol=0.03)
+
+    ica = ICA(n_components=4, max_iter=2, method='fastica', allow_ref_meg=True)
+    with pytest.warns(UserWarning, match='did not converge'):
+        ica.fit(raw, picks="eeg")
+    ica.find_bads_muscle(raw)
+    assert 'muscle' in ica.labels_
+
 
 @requires_sklearn
 @testing.requires_testing_data
@@ -1435,6 +1550,7 @@ def test_read_ica_eeglab_mismatch(tmp_path):
     assert 'unmixing_matrix_' in attrs
     assert ica.labels_ == ica_correct.labels_ == {}
     attrs.pop(attrs.index('labels_'))
+    attrs.pop(attrs.index('reject_'))
     for attr in attrs:
         a, b = getattr(ica, attr), getattr(ica_correct, attr)
         assert_allclose(a, b, rtol=1e-12, atol=1e-12, err_msg=attr)
@@ -1484,6 +1600,7 @@ def _assert_ica_attributes(ica, data=None, limits=(1.0, 70)):
         # at least close to normal
         assert norms.min() > limits[0], 'Not roughly unity'
         assert norms.max() < limits[1], 'Not roughly unity'
+    assert hasattr(ica, 'reject_')
 
 
 @pytest.mark.parametrize("ch_type", ["dbs", "seeg"])
