@@ -120,7 +120,7 @@ def test_compute_proj_parallel(short_raw):
     with pytest.warns(RuntimeWarning, match='Attenuation'):
         projs, _ = compute_proj_eog(
             raw, n_eeg=2, bads=raw.ch_names[1:2], average=False,
-            avg_ref=True, no_proj=False, n_jobs=1, l_freq=None, h_freq=None,
+            avg_ref=True, no_proj=False, n_jobs=None, l_freq=None, h_freq=None,
             reject=None, tmax=dur_use, filter_length=100)
     raw_2 = short_raw.copy()
     with pytest.warns(RuntimeWarning, match='Attenuation'):
