@@ -6,14 +6,15 @@
 #          Samuel Deslauriers-Gauthier <sam.deslauriers@gmail.com>
 #          Ivana Kojcic <ivana.kojcic@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import numpy as np
 
 from ..source_estimate import SourceEstimate, VolSourceEstimate
 from ..source_space import _ensure_src
 from ..fixes import rng_uniform
-from ..utils import check_random_state, warn, _check_option, fill_doc
+from ..utils import (check_random_state, warn, _check_option, fill_doc,
+                     _ensure_int, _ensure_events)
 from ..label import Label
 from ..surface import _compute_nearest
 
@@ -26,9 +27,9 @@ def select_source_in_label(src, label, random_state=None, location='random',
     Parameters
     ----------
     src : list of dict
-        The source space
+        The source space.
     label : Label
-        the label (read with mne.read_label)
+        The label.
     %(random_state)s
     location : str
         The label location to choose. Can be 'random' (default) or 'center'
@@ -37,20 +38,14 @@ def select_source_in_label(src, label, random_state=None, location='random',
         mode the label values are used as weights.
 
         .. versionadded:: 0.13
-
-    subject : string | None
+    subject : str | None
         The subject the label is defined for.
         Only used with ``location='center'``.
 
         .. versionadded:: 0.13
-
-    subjects_dir : str, or None
-        Path to the SUBJECTS_DIR. If None, the path is obtained by using
-        the environment variable SUBJECTS_DIR.
-        Only used with ``location='center'``.
+    %(subjects_dir)s
 
         .. versionadded:: 0.13
-
     surf : str
         The surface to use for Euclidean distance center of mass
         finding. The default here is "sphere", which finds the center
@@ -62,9 +57,9 @@ def select_source_in_label(src, label, random_state=None, location='random',
     Returns
     -------
     lh_vertno : list
-        selected source coefficients on the left hemisphere
+        Selected source coefficients on the left hemisphere.
     rh_vertno : list
-        selected source coefficients on the right hemisphere
+        Selected source coefficients on the right hemisphere.
     """
     lh_vertno = list()
     rh_vertno = list()
@@ -107,7 +102,7 @@ def simulate_sparse_stc(src, n_dipoles, times,
     n_dipoles : int
         Number of dipoles to simulate.
     times : array
-        Time array
+        Time array.
     data_fun : callable
         Function to generate the waveforms. The default is a 100 nAm, 10 Hz
         sinusoid as ``1e-7 * np.sin(20 * pi * t)``. The function should take
@@ -122,20 +117,14 @@ def simulate_sparse_stc(src, n_dipoles, times,
         mode the label values are used as weights.
 
         .. versionadded:: 0.13
-
-    subject : string | None
+    subject : str | None
         The subject the label is defined for.
         Only used with ``location='center'``.
 
         .. versionadded:: 0.13
-
-    subjects_dir : str, or None
-        Path to the SUBJECTS_DIR. If None, the path is obtained by using
-        the environment variable SUBJECTS_DIR.
-        Only used with ``location='center'``.
+    %(subjects_dir)s
 
         .. versionadded:: 0.13
-
     surf : str
         The surface to use for Euclidean distance center of mass
         finding. The default here is "sphere", which finds the center
@@ -161,7 +150,7 @@ def simulate_sparse_stc(src, n_dipoles, times,
     """
     rng = check_random_state(random_state)
     src = _ensure_src(src, verbose=False)
-    subject_src = src[0].get('subject_his_id')
+    subject_src = src._subject
     if subject is None:
         subject = subject_src
     elif subject_src is not None and subject != subject_src:
@@ -230,21 +219,21 @@ def simulate_stc(src, labels, stc_data, tmin, tstep, value_fun=None,
     Parameters
     ----------
     src : instance of SourceSpaces
-        The source space
+        The source space.
     labels : list of Label
-        The labels
+        The labels.
     stc_data : array, shape (n_labels, n_times)
-        The waveforms
+        The waveforms.
     tmin : float
-        The beginning of the timeseries
+        The beginning of the timeseries.
     tstep : float
-        The time step (1 / sampling frequency)
+        The time step (1 / sampling frequency).
     value_fun : callable | None
         Function to apply to the label values to obtain the waveform
         scaling for each vertex in the label. If None (default), uniform
         scaling is used.
     allow_overlap : bool
-        Allow overlapping labels or not. Default value is False
+        Allow overlapping labels or not. Default value is False.
 
         .. versionadded:: 0.18
 
@@ -322,9 +311,8 @@ def simulate_stc(src, labels, stc_data, tmin, tstep, value_fun=None,
             data.append(stc_data_extended[i][idx])
             vertno[i] = vertno[i][idx]
 
-    subject = src[0].get('subject_his_id')
     stc = SourceEstimate(np.concatenate(data), vertices=vertno, tmin=tmin,
-                         tstep=tstep, subject=subject)
+                         tstep=tstep, subject=src._subject)
     return stc
 
 
@@ -340,6 +328,10 @@ class SourceSimulator(object):
     duration : float | None
         Time interval during which the simulation takes place in seconds.
         If None, it is computed using existing events and waveform lengths.
+    first_samp : int
+        First sample from which the simulation takes place, as an integer.
+        Comparable to the :term:`first_samp` property of `~mne.io.Raw` objects.
+        Default is 0.
 
     Attributes
     ----------
@@ -349,28 +341,39 @@ class SourceSimulator(object):
         The number of time samples of the simulation.
     """
 
-    def __init__(self, src, tstep=1e-3, duration=None):
+    def __init__(self, src, tstep=1e-3, duration=None, first_samp=0):
+        if duration is not None and duration < tstep:
+            raise ValueError('duration must be None or >= tstep.')
+        self.first_samp = _ensure_int(first_samp, 'first_samp')
         self._src = src
         self._tstep = tstep
         self._labels = []
         self._waveforms = []
         self._events = np.empty((0, 3), dtype=int)
-        self._duration = duration
+        self._duration = duration  # if not None, sets # samples
         self._last_samples = []
         self._chk_duration = 1000
 
     @property
     def duration(self):
-        """Duration of the simulation"""
-        # If not, the precomputed maximum last sample is used
-        if self._duration is None:
-            return np.max(self._last_samples) * self._tstep
-        return self._duration
+        """Duration of the simulation in same units as tstep."""
+        if self._duration is not None:
+            return self._duration
+        return self.n_times * self._tstep
 
     @property
     def n_times(self):
-        """Number of time samples in the simulation"""
-        return int(self.duration / self._tstep)
+        """Number of time samples in the simulation."""
+        if self._duration is not None:
+            return int(self._duration / self._tstep)
+        ls = self.first_samp
+        if len(self._last_samples) > 0:
+            ls = np.max(self._last_samples)
+        return ls - self.first_samp + 1  # >= 1
+
+    @property
+    def last_samp(self):
+        return self.first_samp + self.n_times - 1
 
     def add_data(self, label, waveform, events):
         """Add data to the simulation.
@@ -380,14 +383,14 @@ class SourceSimulator(object):
 
         Parameters
         ----------
-        label : Label
+        label : instance of Label
             The label (as created for example by mne.read_label). If the label
             does not match any sources in the SourceEstimate, a ValueError is
             raised.
         waveform : array, shape (n_times,) or (n_events, n_times) | list
             The waveform(s) describing the activity on the label vertices.
-            If list, it must have the same length as events
-        events: array of int, shape (n_events, 3)
+            If list, it must have the same length as events.
+        events : array of int, shape (n_events, 3)
             Events associated to the waveform(s) to specify when the activity
             should occur.
         """
@@ -407,42 +410,48 @@ class SourceSimulator(object):
             raise ValueError('Number of waveforms and events should match or '
                              'there should be a single waveform (%d != %d).' %
                              (len(waveform), len(events)))
-        # Update the maximum duration possible based on the events
+        events = _ensure_events(events).astype(np.int64)
+        # Update the last sample possible based on events + waveforms
         self._labels.extend([label] * len(events))
         self._waveforms.extend(waveform)
-        self._events = np.vstack([self._events, events])
+        self._events = np.concatenate([self._events, events])
+        assert self._events.dtype == np.int64
         # First sample per waveform is the first column of events
         # Last is computed below
-        self._last_samples = np.array([self._events[i, 0] + len(w)
+        self._last_samples = np.array([self._events[i, 0] + len(w) - 1
                                       for i, w in enumerate(self._waveforms)])
 
     def get_stim_channel(self, start_sample=0, stop_sample=None):
         """Get the stim channel from the provided data.
 
         Returns the stim channel data according to the simulation parameters
-        which should be added through function add_data. If both start_sample
+        which should be added through the add_data method. If both start_sample
         and stop_sample are not specified, the entire duration is used.
 
         Parameters
         ----------
         start_sample : int
-            First sample in chunk. Default is 0.
+            First sample in chunk. Default is the value of the ``first_samp``
+            attribute.
         stop_sample : int | None
-            The stop sample of the returned stc. This sample is not part of the
-            output to follow slicing semantics. If None, then all samples past
-            start_sample is returned.
+            The final sample of the returned stc. If None, then all samples
+            from start_sample onward are returned.
 
         Returns
         -------
         stim_data : ndarray of int, shape (n_samples,)
             The stimulation channel data.
         """
+        if start_sample is None:
+            start_sample = self.first_samp
         if stop_sample is None:
-            stop_sample = self.n_times
-        n_samples = stop_sample - start_sample
+            stop_sample = start_sample + self.n_times - 1
+        elif stop_sample < start_sample:
+            raise ValueError('Argument start_sample must be >= stop_sample.')
+        n_samples = stop_sample - start_sample + 1
 
         # Initialize the stim data array
-        stim_data = np.zeros(n_samples, dtype=int)
+        stim_data = np.zeros(n_samples, dtype=np.int64)
 
         # Select only events in the time chunk
         stim_ind = np.where(np.logical_and(
@@ -450,12 +459,12 @@ class SourceSimulator(object):
             self._events[:, 0] < stop_sample))[0]
 
         if len(stim_ind) > 0:
-            relative_ind = self._events[stim_ind, 0].astype(int) - start_sample
+            relative_ind = self._events[stim_ind, 0] - start_sample
             stim_data[relative_ind] = self._events[stim_ind, 2]
 
         return stim_data
 
-    def get_stc(self, start_sample=0, stop_sample=None):
+    def get_stc(self, start_sample=None, stop_sample=None):
         """Simulate a SourceEstimate from the provided data.
 
         Returns a SourceEstimate object constructed according to the simulation
@@ -465,12 +474,12 @@ class SourceSimulator(object):
 
         Parameters
         ----------
-        start_sample : int
-            First sample in chunk. Default is 0.
+        start_sample : int | None
+            First sample in chunk. If ``None`` the value of the ``first_samp``
+            attribute is used. Defaults to ``None``.
         stop_sample : int | None
-            The stop sample of the returned stc. This sample is not part of the
-            output to follow slicing semantics. If None, then all samples past
-            start_sample is returned.
+            The final sample of the returned STC. If ``None``, then all samples
+            past ``start_sample`` are returned.
 
         Returns
         -------
@@ -480,19 +489,22 @@ class SourceSimulator(object):
         if len(self._labels) == 0:
             raise ValueError('No simulation parameters were found. Please use '
                              'function add_data to add simulation parameters.')
+        if start_sample is None:
+            start_sample = self.first_samp
         if stop_sample is None:
-            stop_sample = self.n_times
+            stop_sample = start_sample + self.n_times - 1
+        elif stop_sample < start_sample:
+            raise ValueError('start_sample must be >= stop_sample.')
+        n_samples = stop_sample - start_sample + 1
 
-        n_samples = stop_sample - start_sample
-
-        # Initialize the stc_data array
+        # Initialize the stc_data array to span all possible samples
         stc_data = np.zeros((len(self._labels), n_samples))
 
-        # Select only the indices that have events in the time chunk
+        # Select only the events that fall within the span
         ind = np.where(np.logical_and(self._last_samples >= start_sample,
-                                      self._events[:, 0] < stop_sample))[0]
+                                      self._events[:, 0] <= stop_sample))[0]
 
-        # Loop only over the items that are in the time chunk
+        # Loop only over the items that are in the time span
         subset_waveforms = [self._waveforms[i] for i in ind]
         for i, (waveform, event) in enumerate(zip(subset_waveforms,
                                                   self._events[ind])):
@@ -502,16 +514,17 @@ class SourceSimulator(object):
             wf_stop = self._last_samples[ind[i]]
 
             # Recover the indices of the event that should be in the chunk
-            waveform_ind = np.in1d(np.arange(wf_start, wf_stop),
-                                   np.arange(start_sample, stop_sample))
+            waveform_ind = np.in1d(np.arange(wf_start, wf_stop + 1),
+                                   np.arange(start_sample, stop_sample + 1))
 
             # Recover the indices that correspond to the overlap
-            stc_ind = np.in1d(np.arange(start_sample, stop_sample),
-                              np.arange(wf_start, wf_stop))
+            stc_ind = np.in1d(np.arange(start_sample, stop_sample + 1),
+                              np.arange(wf_start, wf_stop + 1))
 
             # add the resulting waveform chunk to the corresponding label
             stc_data[ind[i]][stc_ind] += waveform[waveform_ind]
 
+        start_sample -= self.first_samp  # STC sample ref is 0
         stc = simulate_stc(self._src, self._labels, stc_data,
                            start_sample * self._tstep, self._tstep,
                            allow_overlap=True)
@@ -520,11 +533,13 @@ class SourceSimulator(object):
 
     def __iter__(self):
         """Iterate over 1 second STCs."""
-        # Arbitrary chunk size, can be modified later to something else
+        # Arbitrary chunk size, can be modified later to something else.
         # Loop over chunks of 1 second - or, maximum sample size.
         # Can be modified to a different value.
-        n_times = self.n_times
-        for start_sample in range(0, n_times, self._chk_duration):
-            stop_sample = min(start_sample + self._chk_duration, n_times)
+        last_sample = self.last_samp
+        for start_sample in range(self.first_samp, last_sample + 1,
+                                  self._chk_duration):
+            stop_sample = min(start_sample + self._chk_duration - 1,
+                              last_sample)
             yield (self.get_stc(start_sample, stop_sample),
                    self.get_stim_channel(start_sample, stop_sample))
