@@ -13,7 +13,7 @@
 import numpy as np
 
 from .parametric import f_oneway, ttest_1samp_no_p
-from ..parallel import parallel_func, check_n_jobs
+from ..parallel import parallel_func
 from ..fixes import jit, has_numba
 from ..utils import (split_list, logger, verbose, ProgressBar, warn, _pl,
                      check_random_state, _check_option, _validate_type)
@@ -562,6 +562,8 @@ def _cluster_mask_to_indices(components, shape):
     for ci, c in enumerate(components):
         if isinstance(c, np.ndarray):  # mask
             components[ci] = np.where(c.reshape(shape))
+        elif isinstance(c, slice):
+            components[ci] = np.arange(c.start, c.stop)
         else:
             assert isinstance(c, tuple), type(c)
             c = list(c)  # tuple->list
@@ -831,7 +833,6 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
                               adjacency, n_jobs, seed, max_step,
                               exclude, step_down_p, t_power, out_type,
                               check_disjoint, buffer_size):
-    n_jobs = check_n_jobs(n_jobs)
     """Aux Function.
 
     Note. X is required to be a list. Depending on the length of X
@@ -904,7 +905,7 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
         partitions = _get_partitions_from_adjacency(adjacency, n_times)
     else:
         partitions = None
-    logger.info('Running initial clustering')
+    logger.info('Running initial clustering …')
     out = _find_clusters(t_obs, threshold, tail, adjacency,
                          max_step=max_step, include=include,
                          partitions=partitions, t_power=t_power,
@@ -918,7 +919,7 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
     if isinstance(threshold, dict):
         t_obs = cluster_stats.reshape(t_obs.shape) * np.sign(t_obs)
 
-    logger.info('Found %d clusters' % len(clusters))
+    logger.info(f'Found {len(clusters)} cluster{_pl(clusters)}')
 
     # convert clusters to old format
     if adjacency is not None and adjacency is not False:
@@ -953,7 +954,7 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
         orders = [rng.permutation(len(X_full))
                   for _ in range(n_permutations - 1)]
     del rng
-    parallel, my_do_perm_func, _ = parallel_func(
+    parallel, my_do_perm_func, n_jobs = parallel_func(
         do_perm_func, n_jobs, verbose=False)
 
     if len(clusters) == 0:
@@ -977,8 +978,11 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
                 this_include = include
         else:
             this_include = step_down_include
-        logger.info('Permuting %d times%s...' % (len(orders), extra))
-        with ProgressBar(len(orders)) as progress_bar:
+
+        with ProgressBar(
+            iterable=range(len(orders)),
+            mesg=f'Permuting{extra}'
+        ) as progress_bar:
             H0 = parallel(
                 my_do_perm_func(X_full, slices, threshold, tail, adjacency,
                                 stat_fun, max_step, this_include, partitions,
@@ -994,7 +998,7 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
             orig = abs(cluster_stats).max()
         H0.insert(0, [orig])
         H0 = np.concatenate(H0)
-        logger.info('Computing cluster p-values')
+        logger.debug('Computing cluster p-values')
         cluster_pv = _pval_from_histogram(cluster_stats, H0, tail)
 
         # figure out how many new ones will be removed for step-down
@@ -1013,7 +1017,7 @@ def _permutation_cluster_test(X, threshold, n_permutations, tail, stat_fun,
                         'cluster%s to exclude from subsequent iterations'
                         % (n_step_downs, n_removed, a_text,
                            _pl(n_removed)))
-    logger.info('Done.')
+
     # The clusters should have the same shape as the samples
     clusters = _reshape_clusters(clusters, sample_shape)
     return t_obs, clusters, cluster_pv, H0
@@ -1056,7 +1060,7 @@ def _check_fun(X, stat_fun, threshold, tail=0, kind='within'):
 @verbose
 def permutation_cluster_test(
         X, threshold=None, n_permutations=1024, tail=0, stat_fun=None,
-        adjacency=None, n_jobs=1, seed=None, max_step=1, exclude=None,
+        adjacency=None, n_jobs=None, seed=None, max_step=1, exclude=None,
         step_down_p=0, t_power=1, out_type='indices', check_disjoint=False,
         buffer_size=1000, verbose=None):
     """Cluster-level statistical permutation test.
@@ -1066,12 +1070,12 @@ def permutation_cluster_test(
     permutations and cluster-level correction. Each element of the list ``X``
     should contain the data for one group of observations (e.g., 2D arrays for
     time series, 3D arrays for time-frequency power values). Permutations are
-    generated with random partitions of the data. See
-    :footcite:`MarisOostenveld2007` for details.
+    generated with random partitions of the data. For details, see
+    :footcite:p:`MarisOostenveld2007,Sassenhagen2019`.
 
     Parameters
     ----------
-    X : list of array, shape (n_observations, p[, q])
+    X : list of array, shape (n_observations, p[, q][, r])
         The data to be clustered. Each array in ``X`` should contain the
         observations for one group. The first dimension of each array is the
         number of observations from that group; remaining dimensions comprise
@@ -1083,28 +1087,25 @@ def permutation_cluster_test(
         dimension represented in the ``adjacency`` parameter
         (e.g., spectral data should be provided as
         ``(observations, frequencies, channels/vertices)``).
-    %(clust_thresh_f)s
-    %(clust_nperm_int)s
-    %(clust_tail)s
-    %(clust_stat_f)s
-    %(clust_adj_n)s
+    %(threshold_clust_f)s
+    %(n_permutations_clust_int)s
+    %(tail_clust)s
+    %(stat_fun_clust_f)s
+    %(adjacency_clust_n)s
     %(n_jobs)s
     %(seed)s
-    %(clust_maxstep)s
-    exclude : bool array or None
-        Mask to apply to the data to exclude certain points from clustering
-        (e.g., medial wall vertices). Should be the same shape as X. If None,
-        no points are excluded.
-    %(clust_stepdown)s
-    %(clust_power_f)s
-    %(clust_out)s
-    %(clust_disjoint)s
-    %(clust_buffer)s
+    %(max_step_clust)s
+    %(exclude_clust)s
+    %(step_down_p_clust)s
+    %(f_power_clust)s
+    %(out_type_clust)s
+    %(check_disjoint_clust)s
+    %(buffer_size_clust)s
     %(verbose)s
 
     Returns
     -------
-    F_obs : array, shape (n_tests,)
+    F_obs : array, shape (p[, q][, r])
         Statistic (F by default) observed for all variables.
     clusters : list
         List type defined by out_type above.
@@ -1113,6 +1114,10 @@ def permutation_cluster_test(
     H0 : array, shape (n_permutations,)
         Max cluster level stats observed under permutation.
 
+    Notes
+    -----
+    %(threshold_clust_f_notes)s
+
     References
     ----------
     .. footbibliography::
@@ -1129,42 +1134,41 @@ def permutation_cluster_test(
 @verbose
 def permutation_cluster_1samp_test(
         X, threshold=None, n_permutations=1024, tail=0, stat_fun=None,
-        adjacency=None, n_jobs=1, seed=None, max_step=1,
+        adjacency=None, n_jobs=None, seed=None, max_step=1,
         exclude=None, step_down_p=0, t_power=1, out_type='indices',
         check_disjoint=False, buffer_size=1000, verbose=None):
     """Non-parametric cluster-level paired t-test.
 
+    For details, see :footcite:p:`MarisOostenveld2007,Sassenhagen2019`.
+
     Parameters
     ----------
-    X : array, shape (n_observations, p[, q])
+    X : array, shape (n_observations, p[, q][, r])
         The data to be clustered. The first dimension should correspond to the
         difference between paired samples (observations) in two conditions.
-        The subarrays ``X[k]`` can be 1D (e.g., time series) or 2D (e.g.,
-        time-frequency image) associated with the kth observation. For
-        spatiotemporal data, see also
-        :func:`mne.stats.spatio_temporal_cluster_1samp_test`.
-    %(clust_thresh_t)s
-    %(clust_nperm_all)s
-    %(clust_tail)s
-    %(clust_stat_t)s
-    %(clust_adj_1)s
+        The subarrays ``X[k]`` can be 1D (e.g., time series), 2D (e.g.,
+        time series over channels), or 3D (e.g., time-frequencies over
+        channels) associated with the kth observation. For spatiotemporal data,
+        see also :func:`mne.stats.spatio_temporal_cluster_1samp_test`.
+    %(threshold_clust_t)s
+    %(n_permutations_clust_all)s
+    %(tail_clust)s
+    %(stat_fun_clust_t)s
+    %(adjacency_clust_1)s
     %(n_jobs)s
     %(seed)s
-    %(clust_maxstep)s
-    exclude : bool array or None
-        Mask to apply to the data to exclude certain points from clustering
-        (e.g., medial wall vertices). Should be the same shape as X. If None,
-        no points are excluded.
-    %(clust_stepdown)s
-    %(clust_power_t)s
-    %(clust_out)s
-    %(clust_disjoint)s
-    %(clust_buffer)s
+    %(max_step_clust)s
+    %(exclude_clust)s
+    %(step_down_p_clust)s
+    %(t_power_clust)s
+    %(out_type_clust)s
+    %(check_disjoint_clust)s
+    %(buffer_size_clust)s
     %(verbose)s
 
     Returns
     -------
-    t_obs : array, shape (n_tests,)
+    t_obs : array, shape (p[, q][, r])
         T-statistic observed for all variables.
     clusters : list
         List type defined by out_type above.
@@ -1188,12 +1192,15 @@ def permutation_cluster_1samp_test(
     computes a 1-sample t-test (by default) and uses sign flipping (always)
     to perform permutations. This might not be suitable for the case where
     there is truly a single observation under test; see :ref:`disc-stats`.
+    %(threshold_clust_t_notes)s
 
-    If ``n_permutations >= 2 ** (n_samples - (tail == 0))``,
-    ``n_permutations`` and ``seed`` will be ignored since an exact test
-    (full permutation test) will be performed.
+    If ``n_permutations`` exceeds the maximum number of possible permutations
+    given the number of observations, then ``n_permutations`` and ``seed``
+    will be ignored since an exact test (full permutation test) will be
+    performed (this is the case when
+    ``n_permutations >= 2 ** (n_observations - (tail == 0))``).
 
-    If no initial clusters are found, i.e., all points in the true
+    If no initial clusters are found because all points in the true
     distribution are below the threshold, then ``clusters``, ``cluster_pv``,
     and ``H0`` will all be empty arrays.
 
@@ -1213,7 +1220,7 @@ def permutation_cluster_1samp_test(
 @verbose
 def spatio_temporal_cluster_1samp_test(
         X, threshold=None, n_permutations=1024, tail=0,
-        stat_fun=None, adjacency=None, n_jobs=1, seed=None,
+        stat_fun=None, adjacency=None, n_jobs=None, seed=None,
         max_step=1, spatial_exclude=None, step_down_p=0, t_power=1,
         out_type='indices', check_disjoint=False, buffer_size=1000,
         verbose=None):
@@ -1221,34 +1228,38 @@ def spatio_temporal_cluster_1samp_test(
 
     This function provides a convenient wrapper for
     :func:`mne.stats.permutation_cluster_1samp_test`, for use with data
-    organized in the form (observations × time × space). See
-    :footcite:`MarisOostenveld2007` for details.
+    organized in the form (observations × time × space),
+    (observations × frequencies × space), or optionally
+    (observations × time × frequencies × space). For details, see
+    :footcite:p:`MarisOostenveld2007,Sassenhagen2019`.
 
     Parameters
     ----------
-    X : array, shape (n_observations, n_times, n_vertices)
+    X : array, shape (n_observations, p[, q], n_vertices)
         The data to be clustered. The first dimension should correspond to the
         difference between paired samples (observations) in two conditions.
-    %(clust_thresh_t)s
-    %(clust_nperm_all)s
-    %(clust_tail)s
-    %(clust_stat_t)s
-    %(clust_adj_st1)s
+        The second, and optionally third, dimensions correspond to the
+        time or time-frequency data. And, the last dimension should be spatial.
+    %(threshold_clust_t)s
+    %(n_permutations_clust_all)s
+    %(tail_clust)s
+    %(stat_fun_clust_t)s
+    %(adjacency_clust_st1)s
     %(n_jobs)s
     %(seed)s
-    %(clust_maxstep)s
+    %(max_step_clust)s
     spatial_exclude : list of int or None
         List of spatial indices to exclude from clustering.
-    %(clust_stepdown)s
-    %(clust_power_t)s
-    %(clust_out)s
-    %(clust_disjoint)s
-    %(clust_buffer)s
+    %(step_down_p_clust)s
+    %(t_power_clust)s
+    %(out_type_clust)s
+    %(check_disjoint_clust)s
+    %(buffer_size_clust)s
     %(verbose)s
 
     Returns
     -------
-    t_obs : array, shape (n_times * n_vertices,)
+    t_obs : array, shape (p[, q], n_vertices)
         T-statistic observed for all variables.
     clusters : list
         List type defined by out_type above.
@@ -1257,15 +1268,18 @@ def spatio_temporal_cluster_1samp_test(
     H0 : array, shape (n_permutations,)
         Max cluster level stats observed under permutation.
 
+    Notes
+    -----
+    %(threshold_clust_t_notes)s
+
     References
     ----------
     .. footbibliography::
     """
-    n_samples, n_times, n_vertices = X.shape
     # convert spatial_exclude before passing on if necessary
     if spatial_exclude is not None:
-        exclude = _st_mask_from_s_inds(n_times, n_vertices,
-                                       spatial_exclude, True)
+        exclude = _st_mask_from_s_inds(
+            np.prod(X.shape[1:-1]), X.shape[-1], spatial_exclude, True)
     else:
         exclude = None
     return permutation_cluster_1samp_test(
@@ -1279,7 +1293,7 @@ def spatio_temporal_cluster_1samp_test(
 @verbose
 def spatio_temporal_cluster_test(
         X, threshold=None, n_permutations=1024, tail=0, stat_fun=None,
-        adjacency=None, n_jobs=1, seed=None, max_step=1,
+        adjacency=None, n_jobs=None, seed=None, max_step=1,
         spatial_exclude=None, step_down_p=0, t_power=1, out_type='indices',
         check_disjoint=False, buffer_size=1000,
         verbose=None):
@@ -1287,38 +1301,41 @@ def spatio_temporal_cluster_test(
 
     This function provides a convenient wrapper for
     :func:`mne.stats.permutation_cluster_test`, for use with data
-    organized in the form (observations × time × space).
-    See :footcite:`MarisOostenveld2007` for more information.
+    organized in the form (observations × time × space),
+    (observations × time × space), or optionally
+    (observations × time × frequencies × space). For more information,
+    see :footcite:p:`MarisOostenveld2007,Sassenhagen2019`.
 
     Parameters
     ----------
-    X : list of array, shape (n_observations, n_times, n_vertices)
+    X : list of array, shape (n_observations, p[, q], n_vertices)
         The data to be clustered. Each array in ``X`` should contain the
         observations for one group. The first dimension of each array is the
-        number of observations from that group (and may vary between groups);
-        the remaining dimensions (times and vertices) should match across all
-        groups.
-    %(clust_thresh_f)s
-    %(clust_nperm_int)s
-    %(clust_tail)s
-    %(clust_stat_f)s
-    %(clust_adj_stn)s
+        number of observations from that group (and may vary between groups).
+        The second, and optionally third, dimensions correspond to the
+        time or time-frequency data. And, the last dimension should be spatial.
+        All dimensions except the first should match across all groups.
+    %(threshold_clust_f)s
+    %(n_permutations_clust_int)s
+    %(tail_clust)s
+    %(stat_fun_clust_f)s
+    %(adjacency_clust_stn)s
     %(n_jobs)s
     %(seed)s
-    %(clust_maxstep)s
+    %(max_step_clust)s
     spatial_exclude : list of int or None
         List of spatial indices to exclude from clustering.
-    %(clust_stepdown)s
-    %(clust_power_f)s
-    %(clust_out)s
-    %(clust_disjoint)s
-    %(clust_buffer)s
+    %(step_down_p_clust)s
+    %(f_power_clust)s
+    %(out_type_clust)s
+    %(check_disjoint_clust)s
+    %(buffer_size_clust)s
     %(verbose)s
 
     Returns
     -------
-    t_obs : array, shape (n_times * n_vertices,)
-        Statistic (t by default) observed for all variables.
+    F_obs : array, shape (p[, q], n_vertices)
+        Statistic (F by default) observed for all variables.
     clusters : list
         List type defined by out_type above.
     cluster_pv: array
@@ -1326,15 +1343,18 @@ def spatio_temporal_cluster_test(
     H0 : array, shape (n_permutations,)
         Max cluster level stats observed under permutation.
 
+    Notes
+    -----
+    %(threshold_clust_f_notes)s
+
     References
     ----------
     .. footbibliography::
     """
-    n_samples, n_times, n_vertices = X[0].shape
     # convert spatial_exclude before passing on if necessary
     if spatial_exclude is not None:
-        exclude = _st_mask_from_s_inds(n_times, n_vertices,
-                                       spatial_exclude, True)
+        exclude = _st_mask_from_s_inds(
+            np.prod(X[0].shape[1:-1]), X[0].shape[-1], spatial_exclude, True)
     else:
         exclude = None
     return permutation_cluster_test(
