@@ -1,7 +1,7 @@
 # Authors: Eric Larson <larson.eric.d@gmail.com>
 #          Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from functools import partial
 import os
@@ -14,14 +14,14 @@ import pytest
 
 from mne import (SourceEstimate, VolSourceEstimate, MixedSourceEstimate,
                  SourceSpaces)
-from mne.parallel import _force_serial
 from mne.stats import ttest_ind_no_p, combine_adjacency
 from mne.stats.cluster_level import (permutation_cluster_test, f_oneway,
                                      permutation_cluster_1samp_test,
                                      spatio_temporal_cluster_test,
                                      spatio_temporal_cluster_1samp_test,
                                      ttest_1samp_no_p, summarize_clusters_stc)
-from mne.utils import catch_logging, check_version, requires_sklearn
+from mne.utils import (catch_logging, check_version, requires_sklearn,
+                       _record_warnings)
 
 
 n_space = 50
@@ -102,9 +102,9 @@ def test_thresholds(numba_conditional):
                 buffer_size=None, out_type='mask')
 
 
-def test_cache_dir(tmpdir, numba_conditional):
+def test_cache_dir(tmp_path, numba_conditional):
     """Test use of cache dir."""
-    tempdir = str(tmpdir)
+    tempdir = str(tmp_path)
     orig_dir = os.getenv('MNE_CACHE_DIR', None)
     orig_size = os.getenv('MNE_MEMMAP_MIN_SIZE', None)
     rng = np.random.RandomState(0)
@@ -249,7 +249,7 @@ def test_cluster_permutation_t_test(numba_conditional, stat_fun):
 
         # test with 2 jobs and buffer_size enabled
         buffer_size = condition1.shape[1] // 10
-        with pytest.warns(None):  # sometimes "independently"
+        with _record_warnings():  # sometimes "independently"
             T_obs_neg_buff, _, cluster_p_values_neg_buff, _ = \
                 permutation_cluster_1samp_test(
                     -condition1, n_permutations=100, tail=-1, out_type='mask',
@@ -271,7 +271,7 @@ def test_cluster_permutation_t_test(numba_conditional, stat_fun):
 
 
 @requires_sklearn
-def test_cluster_permutation_with_adjacency(numba_conditional):
+def test_cluster_permutation_with_adjacency(numba_conditional, monkeypatch):
     """Test cluster level permutations with adjacency matrix."""
     from sklearn.feature_extraction.image import grid_to_graph
     condition1_1d, condition2_1d, condition1_2d, condition2_2d = \
@@ -370,10 +370,11 @@ def test_cluster_permutation_with_adjacency(numba_conditional):
         sums_5 = np.sort(sums_5)
         assert_array_almost_equal(sums_4, sums_5)
 
-        if not _force_serial:
-            pytest.raises(ValueError, spatio_temporal_func, X1d_3,
-                          n_permutations=1, adjacency=adjacency,
-                          max_step=1, threshold=1.67, n_jobs=-1000)
+        monkeypatch.delenv('MNE_FORCE_SERIAL', raising=False)
+        with pytest.raises(ValueError, match='must not be less'):
+            spatio_temporal_func(
+                X1d_3, n_permutations=1, adjacency=adjacency,
+                max_step=1, threshold=1.67, n_jobs=-1000)
 
         # not enough TFCE params
         with pytest.raises(KeyError, match='threshold, if dict, must have'):
@@ -381,7 +382,7 @@ def test_cluster_permutation_with_adjacency(numba_conditional):
                 X1d_3, adjacency=adjacency, threshold=dict(me='hello'))
 
         # too extreme a start threshold
-        with pytest.warns(None) as w:
+        with _record_warnings() as w:
             spatio_temporal_func(X1d_3, adjacency=adjacency,
                                  threshold=dict(start=10, step=1))
         if not did_warn:
@@ -410,7 +411,8 @@ def test_cluster_permutation_with_adjacency(numba_conditional):
             spatio_temporal_func(
                 X1d_3, adjacency=adjacency, threshold=[])
         with pytest.raises(ValueError, match='Invalid value for the \'tail\''):
-            with pytest.warns(None):  # sometimes ignoring tail
+            # sometimes ignoring tail
+            with _record_warnings():
                 spatio_temporal_func(
                     X1d_3, adjacency=adjacency, tail=2)
 
@@ -669,6 +671,10 @@ def test_tfce_thresholds(numba_conditional):
         permutation_cluster_1samp_test(
             data, tail=1, out_type='mask', threshold=dict(start=1, step=-0.5))
 
+    # Should work with 2D data too
+    permutation_cluster_1samp_test(X=data[..., 0],
+                                   threshold=dict(start=0, step=0.2))
+
 
 # 1D gives slices, 2D+ gives boolean masks
 @pytest.mark.parametrize('shape', ((11,), (11, 3), (11, 1, 2)))
