@@ -1,11 +1,12 @@
 # Author: Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import os.path as op
 
 import numpy as np
-from numpy.testing import assert_allclose, assert_array_less
+from numpy.testing import (assert_allclose, assert_array_less,
+                           assert_array_equal)
 from scipy.interpolate import interp1d
 from scipy.spatial.distance import cdist
 import pytest
@@ -16,16 +17,16 @@ from mne.io import (read_raw_fif, read_raw_artemis123, read_raw_ctf, read_info,
                     RawArray, read_raw_kit)
 from mne.io.constants import FIFF
 from mne.chpi import (compute_chpi_amplitudes, compute_chpi_locs,
-                      compute_head_pos, _setup_ext_proj,
+                      compute_chpi_snr, compute_head_pos, _setup_ext_proj,
                       _chpi_locs_to_times_dig, _compute_good_distances,
                       extract_chpi_locs_ctf, head_pos_to_trans_rot_t,
                       read_head_pos, write_head_pos, filter_chpi,
-                      _get_hpi_info, _get_hpi_initial_fit,
+                      get_active_chpi, get_chpi_info, _get_hpi_initial_fit,
                       extract_chpi_locs_kit)
 from mne.datasets import testing
 from mne.simulation import add_chpi
 from mne.transforms import rot_to_quat, _angle_between_quats
-from mne.utils import catch_logging, assert_meg_snr, verbose
+from mne.utils import catch_logging, assert_meg_snr, verbose, object_diff
 from mne.viz import plot_head_positions
 
 base_dir = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data')
@@ -64,7 +65,7 @@ def test_chpi_adjust():
     raw = read_raw_fif(chpi_fif_fname, allow_maxshield='yes')
     with catch_logging() as log:
         _get_hpi_initial_fit(raw.info, adjust=True, verbose='debug')
-        _get_hpi_info(raw.info, verbose='debug')
+        get_chpi_info(raw.info, on_missing='raise', verbose='debug')
     # Ran MaxFilter (with -list, -v, -movecomp, etc.), and got:
     msg = ['HPIFIT: 5 coils digitized in order 5 1 4 3 2',
            'HPIFIT: 3 coils accepted: 1 2 4',
@@ -92,15 +93,15 @@ def test_chpi_adjust():
         'Note: HPI coil 5 isotrak is adjusted by 3.2 mm!'] + msg[-2:]
     with catch_logging() as log:
         _get_hpi_initial_fit(raw.info, adjust=True, verbose='debug')
-        _get_hpi_info(raw.info, verbose='debug')
+        get_chpi_info(raw.info, on_missing='raise', verbose='debug')
     log = log.getvalue().splitlines()
     assert set(log) == set(msg), '\n' + '\n'.join(set(msg) - set(log))
 
 
 @testing.requires_testing_data
-def test_read_write_head_pos(tmpdir):
+def test_read_write_head_pos(tmp_path):
     """Test reading and writing head position quaternion parameters."""
-    temp_name = op.join(str(tmpdir), 'temp.pos')
+    temp_name = op.join(str(tmp_path), 'temp.pos')
     # This isn't a 100% valid quat matrix but it should be okay for tests
     head_pos_rand = np.random.RandomState(0).randn(20, 10)
     # This one is valid
@@ -118,9 +119,9 @@ def test_read_write_head_pos(tmpdir):
 
 
 @testing.requires_testing_data
-def test_hpi_info(tmpdir):
+def test_hpi_info(tmp_path):
     """Test getting HPI info."""
-    temp_name = op.join(str(tmpdir), 'temp_raw.fif')
+    temp_name = op.join(str(tmp_path), 'temp_raw.fif')
     for fname in (chpi_fif_fname, sss_fif_fname):
         raw = read_raw_fif(fname, allow_maxshield='yes').crop(0, 0.1)
         assert len(raw.info['hpi_subsystem']) > 0
@@ -128,6 +129,32 @@ def test_hpi_info(tmpdir):
         info = read_info(temp_name)
         assert len(info['hpi_subsystem']) == len(raw.info['hpi_subsystem'])
 
+    # test get_chpi_info()
+    info = read_info(chpi_fif_fname)
+    hpi_freqs, stim_ch_idx, hpi_on_codes = get_chpi_info(info)
+
+    assert_allclose(hpi_freqs, np.array([83., 143., 203., 263., 323.]))
+    assert stim_ch_idx == 378
+    assert_allclose(hpi_on_codes, np.array([256, 512, 1024, 2048, 4096]))
+
+    # test get_chpi_info() if no proper cHPI info is available
+    with info._unlock():
+        info['hpi_subsystem'] = None
+        info['hpi_meas'] = []
+        info['hpi_results'] = []
+
+    with pytest.raises(ValueError, match='No appropriate cHPI information'):
+        get_chpi_info(info)
+
+    with pytest.warns(RuntimeWarning, match='No appropriate cHPI information'):
+        get_chpi_info(info, on_missing='warn')
+
+    hpi_freqs, stim_ch_idx, hpi_on_codes = get_chpi_info(info,
+                                                         on_missing='ignore')
+    assert_array_equal([], hpi_freqs)
+    assert stim_ch_idx is None
+    assert_array_equal([], hpi_on_codes)
+
 
 def _assert_quats(actual, desired, dist_tol=0.003, angle_tol=5., err_rtol=0.5,
                   gof_rtol=0.001, vel_atol=2e-3):  # 2 mm/s
@@ -194,7 +221,8 @@ def _decimate_chpi(raw, decim=4):
     """Decimate raw data (with aliasing) in cHPI-fitting compatible way."""
     raw_dec = RawArray(
         raw._data[:, ::decim], raw.info, first_samp=raw.first_samp // decim)
-    raw_dec.info['sfreq'] /= decim
+    with raw_dec.info._unlock():
+        raw_dec.info['sfreq'] /= decim
     for coil in raw_dec.info['hpi_meas'][0]['hpi_coils']:
         if coil['coil_freq'] > raw_dec.info['sfreq']:
             coil['coil_freq'] = np.mod(coil['coil_freq'],
@@ -222,6 +250,17 @@ def _calculate_chpi_positions(raw, t_step_min=0.01, t_step_max=1.,
     return head_pos
 
 
+@testing.requires_testing_data
+def test_calculate_chpi_positions_preload():
+    """Test calculation of cHPI positions with and without data loaded."""
+    raw = read_raw_fif(chpi_fif_fname, allow_maxshield='yes').crop(0, 2)
+    kwargs = dict(t_step_min=0.1, t_window='auto', verbose=True)
+    pos = compute_chpi_amplitudes(raw, **kwargs)
+    raw.load_data()
+    pos_preload = compute_chpi_amplitudes(raw, **kwargs)
+    assert object_diff(pos, pos_preload) == ''
+
+
 @pytest.mark.slowtest
 @testing.requires_testing_data
 def test_calculate_chpi_positions_vv():
@@ -249,11 +288,11 @@ def test_calculate_chpi_positions_vv():
     _assert_quats(py_quats, mf_quats, dist_tol=0.001, angle_tol=0.7)
     # degenerate conditions
     raw_no_chpi = read_raw_fif(sample_fname)
-    with pytest.raises(RuntimeError, match='cHPI information not found'):
+    with pytest.raises(ValueError, match='No appropriate cHPI information'):
         _calculate_chpi_positions(raw_no_chpi)
     raw_bad = raw.copy()
     del raw_bad.info['hpi_meas'][0]['hpi_coils'][0]['coil_freq']
-    with pytest.raises(RuntimeError, match='cHPI information not found'):
+    with pytest.raises(ValueError, match='No appropriate cHPI information'):
         _calculate_chpi_positions(raw_bad)
     raw_bad = raw.copy()
     for d in raw_bad.info['dig']:
@@ -277,11 +316,29 @@ def test_calculate_chpi_positions_vv():
     assert '0/5 good HPI fits' in log_file.getvalue()
 
     # half the rate cuts off cHPI coils
-    raw.info['lowpass'] /= 2.
+    with raw.info._unlock():
+        raw.info['lowpass'] /= 2.
     with pytest.raises(RuntimeError, match='above the'):
         _calculate_chpi_positions(raw)
 
 
+@testing.requires_testing_data
+@pytest.mark.slowtest
+def test_calculate_chpi_snr():
+    """Test cHPI SNR calculation."""
+    raw = read_raw_fif(chpi_fif_fname, allow_maxshield='yes')
+    result = compute_chpi_snr(raw)
+    # make sure all the entries are there
+    keys = {f'{ch_type}_{key}' for ch_type in ('mag', 'grad') for key in
+            ('snr', 'power', 'resid')}
+    assert set(result) == keys.union({'times', 'freqs'})
+    # make sure the values are plausible, given the sample data file
+    assert result['mag_snr'].min() > 1
+    assert result['mag_snr'].max() < 40
+    assert result['grad_snr'].min() > 1
+    assert result['grad_snr'].max() < 40
+
+
 @testing.requires_testing_data
 @pytest.mark.slowtest
 def test_calculate_chpi_positions_artemis():
@@ -392,15 +449,17 @@ def test_simulate_calculate_head_pos_chpi():
                                                 dtype=np.int32)}],
                      'ncoil': ncoil}
 
-    info['hpi_subsystem'] = hpi_subsystem
-    for fi, freq in enumerate(coil_freq):
-        info['hpi_meas'][0]['hpi_coils'][fi]['coil_freq'] = freq
+    with info._unlock():
+        info['hpi_subsystem'] = hpi_subsystem
+        for fi, freq in enumerate(coil_freq):
+            info['hpi_meas'][0]['hpi_coils'][fi]['coil_freq'] = freq
+        info['sfreq'] = 100.  # this will speed it up a lot
     picks = pick_types(info, meg=True, stim=True, eeg=False, exclude=[])
-    info['sfreq'] = 100.  # this will speed it up a lot
     info = pick_info(info, picks)
     info['chs'][info['ch_names'].index('STI 001')]['ch_name'] = 'STI201'
     info._update_redundant()
-    info['projs'] = []
+    with info._unlock():
+        info['projs'] = []
 
     info_trans = info['dev_head_t']['trans'].copy()
 
@@ -496,7 +555,9 @@ def test_calculate_chpi_coil_locs_artemis():
     times, cHPI_digs = _calculate_chpi_coil_locs(raw, verbose='debug')
 
     assert len(np.setdiff1d(times, raw.times + raw.first_time)) == 0
-    assert_allclose(times[5], 1.5, atol=1e-3)
+    # Should be somewhere around 1.5 sec, depending on coil GOF values
+    # around 0.98 it can change
+    assert_allclose(times[5], 1.5, atol=2e-1)
     assert_allclose(cHPI_digs[5][0]['gof'], 0.995, atol=5e-3)
     assert_allclose(cHPI_digs[5][0]['r'],
                     [-0.0157, 0.0655, 0.0018], atol=1e-3)
@@ -579,8 +640,9 @@ def test_chpi_subtraction_filter_chpi():
     assert raw.info['line_freq'] is None
     with pytest.raises(RuntimeError, match='line_freq.*consider setting it'):
         filter_chpi(raw, t_window=0.2)
-    raw.info['line_freq'] = 60.
-    with pytest.raises(RuntimeError, match='cHPI information not found'):
+    with raw.info._unlock():
+        raw.info['line_freq'] = 60.
+    with pytest.raises(ValueError, match='No appropriate cHPI information'):
         filter_chpi(raw, t_window=0.2)
     # but this is allowed
     with catch_logging() as log:
@@ -597,10 +659,11 @@ def test_chpi_subtraction_filter_chpi():
     # it can strip out some values of info, which we emulate here:
     raw = read_raw_fif(chpi_fif_fname, allow_maxshield='yes')
     raw = raw.crop(0, 1).load_data().resample(600., npad='auto')
-    raw.info['lowpass'] = 200.
-    del raw.info['maxshield']
-    del raw.info['hpi_results'][0]['moments']
-    del raw.info['hpi_subsystem']['event_channel']
+    with raw.info._unlock():
+        raw.info['lowpass'] = 200.
+        del raw.info['maxshield']
+        del raw.info['hpi_results'][0]['moments']
+        del raw.info['hpi_subsystem']['event_channel']
     with catch_logging() as log:
         filter_chpi(raw, t_window='auto', verbose=True)
     with pytest.raises(ValueError, match='must be > 0'):
@@ -647,3 +710,37 @@ def test_calculate_head_pos_kit():
         extract_chpi_locs_kit(raw_berlin, 'STI 014')
     with pytest.raises(RuntimeError, match='no initial cHPI'):
         compute_head_pos(raw_berlin.info, chpi_locs)
+
+
+@testing.requires_testing_data
+def test_get_active_chpi_ctf():
+    """Test extracting of cHPI positions from CTF data."""
+    raw = read_raw_ctf(ctf_chpi_fname)
+    with pytest.raises(NotImplementedError,
+                       match='not implemented for other systems'):
+        get_active_chpi(raw)
+
+
+@testing.requires_testing_data
+def test_get_active_chpi_neuromag():
+    """Test extracting of cHPI positions from neuromag data."""
+    raw = read_raw_fif(chpi_fif_fname, allow_maxshield='yes', preload=True)
+    status_ch = raw.ch_names.index('STI201')
+
+    # make artificial chpi signal
+    first_three_on = 256 + 512 + 1024
+    all_on = 256 + 512 + 1024 + 2048 + 4096
+    raw._data[status_ch][:1000] = 0
+    raw._data[status_ch][1000:2000] = first_three_on
+    raw._data[status_ch][2000:] = all_on
+
+    # build target signal
+    target_signal = 5 * np.ones_like(raw.times)
+    target_signal[:1000] = 0
+    target_signal[1000:2000] = 3
+
+    assert_allclose(get_active_chpi(raw, on_missing='ignore'), target_signal)
+
+    raw_no_chpi = read_raw_fif(sample_fname)
+    assert_allclose(get_active_chpi(raw_no_chpi, on_missing='ignore'),
+                    np.zeros_like(raw_no_chpi.times))
