@@ -1,10 +1,13 @@
 # Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #         Denis Engemann <denis.engemann@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import os.path as op
 import itertools as itt
+import sys
+
+from inspect import signature
 
 from numpy.testing import (assert_array_almost_equal, assert_array_equal,
                            assert_equal, assert_allclose)
@@ -24,12 +27,12 @@ from mne import (read_cov, write_cov, Epochs, merge_events,
                  make_fixed_length_events, create_info, compute_rank)
 from mne.channels import equalize_channels
 from mne.datasets import testing
-from mne.fixes import _get_args
 from mne.io import read_raw_fif, RawArray, read_raw_ctf, read_info
 from mne.io.pick import _DATA_CH_TYPES_SPLIT, pick_info
 from mne.preprocessing import maxwell_filter
 from mne.rank import _compute_rank_int
-from mne.utils import requires_sklearn, catch_logging, assert_snr
+from mne.utils import (requires_sklearn, catch_logging, assert_snr,
+                       _record_warnings)
 
 base_dir = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data')
 cov_fname = op.join(base_dir, 'test-cov.fif')
@@ -56,6 +59,7 @@ def test_compute_whitener(proj, pca):
         raw.del_proj()
     with pytest.warns(RuntimeWarning, match='Too few samples'):
         cov = compute_raw_covariance(raw)
+    assert cov['names'] == raw.ch_names
     W, _, C = compute_whitener(cov, raw.info, pca=pca, return_colorer=True,
                                verbose='error')
     n_channels = len(raw.ch_names)
@@ -75,6 +79,26 @@ def test_compute_whitener(proj, pca):
         assert pca is False
         assert_allclose(round_trip, np.eye(n_channels), atol=0.05)
 
+    raw.info['bads'] = [raw.ch_names[0]]
+    picks = pick_types(raw.info, meg=True, eeg=True, exclude=[])
+    with pytest.warns(RuntimeWarning, match='Too few samples'):
+        cov2 = compute_raw_covariance(raw, picks=picks)
+        cov3 = compute_raw_covariance(raw, picks=None)
+    assert_allclose(cov2['data'][1:, 1:], cov3['data'])
+    W2, _, C2 = compute_whitener(cov2, raw.info, pca=pca, return_colorer=True,
+                                 picks=picks, verbose='error')
+    W3, _, C3 = compute_whitener(cov3, raw.info, pca=pca, return_colorer=True,
+                                 picks=None, verbose='error')
+    # this tol is not great, but Windows needs it
+    rtol = 1e-3 if sys.platform.startswith('win') else 1e-11
+    assert_allclose(W, W2, rtol=rtol)
+    assert_allclose(C, C2, rtol=rtol)
+    n_channels = len(raw.ch_names) - len(raw.info['bads'])
+    n_reduced = len(raw.ch_names) - len(raw.info['bads'])
+    rank = n_channels - len(raw.info['projs'])
+    n_reduced = rank if pca is True else n_channels
+    assert W3.shape == C3.shape[::-1] == (n_reduced, n_channels)
+
 
 def test_cov_mismatch():
     """Test estimation with MEG<->Head mismatch."""
@@ -93,7 +117,7 @@ def test_cov_mismatch():
             epochs_2.info['dev_head_t'] = None
         pytest.raises(ValueError, compute_covariance, [epochs, epochs_2])
         compute_covariance([epochs, epochs_2], on_mismatch='ignore')
-        with pytest.raises(RuntimeWarning, match='transform mismatch'):
+        with pytest.warns(RuntimeWarning, match='transform mismatch'):
             compute_covariance([epochs, epochs_2], on_mismatch='warn')
         with pytest.raises(ValueError, match='Invalid value'):
             compute_covariance(epochs, on_mismatch='x')
@@ -171,9 +195,9 @@ def _assert_reorder(cov_new, cov_orig, order):
                     cov_orig['data'], atol=1e-20)
 
 
-def test_ad_hoc_cov(tmpdir):
+def test_ad_hoc_cov(tmp_path):
     """Test ad hoc cov creation and I/O."""
-    out_fname = tmpdir.join('test-cov.fif')
+    out_fname = tmp_path / 'test-cov.fif'
     evoked = read_evokeds(ave_fname)[0]
     cov = make_ad_hoc_cov(evoked.info)
     cov.save(out_fname)
@@ -182,7 +206,7 @@ def test_ad_hoc_cov(tmpdir):
     assert_array_almost_equal(cov['data'], cov2['data'])
     std = dict(grad=2e-13, mag=10e-15, eeg=0.1e-6)
     cov = make_ad_hoc_cov(evoked.info, std)
-    cov.save(out_fname)
+    cov.save(out_fname, overwrite=True)
     assert 'Covariance' in repr(cov)
     cov2 = read_cov(out_fname)
     assert_array_almost_equal(cov['data'], cov2['data'])
@@ -196,13 +220,13 @@ def test_ad_hoc_cov(tmpdir):
         cov._get_square()
 
 
-def test_io_cov(tmpdir):
+def test_io_cov(tmp_path):
     """Test IO for noise covariance matrices."""
     cov = read_cov(cov_fname)
     cov['method'] = 'empirical'
     cov['loglik'] = -np.inf
-    cov.save(tmpdir.join('test-cov.fif'))
-    cov2 = read_cov(tmpdir.join('test-cov.fif'))
+    cov.save(tmp_path / 'test-cov.fif')
+    cov2 = read_cov(tmp_path / 'test-cov.fif')
     assert_array_almost_equal(cov.data, cov2.data)
     assert_equal(cov['method'], cov2['method'])
     assert_equal(cov['loglik'], cov2['loglik'])
@@ -210,32 +234,36 @@ def test_io_cov(tmpdir):
 
     cov2 = read_cov(cov_gz_fname)
     assert_array_almost_equal(cov.data, cov2.data)
-    cov2.save(tmpdir.join('test-cov.fif.gz'))
-    cov2 = read_cov(tmpdir.join('test-cov.fif.gz'))
+    cov2.save(tmp_path / 'test-cov.fif.gz')
+    cov2 = read_cov(tmp_path / 'test-cov.fif.gz')
     assert_array_almost_equal(cov.data, cov2.data)
 
     cov['bads'] = ['EEG 039']
     cov_sel = pick_channels_cov(cov, exclude=cov['bads'])
     assert cov_sel['dim'] == (len(cov['data']) - len(cov['bads']))
     assert cov_sel['data'].shape == (cov_sel['dim'], cov_sel['dim'])
-    cov_sel.save(tmpdir.join('test-cov.fif'))
+    cov_sel.save(tmp_path / 'test-cov.fif', overwrite=True)
 
     cov2 = read_cov(cov_gz_fname)
     assert_array_almost_equal(cov.data, cov2.data)
-    cov2.save(tmpdir.join('test-cov.fif.gz'))
-    cov2 = read_cov(tmpdir.join('test-cov.fif.gz'))
+    cov2.save(tmp_path / 'test-cov.fif.gz', overwrite=True)
+    cov2 = read_cov(tmp_path / 'test-cov.fif.gz')
     assert_array_almost_equal(cov.data, cov2.data)
 
     # test warnings on bad filenames
-    cov_badname = tmpdir.join('test-bad-name.fif.gz')
+    cov_badname = tmp_path / 'test-bad-name.fif.gz'
     with pytest.warns(RuntimeWarning, match='-cov.fif'):
         write_cov(cov_badname, cov)
     with pytest.warns(RuntimeWarning, match='-cov.fif'):
         read_cov(cov_badname)
 
 
-@pytest.mark.parametrize('method', (None, 'empirical', 'shrunk'))
-def test_cov_estimation_on_raw(method, tmpdir):
+@pytest.mark.parametrize('method', [
+    None,
+    'empirical',
+    pytest.param('shrunk', marks=pytest.mark.slowtest),
+])
+def test_cov_estimation_on_raw(method, tmp_path):
     """Test estimation from raw (typically empty room)."""
     if method == 'shrunk':
         try:
@@ -249,7 +277,7 @@ def test_cov_estimation_on_raw(method, tmpdir):
     # The pure-string uses the more efficient numpy-based method, the
     # the list gets triaged to compute_covariance (should be equivalent
     # but use more memory)
-    with pytest.warns(None):  # can warn about EEG ref
+    with _record_warnings():  # can warn about EEG ref
         cov = compute_raw_covariance(
             raw, tstep=None, method=method, rank='full',
             method_params=method_params)
@@ -272,15 +300,15 @@ def test_cov_estimation_on_raw(method, tmpdir):
         assert_snr(cov.data, other.data, 1e6)
 
     # tstep=0.2 (default)
-    with pytest.warns(None):  # can warn about EEG ref
+    with _record_warnings():  # can warn about EEG ref
         cov = compute_raw_covariance(raw, method=method, rank='full',
                                      method_params=method_params)
     assert_equal(cov.nfree, cov_mne.nfree - 120)  # cutoff some samples
     assert_snr(cov.data, cov_mne.data, 170)
 
     # test IO when computation done in Python
-    cov.save(tmpdir.join('test-cov.fif'))  # test saving
-    cov_read = read_cov(tmpdir.join('test-cov.fif'))
+    cov.save(tmp_path / 'test-cov.fif')  # test saving
+    cov_read = read_cov(tmp_path / 'test-cov.fif')
     assert cov_read.ch_names == cov.ch_names
     assert cov_read.nfree == cov.nfree
     assert_array_almost_equal(cov.data, cov_read.data)
@@ -304,7 +332,7 @@ def test_cov_estimation_on_raw(method, tmpdir):
     pytest.raises(ValueError, compute_raw_covariance, raw, tstep=None,
                   method='empirical', reject=dict(eog=200e-6))
     # but this should work
-    with pytest.warns(None):  # sklearn
+    with _record_warnings():  # sklearn
         cov = compute_raw_covariance(
             raw.copy().crop(0, 10.), tstep=None, method=method,
             reject=dict(eog=1000e-6), method_params=method_params,
@@ -316,12 +344,12 @@ def test_cov_estimation_on_raw(method, tmpdir):
 def test_cov_estimation_on_raw_reg():
     """Test estimation from raw with regularization."""
     raw = read_raw_fif(raw_fname, preload=True)
-    raw.info['sfreq'] /= 10.
+    with raw.info._unlock():
+        raw.info['sfreq'] /= 10.
     raw = RawArray(raw._data[:, ::10].copy(), raw.info)  # decimate for speed
     cov_mne = read_cov(erm_cov_fname)
     with pytest.warns(RuntimeWarning, match='Too few samples'):
-        # XXX don't use "shrunk" here, for some reason it makes Travis 2.7
-        # hang... "diagonal_fixed" is much faster. Use long epochs for speed.
+        # "diagonal_fixed" is much faster. Use long epochs for speed.
         cov = compute_raw_covariance(raw, tstep=5., method='diagonal_fixed')
     assert_snr(cov.data, cov_mne.data, 5)
 
@@ -337,7 +365,7 @@ def _assert_cov(cov, cov_desired, tol=0.005, nfree=True):
 
 @pytest.mark.slowtest
 @pytest.mark.parametrize('rank', ('full', None))
-def test_cov_estimation_with_triggers(rank, tmpdir):
+def test_cov_estimation_with_triggers(rank, tmp_path):
     """Test estimation from raw with triggers."""
     raw = read_raw_fif(raw_fname)
     raw.set_eeg_reference(projection=True).load_data()
@@ -384,8 +412,8 @@ def test_cov_estimation_with_triggers(rank, tmpdir):
                   keep_sample_mean=False, method='shrunk', rank=rank)
 
     # test IO when computation done in Python
-    cov.save(tmpdir.join('test-cov.fif'))  # test saving
-    cov_read = read_cov(tmpdir.join('test-cov.fif'))
+    cov.save(tmp_path / 'test-cov.fif')  # test saving
+    cov_read = read_cov(tmp_path / 'test-cov.fif')
     _assert_cov(cov, cov_read, 1e-5)
 
     # cov with list of epochs with different projectors
@@ -440,7 +468,8 @@ def test_regularize_cov():
     assert noise_cov['data'].shape == reg_noise_cov['data'].shape
     assert np.mean(noise_cov['data'] < reg_noise_cov['data']) < 0.08
     # make sure all args are represented
-    assert set(_DATA_CH_TYPES_SPLIT) - set(_get_args(regularize)) == set()
+    assert (set(_DATA_CH_TYPES_SPLIT) - set(signature(regularize).parameters)
+            == set())
 
 
 def test_whiten_evoked():
@@ -546,12 +575,12 @@ def test_compute_covariance_auto_reg(rank):
     covs = compute_covariance(epochs, method='auto',
                               method_params=method_params,
                               return_estimators=True, rank=rank)
-    # make sure regularization produces structured differencess
+    # make sure regularization produces structured differences
     diag_mask = np.eye(len(epochs.ch_names)).astype(bool)
     off_diag_mask = np.invert(diag_mask)
     for cov_a, cov_b in itt.combinations(covs, 2):
         if (cov_a['method'] == 'diagonal_fixed' and
-                # here we have diagnoal or no regularization.
+                # here we have diagonal or no regularization.
                 cov_b['method'] == 'empirical' and rank == 'full'):
 
             assert not np.any(cov_a['data'][diag_mask] ==
@@ -617,7 +646,7 @@ def _cov_rank(cov, info, proj=True):
     # ignore warnings about rank mismatches: sometimes we will intentionally
     # violate the computed/info assumption, such as when using SSS with
     # `rank='full'`
-    with pytest.warns(None):
+    with _record_warnings():
         return _compute_rank_int(cov, info=info, proj=proj)
 
 
@@ -699,7 +728,8 @@ def test_low_rank_cov(raw_epochs_events):
     # test that rank=306 is same as rank='full'
     epochs_meg = epochs.copy().pick_types(meg=True)
     assert len(epochs_meg.ch_names) == 306
-    epochs_meg.info.update(bads=[], projs=[])
+    with epochs_meg.info._unlock():
+        epochs_meg.info.update(bads=[], projs=[])
     cov_full = compute_covariance(epochs_meg, method='oas',
                                   rank='full', verbose='error')
     assert _cov_rank(cov_full, epochs_meg.info) == 306
@@ -787,7 +817,8 @@ def test_compute_whitener_rank():
     """Test risky rank options."""
     info = read_info(ave_fname)
     info = pick_info(info, pick_types(info, meg=True))
-    info['projs'] = []
+    with info._unlock():
+        info['projs'] = []
     # need a square version because the diag one takes shortcuts in
     # compute_whitener (users shouldn't even need this function so it's
     # private)
