@@ -2,7 +2,7 @@
 # Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #         Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 import os.path as op
 import os
 
@@ -16,9 +16,8 @@ from mne import (read_events, write_events, make_fixed_length_events,
                  read_evokeds, Epochs, create_info, compute_raw_covariance,
                  Annotations)
 from mne.io import read_raw_fif, RawArray
-from mne.utils import run_tests_if_main
 from mne.event import (define_target_events, merge_events, AcqParserFIF,
-                       shift_time_events)
+                       shift_time_events, match_event_names)
 from mne.datasets import testing
 
 base_dir = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data')
@@ -111,11 +110,11 @@ def test_merge_events():
         assert_array_equal(events, events_good)
 
 
-def test_io_events(tmpdir):
+def test_io_events(tmp_path):
     """Test IO for events."""
     # Test binary fif IO
     events = read_events(fname)  # Use as the gold standard
-    fname_temp = tmpdir.join('events-eve.fif')
+    fname_temp = tmp_path / 'events-eve.fif'
     write_events(fname_temp, events)
     events2 = read_events(fname_temp)
     assert_array_almost_equal(events, events2)
@@ -123,13 +122,13 @@ def test_io_events(tmpdir):
     # Test binary fif.gz IO
     events2 = read_events(fname_gz)  # Use as the gold standard
     assert_array_almost_equal(events, events2)
-    fname_temp += '.gz'
+    fname_temp = str(fname_temp) + '.gz'
     write_events(fname_temp, events2)
     events2 = read_events(fname_temp)
     assert_array_almost_equal(events, events2)
 
     # Test new format text file IO
-    fname_temp = str(tmpdir.join('events.eve'))
+    fname_temp = tmp_path / 'events.eve'
     write_events(fname_temp, events)
     events2 = read_events(fname_temp)
     assert_array_almost_equal(events, events2)
@@ -140,12 +139,12 @@ def test_io_events(tmpdir):
     # Test old format text file IO
     events2 = read_events(fname_old_txt)
     assert_array_almost_equal(events, events2)
-    write_events(fname_temp, events)
+    write_events(fname_temp, events, overwrite=True)
     events2 = read_events(fname_temp)
     assert_array_almost_equal(events, events2)
 
     # Test event selection
-    fname_temp = tmpdir.join('events-eve.fif')
+    fname_temp = tmp_path / 'events-eve.fif'
     a = read_events(fname_temp, include=1)
     b = read_events(fname_temp, include=[1])
     c = read_events(fname_temp, exclude=[2, 3, 4, 5, 32])
@@ -157,24 +156,24 @@ def test_io_events(tmpdir):
     # test reading file with mask=None
     events2 = events.copy()
     events2[:, -1] = range(events2.shape[0])
-    write_events(fname_temp, events2)
+    write_events(fname_temp, events2, overwrite=True)
     events3 = read_events(fname_temp, mask=None)
     assert_array_almost_equal(events2, events3)
 
     # Test binary file IO for 1 event
     events = read_events(fname_1)  # Use as the new gold standard
-    write_events(fname_temp, events)
+    write_events(fname_temp, events, overwrite=True)
     events2 = read_events(fname_temp)
     assert_array_almost_equal(events, events2)
 
     # Test text file IO for 1 event
-    fname_temp = str(tmpdir.join('events.eve'))
-    write_events(fname_temp, events)
+    fname_temp = tmp_path / 'events.eve'
+    write_events(fname_temp, events, overwrite=True)
     events2 = read_events(fname_temp)
     assert_array_almost_equal(events, events2)
 
     # test warnings on bad filenames
-    fname2 = tmpdir.join('test-bad-name.fif')
+    fname2 = tmp_path / 'test-bad-name.fif'
     with pytest.warns(RuntimeWarning, match='-eve.fif'):
         write_events(fname2, events)
     with pytest.warns(RuntimeWarning, match='-eve.fif'):
@@ -217,8 +216,9 @@ def test_find_events():
     assert_array_equal(events11, events22)
 
     # Reset some data for ease of comparison
-    raw._first_samps[0] = 0
-    raw.info['sfreq'] = 1000
+    raw._cropped_samp = 0
+    with raw.info._unlock():
+        raw.info['sfreq'] = 1000
 
     stim_channel = 'STI 014'
     stim_channel_idx = pick_channels(raw.info['ch_names'],
@@ -381,12 +381,12 @@ def test_find_events():
     # test initial_event argument
     info = create_info(['MYSTI'], 1000, 'stim')
     data = np.zeros((1, 1000))
-    raw = RawArray(data, info)
+    raw = RawArray(data, info, first_samp=7)
     data[0, :10] = 100
     data[0, 30:40] = 200
-    assert_array_equal(find_events(raw, 'MYSTI'), [[30, 0, 200]])
+    assert_array_equal(find_events(raw, 'MYSTI'), [[37, 0, 200]])
     assert_array_equal(find_events(raw, 'MYSTI', initial_event=True),
-                       [[0, 0, 100], [30, 0, 200]])
+                       [[7, 0, 100], [37, 0, 200]])
 
     # test error message for raw without stim channels
     raw = read_raw_fif(raw_fname, preload=True)
@@ -461,7 +461,7 @@ def test_make_fixed_length_events():
     # Make sure it gets used properly by compute_raw_covariance
     cov = compute_raw_covariance(raw, tstep=None)
     expected = np.cov(data[:, :21216])
-    np.testing.assert_allclose(cov['data'], expected, atol=1e-12)
+    assert_allclose(cov['data'], expected, atol=1e-12)
 
     # overlaps
     events = make_fixed_length_events(raw, 1, duration=1)
@@ -593,4 +593,48 @@ def test_shift_time_events():
     assert all(new_events[:, 0] == EXPECTED)
 
 
-run_tests_if_main()
+def test_match_event_names():
+    """Test event name / event group matching."""
+    event_names = [
+        'auditory/left',
+        'auditory/right',
+        'visual/left',
+        'visual/right'
+    ]
+
+    keys = ['auditory', 'left']
+    expected_matches = ['auditory/left', 'auditory/right', 'visual/left']
+    matches = match_event_names(event_names=event_names, keys=keys)
+    assert matches == expected_matches
+
+    # `keys` order shouldn't matter
+    keys = ['left', 'auditory']
+    matches = match_event_names(event_names=event_names, keys=keys)
+    assert matches == expected_matches
+
+    # Test `keys` is string
+    keys = 'left'
+    expected_matches = ['auditory/left', 'visual/left']
+    matches = match_event_names(event_names=event_names, keys=keys)
+    assert matches == expected_matches
+
+    # Test `keys` is invalid type
+    for keys in (123, [123, 456]):
+        with pytest.raises(ValueError, match='keys must be strings'):
+            match_event_names(event_names=event_names, keys=keys)
+
+    # Test no matches
+    keys = 'laboratory'
+    with pytest.raises(KeyError, match='could not be found'):
+        match_event_names(event_names=event_names, keys=keys)
+
+    with pytest.warns(RuntimeWarning, match='could not be found'):
+        matches = match_event_names(
+            event_names=event_names, keys=keys, on_missing='warn'
+        )
+        assert matches == []
+
+    matches = match_event_names(
+        event_names=event_names, keys=keys, on_missing='ignore'
+    )
+    assert matches == []
