@@ -2,37 +2,40 @@
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import os.path as op
 from shutil import copytree
 
 import pytest
+import scipy
 import numpy as np
-from numpy.testing import assert_array_equal, assert_allclose, assert_equal
+from numpy.testing import (assert_array_equal, assert_allclose, assert_equal,
+                           assert_array_less)
 from mne.datasets import testing
 import mne
-from mne import (read_source_spaces, vertex_to_mni, write_source_spaces,
+from mne import (read_source_spaces, write_source_spaces,
                  setup_source_space, setup_volume_source_space,
                  add_source_space_distances, read_bem_surfaces,
                  morph_source_spaces, SourceEstimate, make_sphere_model,
-                 head_to_mni, read_trans, compute_source_morph,
-                 read_bem_solution)
-from mne.utils import (requires_fs_or_nibabel, requires_nibabel,
-                       requires_freesurfer, run_subprocess, modified_env,
-                       requires_mne, run_tests_if_main)
+                 compute_source_morph, pick_types,
+                 read_bem_solution, read_freesurfer_lut,
+                 read_trans)
+from mne.fixes import _get_img_fdata
+from mne.utils import (requires_nibabel, run_subprocess, _record_warnings,
+                       requires_mne, check_version)
 from mne.surface import _accumulate_normals, _triangle_neighbors
-from mne.source_space import _get_mri_header, _get_mgz_header, _read_talxfm
 from mne.source_estimate import _get_src_type
-from mne.transforms import apply_trans, invert_transform
-from mne.source_space import (get_volume_labels_from_aseg, SourceSpaces,
-                              get_volume_labels_from_src,
-                              _compare_source_spaces)
+from mne.source_space import (get_volume_labels_from_src,
+                              _compare_source_spaces, get_decimated_surfaces,
+                              compute_distance_to_sensors)
+from mne.io.pick import _picks_to_idx
 from mne.io.constants import FIFF
 
 data_path = testing.data_path(download=False)
 subjects_dir = op.join(data_path, 'subjects')
 fname_mri = op.join(data_path, 'subjects', 'sample', 'mri', 'T1.mgz')
+aseg_fname = op.join(data_path, 'subjects', 'sample', 'mri', 'aseg.mgz')
 fname = op.join(subjects_dir, 'sample', 'bem', 'sample-oct-6-src.fif')
 fname_vol = op.join(subjects_dir, 'sample', 'bem',
                     'sample-volume-7mm-src.fif')
@@ -47,34 +50,87 @@ fname_bem_3_sol = op.join(data_path, 'subjects', 'sample', 'bem',
 fname_fs = op.join(subjects_dir, 'fsaverage', 'bem', 'fsaverage-ico-5-src.fif')
 fname_morph = op.join(subjects_dir, 'sample', 'bem',
                       'sample-fsaverage-ico-5-src.fif')
+fname_src = op.join(
+    data_path, 'subjects', 'sample', 'bem', 'sample-oct-4-src.fif')
+fname_fwd = op.join(
+    data_path, 'MEG', 'sample', 'sample_audvis_trunc-meg-eeg-oct-4-fwd.fif')
 trans_fname = op.join(data_path, 'MEG', 'sample',
                       'sample_audvis_trunc-trans.fif')
 
 base_dir = op.join(op.dirname(__file__), '..', 'io', 'tests', 'data')
 fname_small = op.join(base_dir, 'small-src.fif.gz')
+fname_ave = op.join(base_dir, 'test-ave.fif')
 rng = np.random.RandomState(0)
 
 
 @testing.requires_testing_data
-@requires_nibabel(vox2ras_tkr=True)
-def test_mgz_header():
-    """Test MGZ header reading."""
-    header = _get_mgz_header(fname_mri)
-    mri_hdr = _get_mri_header(fname_mri)
-    assert_allclose(mri_hdr.get_data_shape(), header['dims'])
-    assert_allclose(mri_hdr.get_vox2ras_tkr(), header['vox2ras_tkr'])
-    assert_allclose(mri_hdr.get_ras2vox(), header['ras2vox'])
+@pytest.mark.parametrize('picks, limits', [
+    ('meg', (0.02, 0.250)),
+    (None, (0.01, 0.250)),  # should be same as EEG
+    ('eeg', (0.01, 0.250)),
+])
+def test_compute_distance_to_sensors(picks, limits):
+    """Test computation of distances between vertices and sensors."""
+    src = read_source_spaces(fname_src)
+    fwd = mne.read_forward_solution(fname_fwd)
+    info = fwd['info']
+    trans = read_trans(trans_fname)
+    # trans = fwd['info']['mri_head_t']
+    if isinstance(picks, str):
+        kwargs = dict()
+        kwargs[picks] = True
+        if picks == 'eeg':
+            info['dev_head_t'] = None  # should not break anything
+        use_picks = pick_types(info, **kwargs, exclude=())
+    else:
+        use_picks = picks
+    n_picks = len(_picks_to_idx(info, use_picks, 'data', exclude=()))
+
+    # Make sure same vertices are used in src and fwd
+    src[0]['inuse'] = fwd['src'][0]['inuse']
+    src[1]['inuse'] = fwd['src'][1]['inuse']
+    src[0]['nuse'] = fwd['src'][0]['nuse']
+    src[1]['nuse'] = fwd['src'][1]['nuse']
+
+    n_verts = src[0]['nuse'] + src[1]['nuse']
+
+    # minimum distances between vertices and sensors
+    depths = compute_distance_to_sensors(src, info=info, picks=use_picks,
+                                         trans=trans)
+    assert depths.shape == (n_verts, n_picks)
+    assert limits[0] * 5 > depths.min()  # meaningful choice of limits
+    assert_array_less(limits[0], depths)
+    assert_array_less(depths, limits[1])
+
+    # If source space from Forward Solution and trans=None (i.e. identity) then
+    # depths2 should be the same as depth.
+    depths2 = compute_distance_to_sensors(src=fwd['src'], info=info,
+                                          picks=use_picks, trans=None)
+    assert_allclose(depths, depths2, rtol=1e-5)
+
+    if picks != 'eeg':
+        # this should break things
+        info['dev_head_t'] = None
+        with pytest.raises(ValueError,
+                           match='Transform between meg<->head'):
+            compute_distance_to_sensors(src, info, use_picks, trans)
+
+
+def _read_small_src(remove=True):
+    src = read_source_spaces(fname_small)
+    if remove:
+        for s in src:
+            s['nearest'] = None
+            s['nearest_dist'] = None
+            s['pinfo'] = None
+    return src
 
 
 def test_add_patch_info(monkeypatch):
     """Test adding patch info to source space."""
     # let's setup a small source space
-    src = read_source_spaces(fname_small)
-    src_new = read_source_spaces(fname_small)
-    for s in src_new:
-        s['nearest'] = None
-        s['nearest_dist'] = None
-        s['pinfo'] = None
+    src = _read_small_src(remove=False)
+    src_new = _read_small_src()
 
     # test that no patch info is added for small dist_limit
     add_source_space_distances(src_new, dist_limit=0.00001)
@@ -83,20 +139,64 @@ def test_add_patch_info(monkeypatch):
     assert all(s['pinfo'] is None for s in src_new)
 
     # now let's use one that works (and test our warning-throwing)
-    monkeypatch.setattr(mne.source_space, '_DIST_WARN_LIMIT', 1)
-    with pytest.warns(RuntimeWarning, match='Computing distances for 258'):
-        add_source_space_distances(src_new)
+    with monkeypatch.context() as m:
+        m.setattr(mne.source_space, '_DIST_WARN_LIMIT', 1)
+        with pytest.warns(RuntimeWarning, match='Computing distances for 258'):
+            add_source_space_distances(src_new)
+    _compare_source_spaces(src, src_new, 'approx')
 
-    for s1, s2 in zip(src, src_new):
-        assert_array_equal(s1['nearest'], s2['nearest'])
-        assert_allclose(s1['nearest_dist'], s2['nearest_dist'], atol=1e-7)
-        assert_equal(len(s1['pinfo']), len(s2['pinfo']))
-        for p1, p2 in zip(s1['pinfo'], s2['pinfo']):
-            assert_array_equal(p1, p2)
+    # Old SciPy can't do patch info only
+    src_new = _read_small_src()
+    with monkeypatch.context() as m:
+        m.setattr(scipy, '__version__', '1.0')
+        with pytest.raises(RuntimeError, match='required to calculate patch '):
+            add_source_space_distances(src_new, dist_limit=0)
+
+    # New SciPy can
+    if check_version('scipy', '1.3'):
+        src_nodist = src.copy()
+        for s in src_nodist:
+            for key in ('dist', 'dist_limit'):
+                s[key] = None
+        add_source_space_distances(src_new, dist_limit=0)
+        _compare_source_spaces(src, src_new, 'approx')
+
+
+# We could test "src_py" here, but we can rely on our existing tests to
+# make sure the pinfo/patch_inds/nearest match
+@testing.requires_testing_data
+@pytest.mark.parametrize('src_kind', ['fwd', 'src'])
+def test_surface_source_space_doc(src_kind):
+    """Test surface source space docstring."""
+    # make sure we're correct about this stuff for both kinds!
+    if src_kind == 'fwd':
+        src = mne.read_source_spaces(fname_fwd)
+    else:
+        assert src_kind == 'src'
+        src = mne.read_source_spaces(fname_src)
+    for s in src:
+        if src_kind == 'src':  # original
+            assert len(s['pinfo']) == s['nuse']
+            assert_array_equal(s['patch_inds'], np.arange(s['nuse']))
+        else:  # pts removed
+            assert len(s['pinfo']) > s['nuse']
+        all_pinfo = np.concatenate(s['pinfo'])
+        assert_array_equal(np.sort(all_pinfo), np.arange(s['np']))
+        assert len(s['patch_inds']) == s['nuse']
+        assert len(s['vertno']) == s['nuse']
+        assert len(s['patch_inds']) == s['nuse']
+        for idx in (0, 42, 173):
+            this_dense_vertex = s['vertno'][idx]
+            # 'pinfo'
+            this_vertex_represents = s['pinfo'][s['patch_inds'][idx]]
+            assert len(this_vertex_represents) > 1
+            # 'nearest'
+            for other in this_vertex_represents:
+                assert s['nearest'][other] == this_dense_vertex
 
 
 @testing.requires_testing_data
-def test_add_source_space_distances_limited(tmpdir):
+def test_add_source_space_distances_limited(tmp_path):
     """Test adding distances to source space with a dist_limit."""
     src = read_source_spaces(fname)
     src_new = read_source_spaces(fname)
@@ -105,7 +205,7 @@ def test_add_source_space_distances_limited(tmpdir):
     n_do = 200  # limit this for speed
     src_new[0]['vertno'] = src_new[0]['vertno'][:n_do].copy()
     src_new[1]['vertno'] = src_new[1]['vertno'][:n_do].copy()
-    out_name = tmpdir.join('temp-src.fif')
+    out_name = tmp_path / 'temp-src.fif'
     add_source_space_distances(src_new, dist_limit=0.007)
     write_source_spaces(out_name, src_new)
     src_new = read_source_spaces(out_name)
@@ -130,7 +230,7 @@ def test_add_source_space_distances_limited(tmpdir):
 
 @pytest.mark.slowtest
 @testing.requires_testing_data
-def test_add_source_space_distances(tmpdir):
+def test_add_source_space_distances(tmp_path):
     """Test adding distances to source space."""
     src = read_source_spaces(fname)
     src_new = read_source_spaces(fname)
@@ -139,9 +239,11 @@ def test_add_source_space_distances(tmpdir):
     n_do = 19  # limit this for speed
     src_new[0]['vertno'] = src_new[0]['vertno'][:n_do].copy()
     src_new[1]['vertno'] = src_new[1]['vertno'][:n_do].copy()
-    out_name = tmpdir.join('temp-src.fif')
+    out_name = tmp_path / 'temp-src.fif'
     n_jobs = 2
     assert n_do % n_jobs != 0
+    with pytest.raises(ValueError, match='non-negative'):
+        add_source_space_distances(src_new, dist_limit=-1)
     add_source_space_distances(src_new, n_jobs=n_jobs)
     write_source_spaces(out_name, src_new)
     src_new = read_source_spaces(out_name)
@@ -172,15 +274,15 @@ def test_add_source_space_distances(tmpdir):
 
 @testing.requires_testing_data
 @requires_mne
-def test_discrete_source_space(tmpdir):
+def test_discrete_source_space(tmp_path):
     """Test setting up (and reading/writing) discrete source spaces."""
     src = read_source_spaces(fname)
     v = src[0]['vertno']
 
     # let's make a discrete version with the C code, and with ours
-    temp_name = tmpdir.join('temp-src.fif')
+    temp_name = tmp_path / 'temp-src.fif'
     # save
-    temp_pos = tmpdir.join('temp-pos.txt')
+    temp_pos = tmp_path / 'temp-pos.txt'
     np.savetxt(str(temp_pos), np.c_[src[0]['rr'][v], src[0]['nn'][v]])
     # let's try the spherical one (no bem or surf supplied)
     run_subprocess(['mne_volume_source_space', '--meters',
@@ -201,19 +303,25 @@ def test_discrete_source_space(tmpdir):
     _compare_source_spaces(src_c, src_c2)
 
     # now do MRI
-    pytest.raises(ValueError, setup_volume_source_space, 'sample',
-                  pos=pos_dict, mri=fname_mri)
-    assert repr(src_new) == repr(src_c)
+    with pytest.raises(ValueError, match='Cannot create interpolation'):
+        setup_volume_source_space('sample', pos=pos_dict, mri=fname_mri)
+    assert repr(src_new).split('~')[0] == repr(src_c).split('~')[0]
+    assert ' kB' in repr(src_new)
     assert src_new.kind == 'discrete'
     assert _get_src_type(src_new, None) == 'discrete'
 
+    with pytest.raises(RuntimeError, match='finite'):
+        setup_volume_source_space(
+            pos=dict(rr=[[0, 0, float('inf')]], nn=[[0, 1, 0]]))
 
+
+@requires_nibabel()
 @pytest.mark.slowtest
 @testing.requires_testing_data
-def test_volume_source_space(tmpdir):
+def test_volume_source_space(tmp_path):
     """Test setting up volume source spaces."""
     src = read_source_spaces(fname_vol)
-    temp_name = tmpdir.join('temp-src.fif')
+    temp_name = tmp_path / 'temp-src.fif'
     surf = read_bem_surfaces(fname_bem, s_id=FIFF.FIFFV_BEM_SURF_ID_BRAIN)
     surf['rr'] *= 1e3  # convert to mm
     bem_sol = read_bem_solution(fname_bem_3_sol)
@@ -231,7 +339,7 @@ def test_volume_source_space(tmpdir):
         del src_new
         src_new = read_source_spaces(temp_name)
         _compare_source_spaces(src, src_new, mode='approx')
-    with pytest.raises(IOError, match='surface file.*not found'):
+    with pytest.raises(IOError, match='surface file.*not exist'):
         setup_volume_source_space(
             'sample', surface='foo', mri=fname_mri, subjects_dir=subjects_dir)
     bem['surfs'][-1]['coord_frame'] = FIFF.FIFFV_COORD_HEAD
@@ -244,14 +352,15 @@ def test_volume_source_space(tmpdir):
             'sample', bem=bem, mri=fname_mri, subjects_dir=subjects_dir)
     del bem
     assert repr(src) == repr(src_new)
+    assert ' MB' in repr(src)
     assert src.kind == 'volume'
     # Spheres
     sphere = make_sphere_model(r0=(0., 0., 0.), head_radius=0.1,
                                relative_radii=(0.9, 1.0), sigmas=(0.33, 1.0))
-    src = setup_volume_source_space(pos=10)
+    src = setup_volume_source_space(pos=10, sphere=(0., 0., 0., 0.09))
     src_new = setup_volume_source_space(pos=10, sphere=sphere)
     _compare_source_spaces(src, src_new, mode='exact')
-    with pytest.raises(ValueError, match='could not convert string to float'):
+    with pytest.raises(ValueError, match='sphere, if str'):
         setup_volume_source_space(sphere='foo')
     # Need a radius
     sphere = make_sphere_model(head_radius=None)
@@ -261,20 +370,22 @@ def test_volume_source_space(tmpdir):
 
 @testing.requires_testing_data
 @requires_mne
-def test_other_volume_source_spaces(tmpdir):
+def test_other_volume_source_spaces(tmp_path):
     """Test setting up other volume source spaces."""
     # these are split off because they require the MNE tools, and
     # Travis doesn't seem to like them
 
     # let's try the spherical one (no bem or surf supplied)
-    temp_name = tmpdir.join('temp-src.fif')
+    temp_name = tmp_path / 'temp-src.fif'
     run_subprocess(['mne_volume_source_space',
                     '--grid', '7.0',
                     '--src', temp_name,
                     '--mri', fname_mri])
     src = read_source_spaces(temp_name)
+    sphere = (0., 0., 0., 0.09)
     src_new = setup_volume_source_space(None, pos=7.0, mri=fname_mri,
-                                        subjects_dir=subjects_dir)
+                                        subjects_dir=subjects_dir,
+                                        sphere=sphere)
     # we use a more accurate elimination criteria, so let's fix the MNE-C
     # source space
     assert len(src_new[0]['vertno']) == 7497
@@ -349,7 +460,7 @@ def test_accumulate_normals():
 
 @pytest.mark.slowtest
 @testing.requires_testing_data
-def test_setup_source_space(tmpdir):
+def test_setup_source_space(tmp_path):
     """Test setting up ico, oct, and all source spaces."""
     fname_ico = op.join(data_path, 'subjects', 'fsaverage', 'bem',
                         'fsaverage-ico-5-src.fif')
@@ -374,19 +485,19 @@ def test_setup_source_space(tmpdir):
 
     # ico 5 (fsaverage) - write to temp file
     src = read_source_spaces(fname_ico)
-    with pytest.warns(None):  # sklearn equiv neighbors
+    with _record_warnings():  # sklearn equiv neighbors
         src_new = setup_source_space('fsaverage', spacing='ico5',
                                      subjects_dir=subjects_dir, add_dist=False)
     _compare_source_spaces(src, src_new, mode='approx')
-    assert_equal(repr(src), repr(src_new))
-    assert_equal(repr(src).count('surface ('), 2)
+    assert repr(src).split('~')[0] == repr(src_new).split('~')[0]
+    assert repr(src).count('surface (') == 2
     assert_array_equal(src[0]['vertno'], np.arange(10242))
     assert_array_equal(src[1]['vertno'], np.arange(10242))
 
     # oct-6 (sample) - auto filename + IO
     src = read_source_spaces(fname)
-    temp_name = tmpdir.join('temp-src.fif')
-    with pytest.warns(None):  # sklearn equiv neighbors
+    temp_name = tmp_path / 'temp-src.fif'
+    with _record_warnings():  # sklearn equiv neighbors
         src_new = setup_source_space('sample', spacing='oct6',
                                      subjects_dir=subjects_dir, add_dist=False)
         write_source_spaces(temp_name, src_new, overwrite=True)
@@ -411,22 +522,24 @@ def test_setup_source_space(tmpdir):
 @pytest.mark.slowtest
 @pytest.mark.timeout(60)
 @pytest.mark.parametrize('spacing', [2, 7])
-def test_setup_source_space_spacing(tmpdir, spacing):
+def test_setup_source_space_spacing(tmp_path, spacing, monkeypatch):
     """Test setting up surface source spaces using a given spacing."""
-    copytree(op.join(subjects_dir, 'sample'), str(tmpdir.join('sample')))
+    copytree(op.join(subjects_dir, 'sample'), tmp_path / 'sample')
     args = [] if spacing == 7 else ['--spacing', str(spacing)]
-    with modified_env(SUBJECTS_DIR=str(tmpdir), SUBJECT='sample'):
-        run_subprocess(['mne_setup_source_space'] + args)
-    src = read_source_spaces(tmpdir.join('sample', 'bem',
-                                         'sample-%d-src.fif' % spacing))
-    src_new = setup_source_space('sample', spacing=spacing, add_dist=False,
-                                 subjects_dir=subjects_dir)
+    monkeypatch.setenv('SUBJECTS_DIR', str(tmp_path))
+    monkeypatch.setenv('SUBJECT', 'sample')
+    run_subprocess(['mne_setup_source_space'] + args)
+    src = read_source_spaces(
+        tmp_path / 'sample' / 'bem' / ('sample-%d-src.fif' % spacing)
+    )
+    # No need to pass subjects_dir here because we've setenv'ed it
+    src_new = setup_source_space('sample', spacing=spacing, add_dist=False)
     _compare_source_spaces(src, src_new, mode='approx', nearest=True)
     # Degenerate conditions
     with pytest.raises(TypeError, match='spacing must be.*got.*float.*'):
-        setup_source_space('sample', 7., subjects_dir=subjects_dir)
+        setup_source_space('sample', 7.)
     with pytest.raises(ValueError, match='spacing must be >= 2, got 1'):
-        setup_source_space('sample', 1, subjects_dir=subjects_dir)
+        setup_source_space('sample', 1)
 
 
 @testing.requires_testing_data
@@ -453,16 +566,16 @@ def test_read_source_spaces():
 
 @pytest.mark.slowtest
 @testing.requires_testing_data
-def test_write_source_space(tmpdir):
+def test_write_source_space(tmp_path):
     """Test reading and writing of source spaces."""
     src0 = read_source_spaces(fname, patch_stats=False)
-    temp_fname = tmpdir.join('tmp-src.fif')
+    temp_fname = tmp_path / 'tmp-src.fif'
     write_source_spaces(temp_fname, src0)
     src1 = read_source_spaces(temp_fname, patch_stats=False)
     _compare_source_spaces(src0, src1)
 
     # test warnings on bad filenames
-    src_badname = tmpdir.join('test-bad-name.fif.gz')
+    src_badname = tmp_path / 'test-bad-name.fif.gz'
     with pytest.warns(RuntimeWarning, match='-src.fif'):
         write_source_spaces(src_badname, src0)
     with pytest.warns(RuntimeWarning, match='-src.fif'):
@@ -470,116 +583,89 @@ def test_write_source_space(tmpdir):
 
 
 @testing.requires_testing_data
-@requires_fs_or_nibabel
-def test_vertex_to_mni():
-    """Test conversion of vertices to MNI coordinates."""
-    # obtained using "tksurfer (sample) (l/r)h white"
-    vertices = [100960, 7620, 150549, 96761]
-    coords = np.array([[-60.86, -11.18, -3.19], [-36.46, -93.18, -2.36],
-                       [-38.00, 50.08, -10.61], [47.14, 8.01, 46.93]])
-    hemis = [0, 0, 0, 1]
-    coords_2 = vertex_to_mni(vertices, hemis, 'sample', subjects_dir)
-    # less than 1mm error
-    assert_allclose(coords, coords_2, atol=1.0)
-
-
-@testing.requires_testing_data
-@requires_fs_or_nibabel
-def test_head_to_mni():
-    """Test conversion of aseg vertices to MNI coordinates."""
-    # obtained using freeview
-    coords = np.array([[22.52, 11.24, 17.72], [22.52, 5.46, 21.58],
-                       [16.10, 5.46, 22.23], [21.24, 8.36, 22.23]])
-
-    xfm = _read_talxfm('sample', subjects_dir)
-    coords_MNI = apply_trans(xfm['trans'], coords)
-
-    trans = read_trans(trans_fname)  # head->MRI (surface RAS)
-    mri_head_t = invert_transform(trans)  # MRI (surface RAS)->head matrix
-
-    # obtained from sample_audvis-meg-oct-6-mixed-fwd.fif
-    coo_right_amygdala = np.array([[0.01745682, 0.02665809, 0.03281873],
-                                   [0.01014125, 0.02496262, 0.04233755],
-                                   [0.01713642, 0.02505193, 0.04258181],
-                                   [0.01720631, 0.03073877, 0.03850075]])
-    coords_MNI_2 = head_to_mni(coo_right_amygdala, 'sample', mri_head_t,
-                               subjects_dir)
-    # less than 1mm error
-    assert_allclose(coords_MNI, coords_MNI_2, atol=10.0)
-
-
-@testing.requires_testing_data
-@requires_freesurfer
 @requires_nibabel()
-def test_vertex_to_mni_fs_nibabel():
-    """Test equivalence of vert_to_mni for nibabel and freesurfer."""
-    n_check = 1000
-    subject = 'sample'
-    vertices = rng.randint(0, 100000, n_check)
-    hemis = rng.randint(0, 1, n_check)
-    coords = vertex_to_mni(vertices, hemis, subject, subjects_dir,
-                           'nibabel')
-    coords_2 = vertex_to_mni(vertices, hemis, subject, subjects_dir,
-                             'freesurfer')
-    # less than 0.1 mm error
-    assert_allclose(coords, coords_2, atol=0.1)
-
-
-@testing.requires_testing_data
-@requires_freesurfer
-@requires_nibabel()
-def test_get_volume_label_names():
-    """Test reading volume label names."""
-    aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
-    label_names, label_colors = get_volume_labels_from_aseg(aseg_fname,
-                                                            return_colors=True)
-    assert_equal(label_names.count('Brain-Stem'), 1)
-
-    assert_equal(len(label_colors), len(label_names))
-
-
-@testing.requires_testing_data
-@requires_freesurfer
-@requires_nibabel()
-def test_source_space_from_label(tmpdir):
+@pytest.mark.parametrize('pass_ids', (True, False))
+def test_source_space_from_label(tmp_path, pass_ids):
     """Test generating a source space from volume label."""
-    aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
-    label_names = get_volume_labels_from_aseg(aseg_fname)
-    volume_label = label_names[int(np.random.rand() * len(label_names))]
+    aseg_short = 'aseg.mgz'
+    atlas_ids, _ = read_freesurfer_lut()
+    volume_label = 'Left-Cerebellum-Cortex'
 
     # Test pos as dict
     pos = dict()
-    pytest.raises(ValueError, setup_volume_source_space, 'sample', pos=pos,
-                  volume_label=volume_label, mri=aseg_fname)
+    with pytest.raises(ValueError, match='mri must be None if pos is a dict'):
+        setup_volume_source_space(
+            'sample', pos=pos, volume_label=volume_label, mri=aseg_short,
+            subjects_dir=subjects_dir)
 
-    # Test no mri provided
-    pytest.raises(RuntimeError, setup_volume_source_space, 'sample', mri=None,
-                  volume_label=volume_label)
+    # Test T1.mgz provided
+    with pytest.raises(RuntimeError, match=r'Must use a \*aseg.mgz file'):
+        setup_volume_source_space(
+            'sample', mri='T1.mgz', volume_label=volume_label,
+            subjects_dir=subjects_dir)
 
     # Test invalid volume label
-    pytest.raises(ValueError, setup_volume_source_space, 'sample',
-                  volume_label='Hello World!', mri=aseg_fname)
+    mri = aseg_short
+    with pytest.raises(ValueError, match="'Left-Cerebral' not found.*Did you"):
+        setup_volume_source_space(
+            'sample', volume_label='Left-Cerebral', mri=mri,
+            subjects_dir=subjects_dir)
 
-    src = setup_volume_source_space('sample', subjects_dir=subjects_dir,
-                                    volume_label=volume_label, mri=aseg_fname,
-                                    add_interpolator=False)
+    # These should be equivalent
+    if pass_ids:
+        use_volume_label = {volume_label: atlas_ids[volume_label]}
+    else:
+        use_volume_label = volume_label
+
+    # ensure it works even when not provided (detect that it should be aseg)
+    src = setup_volume_source_space(
+        'sample', volume_label=use_volume_label, add_interpolator=False,
+        subjects_dir=subjects_dir)
     assert_equal(volume_label, src[0]['seg_name'])
+    assert src[0]['nuse'] == 404  # for our given pos and label
 
     # test reading and writing
-    out_name = tmpdir.join('temp-src.fif')
+    out_name = tmp_path / 'temp-src.fif'
     write_source_spaces(out_name, src)
     src_from_file = read_source_spaces(out_name)
     _compare_source_spaces(src, src_from_file, mode='approx')
 
 
+@testing.requires_testing_data
+@requires_nibabel()
+def test_source_space_exclusive_complete(src_volume_labels):
+    """Test that we produce exclusive and complete labels."""
+    # these two are neighbors and are quite large, so let's use them to
+    # ensure no overlaps
+    src, volume_labels, _ = src_volume_labels
+    ii = volume_labels.index('Left-Cerebral-White-Matter')
+    jj = volume_labels.index('Left-Cerebral-Cortex')
+    assert src[ii]['nuse'] == 755  # 2034 with pos=5, was 2832
+    assert src[jj]['nuse'] == 616  # 1520 with pos=5, was 2623
+    src_full = read_source_spaces(fname_vol)
+    # This implicitly checks for overlap because np.sort would preserve
+    # duplicates, and it checks for completeness because the sets should match
+    assert_array_equal(src_full[0]['vertno'],
+                       np.sort(np.concatenate([s['vertno'] for s in src])))
+    for si, s in enumerate(src):
+        assert_allclose(src_full[0]['rr'], s['rr'], atol=1e-6)
+    # also check single_volume=True -- should be the same result
+    with pytest.warns(RuntimeWarning, match='Found no usable.*Left-vessel.*'):
+        src_single = setup_volume_source_space(
+            src[0]['subject_his_id'], 7., 'aseg.mgz', bem=fname_bem,
+            volume_label=volume_labels, single_volume=True,
+            add_interpolator=False, subjects_dir=subjects_dir)
+    assert len(src_single) == 1
+    assert 'Unknown+Left-Cerebral-White-Matter+Left-' in repr(src_single)
+    assert_array_equal(src_full[0]['vertno'], src_single[0]['vertno'])
+
+
 @pytest.mark.timeout(60)  # ~24 sec on Travis
 @pytest.mark.slowtest
 @testing.requires_testing_data
-@requires_freesurfer
 @requires_nibabel()
 def test_read_volume_from_src():
     """Test reading volumes from a mixed source space."""
-    aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
     labels_vol = ['Left-Amygdala',
                   'Brain-Stem',
                   'Right-Amygdala']
@@ -592,8 +678,17 @@ def test_read_volume_from_src():
                                         bem=fname_bem,
                                         volume_label=labels_vol,
                                         subjects_dir=subjects_dir)
-    # Generate the mixed source space
+    # Generate the mixed source space, testing some list methods
+    assert src.kind == 'surface'
+    assert vol_src.kind == 'volume'
     src += vol_src
+    assert src.kind == 'mixed'
+    assert vol_src.kind == 'volume'
+    assert src[:2].kind == 'surface'
+    assert src[2:].kind == 'volume'
+    assert src[:].kind == 'mixed'
+    with pytest.raises(RuntimeError, match='Invalid source space'):
+        src[::2]
 
     volume_src = get_volume_labels_from_src(src, 'sample', subjects_dir)
     volume_label = volume_src[0].name
@@ -606,17 +701,17 @@ def test_read_volume_from_src():
 
 
 @testing.requires_testing_data
-@requires_freesurfer
 @requires_nibabel()
-def test_combine_source_spaces(tmpdir):
+def test_combine_source_spaces(tmp_path):
     """Test combining source spaces."""
-    aseg_fname = op.join(subjects_dir, 'sample', 'mri', 'aseg.mgz')
-    label_names = get_volume_labels_from_aseg(aseg_fname)
-    volume_labels = [label_names[int(np.random.rand() * len(label_names))]
-                     for ii in range(2)]
+    import nibabel as nib
+    rng = np.random.RandomState(2)
+    volume_labels = ['Brain-Stem', 'Right-Hippocampus']  # two fairly large
 
-    # get a surface source space (no need to test creation here)
-    srf = read_source_spaces(fname, patch_stats=False)
+    # create a sparse surface source space to ensure all get mapped
+    # when mri_resolution=False
+    srf = setup_source_space('sample', 'oct3', add_dist=False,
+                             subjects_dir=subjects_dir)
 
     # setup 2 volume source spaces
     vol = setup_volume_source_space('sample', subjects_dir=subjects_dir,
@@ -624,7 +719,7 @@ def test_combine_source_spaces(tmpdir):
                                     mri=aseg_fname, add_interpolator=False)
 
     # setup a discrete source space
-    rr = rng.randint(0, 20, (100, 3)) * 1e-3
+    rr = rng.randint(0, 11, (20, 3)) * 5e-3
     nn = np.zeros(rr.shape)
     nn[:, -1] = 1
     pos = {'rr': rr, 'nn': nn}
@@ -632,18 +727,24 @@ def test_combine_source_spaces(tmpdir):
                                      pos=pos, verbose='error')
 
     # combine source spaces
+    assert srf.kind == 'surface'
+    assert vol.kind == 'volume'
+    assert disc.kind == 'discrete'
     src = srf + vol + disc
+    assert src.kind == 'mixed'
+    assert srf.kind == 'surface'
+    assert vol.kind == 'volume'
+    assert disc.kind == 'discrete'
 
     # test addition of source spaces
-    assert_equal(type(src), SourceSpaces)
-    assert_equal(len(src), 4)
+    assert len(src) == 4
 
     # test reading and writing
-    src_out_name = tmpdir.join('temp-src.fif')
+    src_out_name = tmp_path / 'temp-src.fif'
     src.save(src_out_name)
     src_from_file = read_source_spaces(src_out_name)
     _compare_source_spaces(src, src_from_file, mode='approx')
-    assert_equal(repr(src), repr(src_from_file))
+    assert repr(src).split('~')[0] == repr(src_from_file).split('~')[0]
     assert_equal(src.kind, 'mixed')
 
     # test that all source spaces are in MRI coordinates
@@ -651,20 +752,21 @@ def test_combine_source_spaces(tmpdir):
     assert (coord_frames == FIFF.FIFFV_COORD_MRI).all()
 
     # test errors for export_volume
-    image_fname = tmpdir.join('temp-image.mgz')
+    image_fname = tmp_path / 'temp-image.mgz'
 
     # source spaces with no volume
-    pytest.raises(ValueError, srf.export_volume, image_fname, verbose='error')
+    with pytest.raises(ValueError, match='at least one volume'):
+        srf.export_volume(image_fname, verbose='error')
 
     # unrecognized source type
     disc2 = disc.copy()
     disc2[0]['type'] = 'kitty'
-    src_unrecognized = src + disc2
-    pytest.raises(ValueError, src_unrecognized.export_volume, image_fname,
-                  verbose='error')
+    with pytest.raises(ValueError, match='Invalid value'):
+        src + disc2
+    del disc2
 
     # unrecognized file type
-    bad_image_fname = tmpdir.join('temp-image.png')
+    bad_image_fname = tmp_path / 'temp-image.png'
     # vertices outside vol space warning
     pytest.raises(ValueError, src.export_volume, bad_image_fname,
                   verbose='error')
@@ -673,8 +775,38 @@ def test_combine_source_spaces(tmpdir):
     disc3 = disc.copy()
     disc3[0]['coord_frame'] = 10
     src_mixed_coord = src + disc3
-    pytest.raises(ValueError, src_mixed_coord.export_volume, image_fname,
-                  verbose='error')
+    with pytest.raises(ValueError, match='must be in head coordinates'):
+        src_mixed_coord.export_volume(image_fname, verbose='error')
+
+    # now actually write it
+    fname_img = tmp_path / 'img.nii'
+    for mri_resolution in (False, 'sparse', True):
+        for src, up in ((vol, 705),
+                        (srf + vol, 27272),
+                        (disc + vol, 705)):
+            src.export_volume(
+                fname_img, use_lut=False,
+                mri_resolution=mri_resolution, overwrite=True)
+            img_data = _get_img_fdata(nib.load(str(fname_img)))
+            n_src = img_data.astype(bool).sum()
+            n_want = sum(s['nuse'] for s in src)
+            if mri_resolution is True:
+                n_want += up
+            assert n_src == n_want, src
+
+    # gh-8004
+    temp_aseg = tmp_path / 'aseg.mgz'
+    aseg_img = nib.load(aseg_fname)
+    aseg_affine = aseg_img.affine
+    aseg_affine[:3, :3] *= 0.7
+    new_aseg = nib.MGHImage(aseg_img.dataobj, aseg_affine)
+    nib.save(new_aseg, str(temp_aseg))
+    lh_cereb = mne.setup_volume_source_space(
+        "sample", mri=temp_aseg, volume_label="Left-Cerebellum-Cortex",
+        add_interpolator=False, subjects_dir=subjects_dir)
+    src = srf + lh_cereb
+    with pytest.warns(RuntimeWarning, match='2 surf vertices lay outside'):
+        src.export_volume(image_fname, mri_resolution="sparse", overwrite=True)
 
 
 @testing.requires_testing_data
@@ -787,7 +919,21 @@ def test_morphed_source_space_return():
                   src, subjects_dir=subjects_dir)
 
 
-run_tests_if_main()
+@testing.requires_testing_data
+@pytest.mark.parametrize('src, n, nv', [
+    (fname_fs, 2, 10242), (fname_src, 2, 258), (fname_vol, 0, 0),
+])
+def test_get_decimated_surfaces(src, n, nv):
+    """Test get_decimated_surfaces."""
+    surfaces = get_decimated_surfaces(src)
+    assert len(surfaces) == n
+    if n == 0:
+        return
+    for s in surfaces:
+        assert set(s) == {'rr', 'tris'}
+        assert len(s['rr']) == nv
+        assert_array_equal(np.unique(s['tris']), np.arange(nv))
+
 
 # The following code was used to generate small-src.fif.gz.
 # Unfortunately the C code bombs when trying to add source space distances,
@@ -802,7 +948,8 @@ import mne
 data_path = mne.datasets.sample.data_path()
 src = mne.setup_source_space('sample', fname=None, spacing='oct5')
 hemis = ['lh', 'rh']
-fnames = [data_path + '/subjects/sample/surf/%s.decimated' % h for h in hemis]
+fnames = [
+    str(data_path) + '/subjects/sample/surf/%s.decimated' % h for h in hemis]
 
 vs = list()
 for s, fname in zip(src, fnames):
@@ -815,7 +962,8 @@ for s, fname in zip(src, fnames):
     mne.write_surface(fname, coords, faces)
 
 # we need to move sphere surfaces
-spheres = [data_path + '/subjects/sample/surf/%s.sphere' % h for h in hemis]
+spheres = [
+    str(data_path) + '/subjects/sample/surf/%s.sphere' % h for h in hemis]
 for s in spheres:
     os.rename(s, s + '.bak')
 try:
