@@ -1,14 +1,15 @@
 """Time frequency analysis tools."""
 
 from .tfr import (morlet, tfr_morlet, AverageTFR, tfr_multitaper, _BaseTFR,
-                  read_tfrs, write_tfrs, EpochsTFR, tfr_array_morlet)
-from .psd import psd_welch, psd_multitaper, psd_array_welch
+                  read_tfrs, write_tfrs, EpochsTFR, tfr_array_morlet, fwhm)
+from .psd import psd_array_welch
 from .csd import (CrossSpectralDensity, csd_fourier, csd_multitaper,
                   csd_morlet, csd_array_fourier, csd_array_multitaper,
                   csd_array_morlet, read_csd,
-                  pick_channels_csd)
+                  pick_channels_csd, csd_tfr)
 from .ar import fit_iir_model_raw
 from .multitaper import (dpss_windows, psd_array_multitaper,
                          tfr_array_multitaper)
-from .stft import stft, istft, stftfreq
+from .spectrum import EpochsSpectrum, Spectrum, read_spectrum
+from ._stft import stft, istft, stftfreq
 from ._stockwell import tfr_stockwell, tfr_array_stockwell
