@@ -10,10 +10,11 @@ import numbers
 
 import numpy as np
 
-from .tfr import _cwt_array, morlet, _get_nfft
+from .tfr import _cwt_array, morlet, _get_nfft, EpochsTFR
 from ..io.pick import pick_channels, _picks_to_idx
 from ..utils import (logger, verbose, warn, copy_function_doc_to_method_doc,
-                     ProgressBar, _check_fname, _import_h5io_funcs)
+                     ProgressBar, _check_fname, _import_h5io_funcs,
+                     _validate_type)
 from ..viz.misc import plot_csd
 from ..time_frequency.multitaper import (_compute_mt_params, _mt_spectra,
                                          _csd_from_mt, _psd_from_mt_adaptive)
@@ -606,6 +607,12 @@ def read_csd(fname):
         fname += '.h5'
 
     csd_dict = read_hdf5(fname, title='conpy')
+
+    if csd_dict["projs"] is not None:
+        # Avoid circular import
+        from ..proj import Projection
+        csd_dict["projs"] = [Projection(**proj) for proj in csd_dict["projs"]]
+
     return CrossSpectralDensity(**csd_dict)
 
 
@@ -634,7 +641,7 @@ def csd_fourier(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None, picks=None,
         ``tmin`` and ``tmax`` will be used.
     projs : list of Projection | None
         List of projectors to store in the CSD object. Defaults to ``None``,
-        which means the projectors defined in the Epochs object will by copied.
+        which means the projectors defined in the Epochs object will be copied.
     %(n_jobs)s
     %(verbose)s
 
@@ -802,7 +809,7 @@ def csd_multitaper(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
 def csd_array_multitaper(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
                          tmax=None, ch_names=None, n_fft=None, bandwidth=None,
                          adaptive=False, low_bias=True, projs=None,
-                         n_jobs=None, *, verbose=None):
+                         n_jobs=None, max_iter=250, *, verbose=None):
     """Estimate cross-spectral density from an array using a multitaper method.
 
     Parameters
@@ -842,6 +849,7 @@ def csd_array_multitaper(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
         List of projectors to store in the CSD object. Defaults to ``None``,
         which means no projectors are stored.
     %(n_jobs)s
+    %(max_iter_multitaper)s
     %(verbose)s
 
     Returns
@@ -885,7 +893,7 @@ def csd_array_multitaper(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
     # Compute the CSD
     return _execute_csd_function(X, times, frequencies, _csd_multitaper,
                                  params=[sfreq, n_times, window_fun, eigvals,
-                                         freq_mask, n_fft, adaptive],
+                                         freq_mask, n_fft, adaptive, max_iter],
                                  n_fft=n_fft, ch_names=ch_names, projs=projs,
                                  n_jobs=n_jobs, verbose=verbose)
 
@@ -1225,37 +1233,13 @@ def _csd_fourier(X, sfreq, n_times, freq_mask, n_fft):
 
 
 def _csd_multitaper(X, sfreq, n_times, window_fun, eigvals, freq_mask, n_fft,
-                    adaptive):
-    """Compute cross spectral density (CSD) using multitaper module.
-
-    Computes the CSD for a single epoch of data.
-
-    Parameters
-    ----------
-    X : ndarray, shape (n_channels, n_times)
-        The time series data consisting of n_channels time-series of length
-        n_times.
-    sfreq : float
-        The sampling frequency of the data in Hertz.
-    n_times : int
-        Number of time samples
-    window_fun : ndarray
-        Window function(s) of length n_times. This corresponds to first output
-        of `dpss_windows`.
-    eigvals : ndarray | float
-        Eigenvalues associated with window functions.
-    freq_mask : ndarray
-        Which frequencies to use.
-    n_fft : int
-        Length of the FFT.
-    adaptive : bool
-        Use adaptive weights to combine the tapered spectra into PSD.
-    """
+                    adaptive, max_iter=250):
+    """Compute cross spectral density (CSD) using multitaper module."""
     x_mt, _ = _mt_spectra(X, window_fun, sfreq, n_fft)
 
     if adaptive:
         # Compute adaptive weights
-        _, weights = _psd_from_mt_adaptive(x_mt, eigvals, freq_mask,
+        _, weights = _psd_from_mt_adaptive(x_mt, eigvals, freq_mask, max_iter,
                                            return_weights=True)
         # Tiling weights so that we can easily use _csd_from_mt()
         weights = weights[:, np.newaxis, :, :]
@@ -1346,5 +1330,70 @@ def _csd_morlet(data, sfreq, wavelets, nfft, tslice=None, use_fft=True,
 
     # Scaling by sampling frequency for compatibility with Matlab
     csds /= sfreq
-
     return csds
+
+
+@verbose
+def csd_tfr(epochs_tfr, tmin=None, tmax=None, picks=None, projs=None,
+            verbose=None):
+    """Compute covariance matrices across frequencies for TFR epochs.
+
+    Parameters
+    ----------
+    epochs_tfr : EpochsTFR
+        The time-frequency resolved epochs over which to compute the
+        covariance.
+    tmin : float | None
+        Minimum time instant to consider, in seconds. If ``None`` start at
+        first sample.
+    tmax : float | None
+        Maximum time instant to consider, in seconds. If ``None`` end at last
+        sample.
+    %(picks_good_data_noref)s
+    projs : list of Projection | None
+        List of projectors to store in the CSD object. Defaults to ``None``,
+        which means the projectors defined in the EpochsTFR object will be
+        copied.
+    %(verbose)s
+
+    Returns
+    -------
+    res : instance of CrossSpectralDensity
+        Cross-spectral density restricted to selected channels.
+    """
+    _validate_type(epochs_tfr, EpochsTFR)
+    epochs_tfr, projs = _prepare_csd(epochs_tfr, tmin, tmax, picks, projs)
+    X = epochs_tfr.data
+    times = epochs_tfr.times
+    n_channels, n_freqs = len(epochs_tfr.ch_names), epochs_tfr.freqs.size
+    data = np.zeros((n_channels * (n_channels + 1) // 2, n_freqs),
+                    dtype=np.complex128)
+
+    # Slice X to the requested time window
+    tstart = None if tmin is None else np.searchsorted(times, tmin - 1e-10)
+    tstop = None if tmax is None else np.searchsorted(times, tmax + 1e-10)
+    X = X[:, :, :, tstart:tstop]
+
+    for idx, epochs_data in enumerate(X):
+        # This is equivalent to:
+        # csds = np.vstack([np.mean(epochs_data[[i]] * epochs_data_conj[i:],
+        #                           axis=2) for i in range(n_channels)])
+        # There is a redundancy in the calculation here because we don't really
+        # need the lower triangle of the matrix, but it should still be faster
+        # than a loop (hopefully!).
+        csds = np.einsum('xft,yft->xyf', epochs_data, np.conj(epochs_data))
+        csds = csds[np.triu_indices(n_channels) + (slice(None),)]
+        csds /= epochs_data.shape[-1]
+
+        # Scaling by sampling frequency for compatibility with Matlab
+        csds /= epochs_tfr.info['sfreq']
+        data += csds
+
+    # scale to compute mean
+    data /= len(epochs_tfr)
+
+    # TO DO: EpochTFR should store n_fft to be consistent
+    return CrossSpectralDensity(data=data, ch_names=epochs_tfr.ch_names,
+                                tmin=tmin, tmax=tmax,
+                                frequencies=epochs_tfr.freqs, n_fft=None,
+                                projs=projs)
