@@ -2,10 +2,8 @@
 """Testing functions."""
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
-from contextlib import contextmanager
-from distutils.version import LooseVersion
 from functools import partial, wraps
 import os
 import inspect
@@ -15,21 +13,15 @@ import sys
 import tempfile
 import traceback
 from unittest import SkipTest
-import warnings
 
 import numpy as np
 from numpy.testing import assert_array_equal, assert_allclose
 
 from ._logging import warn, ClosingStringIO
+from .check import check_version
 from .numerics import object_diff
 
 
-def nottest(f):
-    """Mark a function as not a test (decorator)."""
-    f.__test__ = False
-    return f
-
-
 def _explain_exception(start=-1, stop=None, prefix='> '):
     """Explain an exception."""
     # start=-1 means "only the most recent caller"
@@ -72,7 +64,7 @@ def requires_nibabel():
 def requires_dipy():
     """Check for dipy."""
     import pytest
-    # for some strange reason on CIs we cane get:
+    # for some strange reason on CIs we can get:
     #
     #     can get weird ImportError: dlopen: cannot load any more object
     #     with static TLS
@@ -83,19 +75,24 @@ def requires_dipy():
         from dipy.align.reslice import reslice  # noqa, analysis:ignore
         from dipy.align.imaffine import AffineMap  # noqa, analysis:ignore
         from dipy.align.imwarp import DiffeomorphicMap  # noqa, analysis:ignore
-    except Exception:
+    except Exception as exc:
         have = False
+        why = str(exc)
     else:
+        why = ''
         have = True
-    return pytest.mark.skipif(not have, reason='Requires dipy >= 0.10.1')
+    return pytest.mark.skipif(
+        not have, reason=f'Requires dipy >= 0.10.1, got: {why}')
 
 
 def requires_version(library, min_version='0.0'):
     """Check for a library version."""
     import pytest
+    reason = f'Requires {library}'
+    if min_version != '0.0':
+        reason += f' version >= {min_version}'
     return pytest.mark.skipif(not check_version(library, min_version),
-                              reason=('Requires %s version >= %s'
-                                      % (library, min_version)))
+                              reason=reason)
 
 
 def requires_module(function, name, call=None):
@@ -114,18 +111,6 @@ def requires_module(function, name, call=None):
     return pytest.mark.skipif(skip, reason=reason)(function)
 
 
-_pandas_call = """
-import pandas
-version = LooseVersion(pandas.__version__)
-if version < '0.8.0':
-    raise ImportError
-"""
-
-_mayavi_call = """
-with warnings.catch_warnings(record=True):  # traits
-    from mayavi import mlab
-"""
-
 _mne_call = """
 if not has_mne_c():
     raise ImportError
@@ -141,13 +126,25 @@ if 'NEUROMAG2FT_ROOT' not in os.environ:
     raise ImportError
 """
 
-requires_pandas = partial(requires_module, name='pandas', call=_pandas_call)
+requires_pandas = partial(requires_module, name='pandas')
 requires_pylsl = partial(requires_module, name='pylsl')
 requires_sklearn = partial(requires_module, name='sklearn')
-requires_mayavi = partial(requires_module, name='mayavi', call=_mayavi_call)
 requires_mne = partial(requires_module, name='MNE-C', call=_mne_call)
 
 
+def requires_mne_mark():
+    """Mark pytest tests that require MNE-C."""
+    import pytest
+    return pytest.mark.skipif(not has_mne_c(), reason='Requires MNE-C')
+
+
+def requires_openmeeg_mark():
+    """Mark pytest tests that require OpenMEEG."""
+    import pytest
+    return pytest.mark.skipif(
+        not check_version('openmeeg', '2.5.5'), reason='Requires OpenMEEG')
+
+
 def requires_freesurfer(arg):
     """Require Freesurfer."""
     if isinstance(arg, str):
@@ -168,16 +165,12 @@ run_subprocess([%r, '--version'])
 requires_neuromag2ft = partial(requires_module, name='neuromag2ft',
                                call=_n2ft_call)
 
-requires_vtk = partial(requires_module, name='vtk')
-requires_pysurfer = partial(requires_module, name='PySurfer',
-                            call="""import warnings
-with warnings.catch_warnings(record=True):
-    from surfer import Brain""")
 requires_good_network = partial(
     requires_module, name='good network connection',
     call='if int(os.environ.get("MNE_SKIP_NETWORK_TESTS", 0)):\n'
          '    raise ImportError')
 requires_nitime = partial(requires_module, name='nitime')
+# just keep this in case downstream packages need it (no coverage hit here)
 requires_h5py = partial(requires_module, name='h5py')
 
 
@@ -186,83 +179,6 @@ def requires_numpydoc(func):
     return requires_version('numpydoc', '1.0')(func)  # validate needs 1.0
 
 
-def check_version(library, min_version):
-    r"""Check minimum library version required.
-
-    Parameters
-    ----------
-    library : str
-        The library name to import. Must have a ``__version__`` property.
-    min_version : str
-        The minimum version string. Anything that matches
-        ``'(\d+ | [a-z]+ | \.)'``. Can also be empty to skip version
-        check (just check for library presence).
-
-    Returns
-    -------
-    ok : bool
-        True if the library exists with at least the specified version.
-    """
-    ok = True
-    try:
-        library = __import__(library)
-    except ImportError:
-        ok = False
-    else:
-        if min_version:
-            this_version = LooseVersion(
-                getattr(library, '__version__', '0.0').lstrip('v'))
-            if this_version < min_version:
-                ok = False
-    return ok
-
-
-def _check_mayavi_version(min_version='4.3.0'):
-    """Check mayavi version."""
-    if not check_version('mayavi', min_version):
-        raise RuntimeError("Need mayavi >= %s" % min_version)
-
-
-def _import_mlab():
-    """Quietly import mlab."""
-    with warnings.catch_warnings(record=True):
-        from mayavi import mlab
-    return mlab
-
-
-@contextmanager
-def traits_test_context():
-    """Context to raise errors in trait handlers."""
-    from traits.api import push_exception_handler
-
-    push_exception_handler(reraise_exceptions=True)
-    try:
-        yield
-    finally:
-        push_exception_handler(reraise_exceptions=False)
-
-
-def traits_test(test_func):
-    """Raise errors in trait handlers (decorator)."""
-    @wraps(test_func)
-    def dec(*args, **kwargs):
-        with traits_test_context():
-            return test_func(*args, **kwargs)
-    return dec
-
-
-@nottest
-def run_tests_if_main():
-    """Run tests in a given file if it is run as a script."""
-    local_vars = inspect.currentframe().f_back.f_locals
-    if local_vars.get('__name__', '') != '__main__':
-        return
-    import pytest
-    code = pytest.main([local_vars['__file__'], '-v'])
-    if code:
-        raise AssertionError('pytest finished with errors (%d)' % (code,))
-
-
 def run_command_if_main():
     """Run a given command if it's __main__."""
     local_vars = inspect.currentframe().f_back.f_locals
@@ -377,8 +293,8 @@ def _raw_annot(meas_date, orig_time):
     raw = RawArray(data=np.empty((10, 10)), info=info, first_samp=10)
     if meas_date is not None:
         meas_date = _handle_meas_date(meas_date)
-    raw.info['meas_date'] = meas_date
-    raw.info._check_consistency()
+    with raw.info._unlock(check_after=True):
+        raw.info['meas_date'] = meas_date
     annot = Annotations([.5], [.2], ['dummy'], orig_time)
     raw.set_annotations(annotations=annot)
     return raw
@@ -508,33 +424,6 @@ def assert_dig_allclose(info_py, info_bin, limit=None):
         assert_allclose(o_head_py, o_head_bin, rtol=1e-5, atol=1e-6)
 
 
-@contextmanager
-def modified_env(**d):
-    """Use a modified os.environ with temporarily replaced key/value pairs.
-
-    Parameters
-    ----------
-    **kwargs : dict
-        The key/value pairs of environment variables to replace.
-    """
-    orig_env = dict()
-    for key, val in d.items():
-        orig_env[key] = os.getenv(key)
-        if val is not None:
-            assert isinstance(val, str)
-            os.environ[key] = val
-        elif key in os.environ:
-            del os.environ[key]
-    try:
-        yield
-    finally:
-        for key, val in orig_env.items():
-            if val is not None:
-                os.environ[key] = val
-            elif key in os.environ:
-                del os.environ[key]
-
-
 def _click_ch_name(fig, ch_index=0, button=1):
     """Click on a channel name in a raw/epochs/ICA browse-style plot."""
     from ..viz.utils import _fake_click
@@ -545,13 +434,3 @@ def _click_ch_name(fig, ch_index=0, button=1):
     y = bbox.intervaly.mean()
     _fake_click(fig, fig.mne.ax_main, (x, y), xform='pix',
                 button=button)
-
-
-def _close_event(fig):
-    """Force calling of the MPL figure close event."""
-    # XXX workaround: plt.close() doesn't spawn close_event on Agg backend
-    # (check MPL github issue #18609; scheduled to be fixed by MPL 3.4)
-    try:
-        fig.canvas.close_event()
-    except ValueError:  # old mpl with Qt
-        pass  # pragma: no cover
