@@ -13,13 +13,13 @@ import os.path as op
 from pathlib import Path
 import re
 import sys
-import warnings
 import numbers
 
 import numpy as np
 
 from ..fixes import _median_complex, _compare_version
-from ._logging import warn, logger, verbose
+from ._logging import (warn, logger, verbose, _record_warnings,
+                       _verbose_safe_false)
 
 
 def _ensure_int(x, name='unknown', must_be='an int', *, extra=''):
@@ -273,8 +273,9 @@ def _check_preload(inst, msg):
     from ..epochs import BaseEpochs
     from ..evoked import Evoked
     from ..time_frequency import _BaseTFR
+    from ..time_frequency.spectrum import BaseSpectrum
 
-    if isinstance(inst, (_BaseTFR, Evoked)):
+    if isinstance(inst, (_BaseTFR, Evoked, BaseSpectrum)):
         pass
     else:
         name = "epochs" if isinstance(inst, BaseEpochs) else 'raw'
@@ -384,6 +385,15 @@ def _check_pybv_installed(strict=True):
     return _soft_import('pybv', 'exporting to BrainVision', strict=strict)
 
 
+def _check_pymatreader_installed(strict=True):
+    """Aux function."""
+    return _soft_import(
+        'pymatreader',
+        'loading v7.3 (HDF5) .MAT files',
+        strict=strict
+    )
+
+
 def _check_pandas_index_arguments(index, valid):
     """Check pandas index arguments."""
     if index is None:
@@ -813,6 +823,7 @@ def _check_stc_units(stc, threshold=1e-7):  # 100 nAm threshold for warning
 
 def _check_qt_version(*, return_api=False):
     """Check if Qt is installed."""
+    from ..viz.backends._utils import _init_mne_qtapp
     try:
         from qtpy import QtCore, API_NAME as api
     except Exception:
@@ -827,6 +838,13 @@ def _check_qt_version(*, return_api=False):
                 warn(f'macOS users should use {api} >= 5.10 for GUIs, '
                      f'got {version}. Please upgrade e.g. with:\n\n'
                      f'    pip install "{api}>=5.10"\n')
+        # Having Qt installed is not enough -- sometimes the app is unusable
+        # for example because there is no usable display (e.g., on a server),
+        # so we have to try instatiating one to actually know.
+        try:
+            _init_mne_qtapp()
+        except Exception:
+            api = version = None
     if return_api:
         return version, api
     else:
@@ -855,7 +873,8 @@ def _check_sphere(sphere, info=None, sphere_units='m'):
         assert info is not None
 
         if sphere == 'auto':
-            R, r0, _ = fit_sphere_to_headshape(info, verbose=False, units='m')
+            R, r0, _ = fit_sphere_to_headshape(
+                info, verbose=_verbose_safe_false(), units='m')
             sphere = tuple(r0) + (R,)
             sphere_units = 'm'
         elif sphere == 'eeglab':
@@ -947,6 +966,47 @@ def _check_sphere(sphere, info=None, sphere_units='m'):
     return sphere
 
 
+def _check_head_radius(radius, add_info=''):
+    """Check that head radius is within a reasonable range (5. - 10.85 cm).
+
+    Parameters
+    ----------
+    radius : float
+        Head radius in meters.
+    add_info : str
+        Additional info to add to the warning message.
+
+    Notes
+    -----
+    The maximum value was taken from the head size percentiles given in the
+    following Wikipedia infographic:
+    https://upload.wikimedia.org/wikipedia/commons/0/06/AvgHeadSizes.png
+
+    the maximum radius is taken from the 99th percentile for men Glabella
+    to back of the head measurements (Glabella is a point just above the
+    Nasion):
+
+        21.7cm / 2 = 10.85 cm = 0.1085 m
+
+    The minimum value was taken from The National Center for Health Statistics
+    (USA) infant head circumference percentiles:
+    https://www.cdc.gov/growthcharts/html_charts/hcageinf.htm
+    we take the minimum to be the radius corresponding to the 3rd percentile
+    head circumference of female 0-month infant, rounded down:
+    31.9302 cm circumference / (2 * pi) = 5.08 cm radius -> 0.05 m
+    """
+    min_radius = 0.05
+    max_radius = 0.1085
+    if radius > max_radius:
+        msg = (f'Estimated head radius ({1e2 * radius:0.1f} cm) is '
+               'above the 99th percentile for adult head size.')
+        warn(msg + add_info)
+    elif radius < min_radius:
+        msg = (f'Estimated head radius ({1e2 * radius:0.1f} cm) is '
+               'below the 3rd percentile for infant head size.')
+        warn(msg + add_info)
+
+
 def _check_freesurfer_home():
     from .config import get_config
     fs_home = get_config('FREESURFER_HOME')
@@ -966,9 +1026,9 @@ def _suggest(val, options, cutoff=0.66):
         return ' Did you mean one of %r?' % (options,)
 
 
-def _check_on_missing(on_missing, name='on_missing'):
+def _check_on_missing(on_missing, name='on_missing', *, extras=()):
     _validate_type(on_missing, str, name)
-    _check_option(name, on_missing, ['raise', 'warn', 'ignore'])
+    _check_option(name, on_missing, ['raise', 'warn', 'ignore'] + list(extras))
 
 
 def _on_missing(on_missing, msg, name='on_missing', error_klass=None):
@@ -997,23 +1057,16 @@ def _safe_input(msg, *, alt=None, use=None):
 
 def _ensure_events(events):
     err_msg = f'events should be a NumPy array of integers, got {type(events)}'
-    if check_version('numpy', '1.24'):
-        # NumPy >= 1.24 raises ValueError
+    with _record_warnings():
         try:
             events = np.asarray(events)
         except ValueError as np_err:
             if str(np_err).startswith(
                     'setting an array element with a sequence. The requested '
                     'array has an inhomogeneous shape'):
-                raise TypeError(err_msg)
+                raise TypeError(err_msg) from None
             else:
                 raise
-    else:
-        # NumPy < 1.24 gives VisibleDeprecationWarning
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter('ignore')
-            events = np.asarray(events)
-    # now that we're sure that events is an array...
     if not np.issubdtype(events.dtype, np.integer):
         raise TypeError(err_msg)
     if events.ndim != 2 or events.shape[1] != 3:
