@@ -2,9 +2,9 @@
 """Some miscellaneous utility functions."""
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
-from contextlib import contextmanager
+from contextlib import contextmanager, ExitStack
 import fnmatch
 import gc
 import inspect
@@ -20,14 +20,13 @@ import traceback
 import numpy as np
 
 from ..utils import _check_option, _validate_type
-from ..fixes import _get_args
 from ._logging import logger, verbose, warn
 
 
-def _pl(x, non_pl=''):
+def _pl(x, non_pl='', pl='s'):
     """Determine if plural should be used."""
     len_x = x if isinstance(x, (int, np.generic)) else len(x)
-    return non_pl if len_x == 1 else 's'
+    return non_pl if len_x == 1 else pl
 
 
 def _explain_exception(start=-1, stop=None, prefix='> '):
@@ -123,49 +122,85 @@ def run_subprocess(command, return_code=False, verbose=None, *args, **kwargs):
     # non-blocking adapted from https://stackoverflow.com/questions/375427/non-blocking-read-on-a-subprocess-pipe-in-python#4896288  # noqa: E501
     out_q = Queue()
     err_q = Queue()
+    control_stdout = 'stdout' not in kwargs
+    control_stderr = 'stderr' not in kwargs
     with running_subprocess(command, *args, **kwargs) as p:
-        out_t = Thread(target=_enqueue_output, args=(p.stdout, out_q))
-        err_t = Thread(target=_enqueue_output, args=(p.stderr, err_q))
-        out_t.daemon = True
-        err_t.daemon = True
-        out_t.start()
-        err_t.start()
+        if control_stdout:
+            out_t = Thread(target=_enqueue_output, args=(p.stdout, out_q))
+            out_t.daemon = True
+            out_t.start()
+        if control_stderr:
+            err_t = Thread(target=_enqueue_output, args=(p.stderr, err_q))
+            err_t.daemon = True
+            err_t.start()
         while True:
             do_break = p.poll() is not None
             # read all current lines without blocking
-            while True:
+            while True:  # process stdout
                 try:
                     out = out_q.get(timeout=0.01)
                 except Empty:
                     break
                 else:
                     out = out.decode('utf-8')
-                    logger.info(out)
+                    # Strip newline at end of the string, otherwise we'll end
+                    # up with two subsequent newlines (as the logger adds one)
+                    #
+                    # XXX Once we drop support for Python <3.9, uncomment the
+                    # following line and remove the if/else block below.
+                    #
+                    # log_out = out.removesuffix('\n')
+                    if sys.version_info[:2] >= (3, 9):
+                        log_out = out.removesuffix('\n')
+                    elif out.endswith('\n'):
+                        log_out = out[:-1]
+                    else:
+                        log_out = out
+
+                    logger.info(log_out)
                     all_out += out
-            while True:
+
+            while True:  # process stderr
                 try:
                     err = err_q.get(timeout=0.01)
                 except Empty:
                     break
                 else:
                     err = err.decode('utf-8')
-                    logger.warning(err)
+                    # Strip newline at end of the string, otherwise we'll end
+                    # up with two subsequent newlines (as the logger adds one)
+                    #
+                    # XXX Once we drop support for Python <3.9, uncomment the
+                    # following line and remove the if/else block below.
+                    #
+                    # err_out = err.removesuffix('\n')
+                    if sys.version_info[:2] >= (3, 9):
+                        err_out = err.removesuffix('\n')
+                    elif err.endswith('\n'):
+                        err_out = err[:-1]
+                    else:
+                        err_out = err
+
+                    # Leave this as logger.warning rather than warn(...) to
+                    # mirror the logger.info above for stdout. This function
+                    # is basically just a version of subprocess.call, and
+                    # shouldn't emit Python warnings due to stderr outputs
+                    # (the calling function can check for stderr output and
+                    # emit a warning if it wants).
+                    logger.warning(err_out)
                     all_err += err
+
             if do_break:
                 break
-    p.stdout.close()
-    p.stderr.close()
     output = (all_out, all_err)
 
     if return_code:
         output = output + (p.returncode,)
     elif p.returncode:
-        print(output)
-        err_fun = subprocess.CalledProcessError.__init__
-        if 'output' in _get_args(err_fun):
-            raise subprocess.CalledProcessError(p.returncode, command, output)
-        else:
-            raise subprocess.CalledProcessError(p.returncode, command)
+        stdout = all_out if control_stdout else None
+        stderr = all_err if control_stderr else None
+        raise subprocess.CalledProcessError(
+            p.returncode, command, output=stdout, stderr=stderr)
 
     return output
 
@@ -197,9 +232,11 @@ def running_subprocess(command, after="wait", verbose=None, *args, **kwargs):
     """
     _validate_type(after, str, 'after')
     _check_option('after', after, ['wait', 'terminate', 'kill', 'communicate'])
-    for stdxxx, sys_stdxxx in (['stderr', sys.stderr], ['stdout', sys.stdout]):
+    contexts = list()
+    for stdxxx in ('stderr', 'stdout'):
         if stdxxx not in kwargs:
             kwargs[stdxxx] = subprocess.PIPE
+            contexts.append(stdxxx)
 
     # Check the PATH environment variable. If run_subprocess() is to be called
     # frequently this should be refactored so as to only check the path once.
@@ -225,7 +262,10 @@ def running_subprocess(command, after="wait", verbose=None, *args, **kwargs):
         logger.error('Command not found: %s' % command_name)
         raise
     try:
-        yield p
+        with ExitStack() as stack:
+            for context in contexts:
+                stack.enter_context(getattr(p, context))
+            yield p
     finally:
         getattr(p, after)()
         p.wait()
@@ -322,6 +362,14 @@ def _file_like(obj):
     return all(callable(getattr(obj, name, None)) for name in ('read', 'seek'))
 
 
+def _fullname(obj):
+    klass = obj.__class__
+    module = klass.__module__
+    if module == 'builtins':
+        return klass.__qualname__
+    return module + '.' + klass.__qualname__
+
+
 def _assert_no_instances(cls, when=''):
     __tracebackhide__ = True
     n = 0
@@ -334,6 +382,9 @@ def _assert_no_instances(cls, when=''):
         except Exception:  # such as a weakref
             check = False
         if check:
+            if cls.__name__ == 'Brain':
+                ref.append(
+                    f'Brain._cleaned = {getattr(obj, "_cleaned", None)}')
             rr = gc.get_referrers(obj)
             count = 0
             for r in rr:
@@ -341,13 +392,82 @@ def _assert_no_instances(cls, when=''):
                         r is not globals() and \
                         r is not locals() and \
                         not inspect.isframe(r):
-                    if isinstance(r, (list, dict)):
+                    if isinstance(r, (list, dict, tuple)):
                         rep = f'len={len(r)}'
+                        r_ = gc.get_referrers(r)
+                        types = (_fullname(x) for x in r_)
+                        types = "/".join(sorted(set(
+                            x for x in types if x is not None)))
+                        rep += f', {len(r_)} referrers: {types}'
+                        del r_
                     else:
                         rep = repr(r)[:100].replace('\n', ' ')
-                    ref.append(f'{r.__class__.__name__}: {rep}')
+                        # If it's a __closure__, get more information
+                        if rep.startswith('<cell at '):
+                            try:
+                                rep += f' ({repr(r.cell_contents)[:100]})'
+                            except Exception:
+                                pass
+                    name = _fullname(r)
+                    ref.append(f'{name}: {rep}')
                     count += 1
                 del r
             del rr
             n += count > 0
-    assert n == 0, f'{n} {when}:\n' + '\n'.join(ref)
+        del obj
+    del objs
+    gc.collect()
+    assert n == 0, f'\n{n} {cls.__name__} @ {when}:\n' + '\n'.join(ref)
+
+
+def _resource_path(submodule, filename):
+    """Return a full system path to a package resource (AKA a file).
+
+    Parameters
+    ----------
+    submodule : str
+        An import-style module or submodule name
+        (e.g., "mne.datasets.testing").
+    filename : str
+        The file whose full path you want.
+
+    Returns
+    -------
+    path : str
+        The full system path to the requested file.
+    """
+    try:
+        from importlib.resources import files
+        return files(submodule).joinpath(filename)
+    except ImportError:
+        from pkg_resources import resource_filename
+        return resource_filename(submodule, filename)
+
+
+def repr_html(f):
+    """Decorate _repr_html_ methods.
+
+    If a _repr_html_ method is decorated with this decorator, the repr in a
+    notebook will show HTML or plain text depending on the config value
+    MNE_REPR_HTML (by default "true", which will render HTML).
+
+    Parameters
+    ----------
+    f : function
+        The function to decorate.
+
+    Returns
+    -------
+    wrapper : function
+        The decorated function.
+    """
+    from ..utils import get_config
+
+    def wrapper(*args, **kwargs):
+        if get_config("MNE_REPR_HTML", "true").lower() == "false":
+            import html
+            r = "<pre>" + html.escape(repr(args[0])) + "</pre>"
+            return r.replace("\n", "<br/>")
+        else:
+            return f(*args, **kwargs)
+    return wrapper
