@@ -3,7 +3,8 @@
 from .topomap import (plot_evoked_topomap, plot_projs_topomap, plot_arrowmap,
                       plot_ica_components, plot_tfr_topomap, plot_topomap,
                       plot_epochs_psd_topomap, plot_layout,
-                      plot_bridged_electrodes, plot_ch_adjacency)
+                      plot_bridged_electrodes, plot_ch_adjacency,
+                      plot_regression_weights)
 from .topo import plot_topo_image_epochs, iter_topography
 from .utils import (tight_layout, mne_analyze_colormap, compare_fiff,
                     ClickableImage, add_background_image, plot_sensors,
