@@ -2,6 +2,7 @@
 
 # Authors: Guillaume Favelier <guillaume.favelier@gmail.com
 #          Eric Larson <larson.eric.d@gmail.com>
+#          Alex Rockhill <aprockhill@mailbox.org>
 #
 # License: Simplified BSD
 
@@ -12,6 +13,41 @@ import warnings
 from ..utils import tight_layout
 
 
+class Figure3D(ABC):
+    """Class that refers to a 3D figure.
+
+    .. note::
+        This class should not be instantiated directly via
+        ``mne.viz.Figure3D(...)``. Instead, use
+        :func:`mne.viz.create_3d_figure`.
+
+    See Also
+    --------
+    mne.viz.create_3d_figure
+    """
+
+    # Here we use _init rather than __init__ so that users are less tempted to
+    # instantiate the class directly. It also helps us
+    # document the class more easily, as we don't have to say what all the
+    # params are in public docs.
+
+    @abstractclassmethod
+    def _init(self, fig=None, size=(600, 600), bgcolor=(0., 0., 0.),
+              name=None, show=False, shape=(1, 1), splash=False):
+        pass
+
+    @property
+    def plotter(self):
+        """The native 3D plotting widget.
+
+        Returns
+        -------
+        plotter : instance of pyvista.Plotter
+            The plotter. Useful for interacting with the native 3D library.
+        """
+        return self._plotter
+
+
 class _AbstractRenderer(ABC):
 
     @abstractclassmethod
@@ -485,6 +521,397 @@ class _AbstractRenderer(ABC):
         """
         pass
 
+# -------------------
+# Widget Abstractions
+# -------------------
+
+
+class _AbstractWidget(ABC):
+
+    @abstractclassmethod
+    def __init__(self):
+        pass
+
+    @abstractmethod
+    def _show(self):
+        pass
+
+    @abstractmethod
+    def _hide(self):
+        pass
+
+    @abstractmethod
+    def _set_enabled(self, state):
+        pass
+
+    @abstractmethod
+    def _is_enabled(self):
+        pass
+
+    @abstractmethod
+    def _update(self, repaint=True):
+        pass
+
+    @abstractmethod
+    def _set_style(self, style):
+        pass
+
+    @abstractmethod
+    def _get_tooltip(self):
+        pass
+
+    @abstractmethod
+    def _set_tooltip(self, tooltip: str):
+        pass
+
+    @abstractmethod
+    def _add_keypress(self, callback):
+        pass
+
+    @abstractmethod
+    def _trigger_keypress(self, key):
+        pass
+
+    @abstractmethod
+    def _set_focus(self):
+        pass
+
+    @abstractmethod
+    def _set_layout(self, layout):
+        pass
+
+    @abstractmethod
+    def _set_theme(self, theme):
+        pass
+
+    @abstractmethod
+    def _set_size(self, width=None, height=None):
+        pass
+
+
+class _AbstractLabel(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, center=False, selectable=False):
+        pass
+
+
+class _AbstractText(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value=None, placeholder=None, callback=None):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+
+class _AbstractButton(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, callback, icon=None):
+        pass
+
+    @abstractmethod
+    def _click(self):
+        pass
+
+    @abstractmethod
+    def _set_icon(self, icon):
+        pass
+
+
+class _AbstractSlider(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, rng, callback, horizontal=True):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+    @abstractmethod
+    def _set_range(self, rng):
+        pass
+
+
+class _AbstractProgressBar(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, count):
+        pass
+
+    @abstractmethod
+    def _increment(self):
+        pass
+
+
+class _AbstractCheckBox(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, callback):
+        pass
+
+    @abstractmethod
+    def _set_checked(self, checked):
+        pass
+
+    @abstractmethod
+    def _get_checked(self):
+        pass
+
+
+class _AbstractSpinBox(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, rng, callback, step=None):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+
+class _AbstractComboBox(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, items, callback):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+
+class _AbstractRadioButtons(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, items, callback):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+    @abstractmethod
+    def _get_value(self):
+        pass
+
+
+class _AbstractGroupBox(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, name, items):
+        pass
+
+
+class _AbstractFileButton(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, callback, content_filter=None, initial_directory=None,
+                 save=False, is_directory=False, icon='folder', window=None):
+        pass
+
+
+class _AbstractPlayMenu(_AbstractWidget):
+
+    @abstractclassmethod
+    def __init__(self, value, rng, callback):
+        pass
+
+    @abstractmethod
+    def _play(self):
+        pass
+
+    @abstractmethod
+    def _pause(self):
+        pass
+
+    @abstractmethod
+    def _reset(self):
+        pass
+
+    @abstractmethod
+    def _loop(self):
+        pass
+
+    @abstractmethod
+    def _set_value(self, value):
+        pass
+
+
+class _AbstractPopup(_AbstractWidget):
+
+    _supported_button_names = ['Ok']
+    # TODO: Add back support for below, file browser takes care of most
+    # so no big need currently
+    '''
+    # from QMessageBox.StandardButtons
+    ['Ok', 'Open', 'Save', 'Cancel', 'Close', 'Discard', 'Apply',
+    'Reset', 'RestoreDefaults', 'Help', 'SaveAll', 'Yes',
+    'YesToAll', 'No', 'NoToAll', 'Abort', 'Retry', 'Ignore']
+    '''
+
+    _supported_icon_names = ['question', 'information', 'warning', 'critical']
+
+    @abstractmethod
+    def __init__(self, title, text, info_text=None, callback=None,
+                 icon='Warning', buttons=None, window=None):
+        pass
+
+    @abstractmethod
+    def _click(self, value):
+        pass
+
+
+# -------
+# Layouts
+# -------
+
+class _AbstractBoxLayout(ABC):
+
+    @abstractmethod
+    def _add_widget(self, widget):
+        pass
+
+    @abstractmethod
+    def _add_stretch(self, amount=1):
+        pass
+
+
+class _AbstractHBoxLayout(_AbstractBoxLayout):
+
+    @abstractmethod
+    def __init__(self, height=None, scroll=None):
+        pass
+
+
+class _AbstractVBoxLayout(_AbstractBoxLayout):
+
+    @abstractmethod
+    def __init__(self, width=None, scroll=None):
+        pass
+
+
+class _AbstractGridLayout(ABC):
+
+    @abstractmethod
+    def __init__(self, height=None, width=None, scroll=None):
+        pass
+
+    @abstractmethod
+    def _add_widget(self, widget, row=None, col=None):
+        pass
+
+
+class _AbstractAppWindow(ABC):
+
+    def __init__(self, size=None, fullscreen=False):
+        pass
+
+    @abstractmethod
+    def _set_central_layout(self, central_layout):
+        pass
+
+    @abstractmethod
+    def _get_dpi(self):
+        pass
+
+    @abstractmethod
+    def _get_size(self):
+        pass
+
+    @abstractmethod
+    def _get_cursor(self):
+        pass
+
+    @abstractmethod
+    def _set_cursor(self, cursor):
+        pass
+
+    @abstractmethod
+    def _new_cursor(self, name):
+        pass
+
+    @abstractmethod
+    def _close_connect(self, func, *, after=True):
+        pass
+
+    @abstractmethod
+    def _close_disconnect(self, after=True):
+        pass
+
+    @abstractmethod
+    def _clean(self):
+        pass
+
+    @abstractmethod
+    def _show(self, block=False):
+        pass
+
+    @abstractmethod
+    def _close(self):
+        pass
+
+
+# -------------------
+# Matplotlib Canvases
+# -------------------
+
+
+class _AbstractCanvas(ABC):
+
+    def __init__(self, width=None, height=None, dpi=None):
+        """Initialize the matplotlib Canvas."""
+        pass
+
+    def show(self):
+        """Show the canvas."""
+        if self.manager is None:
+            self.show()
+        else:
+            self.manager.show()
+
+    def close(self):
+        """Close the canvas."""
+        self.close()
+
+    def update(self):
+        """Update the canvas."""
+        self.fig.canvas.draw()
+        self.fig.canvas.flush_events()
+
+    def clear(self):
+        """Clear internal variables."""
+        self.close()
+        self.ax.clear()
+        self.fig.clear()
+        self.manager = None
+
+    @abstractmethod
+    def _set_size(self, width=None, height=None):
+        pass
+
+# ------------------------------------
+# Non-object-based Widget Abstractions
+# ------------------------------------
+# These are planned to be deprecated in favor of the simpler, object-
+# oriented abstractions above when time allows.
+
 
 class _AbstractToolBar(ABC):
     @abstractmethod
@@ -693,7 +1120,7 @@ class _AbstractWidgetList(ABC):
         pass
 
 
-class _AbstractWidget(ABC):
+class _AbstractWdgt(ABC):
     def __init__(self, widget):
         self._widget = widget
 
@@ -960,33 +1387,3 @@ class _AbstractWindow(ABC):
     @abstractmethod
     def _window_create(self):
         pass
-
-
-class Figure3D(ABC):
-    """Class that refers to a 3D figure.
-
-    .. note::
-        This class is not meant to be instantiated directly, use
-        :func:`mne.viz.create_3d_figure` instead.
-    """
-
-    # Here we use _init rather than __init__ so that users are less tempted to
-    # instantiate the class directly. It also helps us
-    # document the class more easily, as we don't have to say what all the
-    # params are in public docs.
-
-    @abstractclassmethod
-    def _init(self, fig=None, size=(600, 600), bgcolor=(0., 0., 0.),
-              name=None, show=False, shape=(1, 1), splash=False):
-        pass
-
-    @property
-    def plotter(self):
-        """The native 3D plotting widget.
-
-        Returns
-        -------
-        plotter : instance of pyvista.Plotter
-            The plotter. Useful for interacting with the native 3D library.
-        """
-        return self._plotter
