@@ -12,6 +12,7 @@ Actual implementation of _Renderer and _Projection classes.
 # License: Simplified BSD
 
 from contextlib import contextmanager
+from inspect import signature
 import os
 import re
 import sys
@@ -22,10 +23,10 @@ import numpy as np
 from ._abstract import _AbstractRenderer, Figure3D
 from ._utils import (_get_colormap_from_array, _alpha_blend_background,
                      ALLOWED_QUIVER_MODES, _init_mne_qtapp)
-from ...fixes import _get_args, _point_data, _cell_data, _compare_version
+from ...fixes import _compare_version
 from ...transforms import apply_trans
 from ...utils import (copy_base_doc_to_subclass_doc, _check_option,
-                      _require_version, _validate_type)
+                      _require_version, _validate_type, warn)
 
 
 with warnings.catch_warnings():
@@ -69,8 +70,13 @@ _FIGURES = dict()
 class PyVistaFigure(Figure3D):
     """PyVista-based 3D Figure.
 
-    This class is not meant to be instantiated directly, use
-    :func:`mne.viz.create_3d_figure` instead.
+    .. note:: This class should not be instantiated directly via
+              ``mne.viz.PyVistaFigure(...)``. Instead, use
+              :func:`mne.viz.create_3d_figure`.
+
+    See Also
+    --------
+    mne.viz.create_3d_figure
     """
 
     def __init__(self):
@@ -103,7 +109,7 @@ class PyVistaFigure(Figure3D):
             self.store['toolbar'] = False
             self.store['update_app_icon'] = False
             self._plotter_class = BackgroundPlotter
-            if 'app_window_class' in _get_args(BackgroundPlotter):
+            if 'app_window_class' in signature(BackgroundPlotter).parameters:
                 from ._qt import _MNEMainWindow
                 self.store['app_window_class'] = _MNEMainWindow
         else:
@@ -195,8 +201,7 @@ class _PyVistaRenderer(_AbstractRenderer):
         self.tube_n_sides = 20
         self.antialias = _get_3d_option('antialias')
         self.depth_peeling = _get_3d_option('depth_peeling')
-        # smooth_shading=True fails on MacOS CIs
-        self.smooth_shading = _get_3d_option('smooth_shading')
+        self.smooth_shading = smooth_shading
         if isinstance(fig, int):
             saved_fig = _FIGURES.get(fig)
             # Restore only active plotter
@@ -339,20 +344,23 @@ class _PyVistaRenderer(_AbstractRenderer):
                 from matplotlib.colors import ListedColormap
                 colormap = ListedColormap(colormap)
             if normals is not None:
-                _point_data(mesh)["Normals"] = normals
+                mesh.point_data["Normals"] = normals
                 mesh.GetPointData().SetActiveNormals("Normals")
             else:
                 _compute_normals(mesh)
             if 'rgba' in kwargs:
                 rgba = kwargs["rgba"]
                 kwargs.pop('rgba')
+            smooth_shading = self.smooth_shading
+            if representation == 'wireframe':
+                smooth_shading = False  # never use smooth shading for wf
             actor = _add_mesh(
                 plotter=self.plotter,
-                mesh=mesh, color=color, scalars=scalars,
+                mesh=mesh, color=color, scalars=scalars, edge_color=color,
                 rgba=rgba, opacity=opacity, cmap=colormap,
                 backface_culling=backface_culling,
                 rng=[vmin, vmax], show_scalar_bar=False,
-                smooth_shading=self.smooth_shading,
+                smooth_shading=smooth_shading,
                 interpolate_before_map=interpolate_before_map,
                 style=representation, line_width=line_width, **kwargs,
             )
@@ -405,7 +413,7 @@ class _PyVistaRenderer(_AbstractRenderer):
             n_triangles = len(triangles)
             triangles = np.c_[np.full(n_triangles, 3), triangles]
             mesh = PolyData(vertices, triangles)
-            _point_data(mesh)['scalars'] = scalars
+            mesh.point_data['scalars'] = scalars
             contour = mesh.contour(isosurfaces=contours)
             line_width = width
             if kind == 'tube':
@@ -420,7 +428,7 @@ class _PyVistaRenderer(_AbstractRenderer):
                 rng=[vmin, vmax],
                 cmap=colormap,
                 opacity=opacity,
-                smooth_shading=self.smooth_shading
+                smooth_shading=self.smooth_shading,
             )
             return actor, contour
 
@@ -437,7 +445,7 @@ class _PyVistaRenderer(_AbstractRenderer):
             mesh = PolyData(vertices, triangles)
         colormap = _get_colormap_from_array(colormap, normalized_colormap)
         if scalars is not None:
-            _point_data(mesh)['scalars'] = scalars
+            mesh.point_data['scalars'] = scalars
         return self.polydata(
             mesh=mesh,
             color=color,
@@ -490,7 +498,7 @@ class _PyVistaRenderer(_AbstractRenderer):
             for (pointa, pointb) in zip(origin, destination):
                 line = Line(pointa, pointb)
                 if scalars is not None:
-                    _point_data(line)['scalars'] = scalars[0, :]
+                    line.point_data['scalars'] = scalars[0, :]
                     scalars = 'scalars'
                     color = None
                 else:
@@ -530,8 +538,8 @@ class _PyVistaRenderer(_AbstractRenderer):
             grid = UnstructuredGrid(*args)
             if scalars is None:
                 scalars = np.ones((n_points,))
-            _point_data(grid)['scalars'] = np.array(scalars)
-            _point_data(grid)['vec'] = vectors
+            grid.point_data['scalars'] = np.array(scalars)
+            grid.point_data['vec'] = vectors
             if mode == '2darrow':
                 return _arrow_glyph(grid, factor), grid
             elif mode == 'arrow':
@@ -634,7 +642,8 @@ class _PyVistaRenderer(_AbstractRenderer):
                 name=text,
                 shape_opacity=0,
             )
-            if 'always_visible' in _get_args(self.plotter.add_point_labels):
+            if ('always_visible'
+                    in signature(self.plotter.add_point_labels).parameters):
                 kwargs['always_visible'] = True
             actor = self.plotter.add_point_labels(**kwargs)
         _hide_testing_actor(actor)
@@ -767,7 +776,7 @@ class _PyVistaRenderer(_AbstractRenderer):
         # issubdtype from `complex` to `np.complexfloating` is deprecated.
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore", category=FutureWarning)
-            _point_data(mesh)[name] = scalars
+            mesh.point_data[name] = scalars
 
     def _set_colormap_range(self, actor, ctable, scalar_bar, rng=None,
                             background_color=None):
@@ -826,7 +835,7 @@ class _PyVistaRenderer(_AbstractRenderer):
         grid.dimensions = dimensions + 1  # inject data on the cells
         grid.origin = origin
         grid.spacing = spacing
-        _cell_data(grid)['values'] = scalars
+        grid.cell_data['values'] = scalars
 
         # Add contour of enclosed volume (use GetOutput instead of
         # GetOutputPort below to avoid updating)
@@ -908,7 +917,7 @@ class _PyVistaRenderer(_AbstractRenderer):
 
 def _compute_normals(mesh):
     """Patch PyVista compute_normals."""
-    if 'Normals' not in _point_data(mesh):
+    if 'Normals' not in mesh.point_data:
         mesh.compute_normals(
             cell_normals=False,
             consistent_normals=False,
@@ -929,7 +938,7 @@ def _add_mesh(plotter, *args, **kwargs):
     if 'render' not in kwargs:
         kwargs['render'] = False
     actor = plotter.add_mesh(*args, **kwargs)
-    if smooth_shading and 'Normals' in _point_data(mesh):
+    if smooth_shading and 'Normals' in mesh.point_data:
         prop = actor.GetProperty()
         prop.SetInterpolationToPhong()
     _hide_testing_actor(actor)
@@ -1010,7 +1019,7 @@ def _set_3d_view(figure, azimuth=None, elevation=None, focalpoint='auto',
     position = np.array(figure.plotter.camera_position[0])
     bounds = np.array(figure.plotter.renderer.ComputeVisiblePropBounds())
     if reset_camera:
-        figure.plotter.reset_camera()
+        figure.plotter.reset_camera(render=False)
 
     # focalpoint: if 'auto', we use the center of mass of the visible
     # bounds, if None, we use the existing camera focal point otherwise
@@ -1094,7 +1103,7 @@ def _close_3d_figure(figure):
         _process_events(plotter)
         # free memory and deregister from the scraper
         plotter.deep_clean()  # remove internal references
-        del _ALL_PLOTTERS[plotter._id_name]
+        _ALL_PLOTTERS.pop(plotter._id_name, None)
         _process_events(plotter)
 
 
@@ -1195,6 +1204,21 @@ def _is_mesa(plotter):
     # MESA (could use GPUInfo / _get_gpu_info here, but it takes
     # > 700 ms to make a new window + report capabilities!)
     # CircleCI's is: "Mesa 20.0.8 via llvmpipe (LLVM 10.0.0, 256 bits)"
-    gpu_info = plotter.ren_win.ReportCapabilities()
-    gpu_info = re.findall("OpenGL renderer string:(.+)\n", gpu_info)
-    return ' mesa ' in ' '.join(gpu_info).lower().split()
+    gpu_info_full = plotter.ren_win.ReportCapabilities()
+    gpu_info = re.findall("OpenGL renderer string:(.+)\n", gpu_info_full)
+    gpu_info = ' '.join(gpu_info).lower()
+    is_mesa = 'mesa' in gpu_info.split()
+    if is_mesa:
+        # Try to warn if it's ancient
+        version = re.findall("mesa ([0-9.]+) .*", gpu_info) or \
+            re.findall("OpenGL version string: .* Mesa ([0-9.]+)\n",
+                       gpu_info_full)
+        if version:
+            version = version[0]
+            if _compare_version(version, '<', '18.3.6'):
+                warn(f'Mesa version {version} is too old for translucent 3D '
+                     'surface rendering, consider upgrading to 18.3.6 or '
+                     'later.')
+        else:
+            raise RuntimeError
+    return is_mesa
