@@ -10,7 +10,7 @@ import warnings
 
 
 def has_pyvista():
-    """Check that pyvista is installed."""
+    """Check that PyVista is installed."""
     try:
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore", category=DeprecationWarning)
@@ -20,20 +20,12 @@ def has_pyvista():
         return False
 
 
-def has_mayavi():
-    """Check that mayavi is installed."""
+def has_pyvistaqt():
+    """Check that PyVistaQt is installed."""
     try:
-        with warnings.catch_warnings(record=True):  # traits
-            from mayavi import mlab  # noqa: F401
-        return True
-    except ImportError:
-        return False
-
-
-def has_pyqt5():
-    """Check if PyQt5 is installed."""
-    try:
-        import PyQt5  # noqa: F401
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", category=DeprecationWarning)
+            import pyvistaqt  # noqa: F401
         return True
     except ImportError:
         return False
@@ -48,7 +40,5 @@ def has_imageio_ffmpeg():
         return False
 
 
-skips_if_not_mayavi = pytest.mark.skipif(
-    not has_mayavi(), reason='requires mayavi')
-skips_if_not_pyvista = pytest.mark.skipif(
-    not has_pyvista(), reason='requires pyvista')
+skips_if_not_pyvistaqt = pytest.mark.skipif(
+    not has_pyvistaqt(), reason='requires pyvistaqt')
