@@ -16,8 +16,8 @@ from numpy.testing import assert_allclose
 import pytest
 import matplotlib.pyplot as plt
 from matplotlib import gridspec
-from matplotlib.cm import get_cmap
 from matplotlib.collections import PolyCollection
+from mpl_toolkits.axes_grid1.parasite_axes import HostAxes  # spatial_colors
 
 import mne
 from mne import (read_events, Epochs, read_cov, compute_covariance,
@@ -25,7 +25,7 @@ from mne import (read_events, Epochs, read_cov, compute_covariance,
 from mne.io import read_raw_fif
 from mne.utils import catch_logging
 from mne.viz import plot_compare_evokeds, plot_evoked_white
-from mne.viz.utils import _fake_click
+from mne.viz.utils import _fake_click, _get_cmap
 from mne.datasets import testing
 from mne.io.constants import FIFF
 from mne.stats.parametric import _parametric_ci
@@ -175,7 +175,8 @@ def test_plot_evoked():
         [(0, 0.1), (0.1, 0.2)]
     ]:
         fig = evoked.plot(time_unit='s', highlight=highlight)
-        for ax in fig.get_axes():
+        regular_axes = [ax for ax in fig.axes if not isinstance(ax, HostAxes)]
+        for ax in regular_axes:
             highlighted_areas = [child for child in ax.get_children()
                                  if isinstance(child, PolyCollection)]
             assert len(highlighted_areas) == len(np.atleast_2d(highlight))
@@ -183,6 +184,13 @@ def test_plot_evoked():
     with pytest.raises(ValueError, match='must be reshapable into a 2D array'):
         fig = evoked.plot(time_unit='s', highlight=0.1)
 
+    # set one channel location to nan, confirm spatial_colors still works
+    evoked = _get_epochs().load_data().average('grad')  # reload data
+    evoked.info['chs'][0]['loc'][:] = np.nan
+    fig = evoked.plot(time_unit='s', spatial_colors=True)
+    line_clr = [x.get_color() for x in fig.axes[0].get_lines()]
+    assert not np.all(np.isnan(line_clr) & (line_clr == 0))
+
 
 def test_constrained_layout():
     """Test that we handle constrained layouts correctly."""
@@ -190,14 +198,19 @@ def test_constrained_layout():
     assert fig.get_constrained_layout()
     evoked = mne.read_evokeds(evoked_fname)[0]
     evoked.pick(evoked.ch_names[:2])
-    evoked.plot(axes=ax)  # smoke test that it does not break things
+
+    # smoke test that it does not break things
+    evoked.plot(axes=ax)
     assert fig.get_constrained_layout()
     plt.close('all')
 
 
 def _get_amplitudes(fig):
-    amplitudes = [line.get_ydata() for ax in fig.axes
+    # ignore the spatial_colors parasite axes
+    regular_axes = [ax for ax in fig.axes if not isinstance(ax, HostAxes)]
+    amplitudes = [line.get_ydata() for ax in regular_axes
                   for line in ax.get_lines()]
+    # this will exclude hlines, which are lists not arrays
     amplitudes = np.array(
         [line for line in amplitudes if isinstance(line, np.ndarray)])
     return amplitudes
@@ -358,6 +371,7 @@ def test_plot_compare_evokeds():
         assert fig[0].axes[0].get_title().endswith(_t)
     # test passing more than one evoked
     red, blue = evoked.copy(), evoked.copy()
+    red.comment = red.comment + '*' * 100
     red.data *= 1.5
     blue.data /= 1.5
     evoked_dict = {'aud/l': blue, 'aud/r': red, 'vis': evoked}
@@ -410,7 +424,7 @@ def test_plot_compare_evokeds():
     plot_compare_evokeds(evoked_dict, colors=list(range(3)))
     plt.close('all')
     # test colormap
-    cmap = get_cmap('viridis')
+    cmap = _get_cmap('viridis')
     plot_compare_evokeds(evoked_dict, cmap=cmap, colors=dict(aud=0.4, vis=0.9))
     plot_compare_evokeds(evoked_dict, cmap=cmap, colors=dict(aud=1, vis=2))
     plot_compare_evokeds(evoked_dict, cmap=('cmap title', 'inferno'),
@@ -469,8 +483,11 @@ def test_plot_compare_evokeds():
     plot_compare_evokeds(csd, picks='csd', axes='topo')
     # old tests
     red.info['chs'][0]['loc'][:2] = 0  # test plotting channel at zero
-    plot_compare_evokeds([red, blue], picks=[0],
-                         ci=lambda x: [x.std(axis=0), -x.std(axis=0)])
+    fig, = plot_compare_evokeds(
+        [red, blue], picks=[0], ci=lambda x: [x.std(axis=0), -x.std(axis=0)])
+    # reasonable legend lengths
+    leg_texts = [t.get_text() for t in fig.axes[0].get_legend().get_texts()]
+    assert all(len(lt) < 50 for lt in leg_texts)
     plot_compare_evokeds([list(evoked_dict.values())], picks=[0],
                          ci=_parametric_ci)
     # smoke test for tmin >= 0 (from mailing list)
