@@ -3,11 +3,12 @@
 # License: Simplified BSD
 
 import os.path as op
-
+import pytest
 import mne
 from mne.utils import requires_version
 
 
+@pytest.mark.pgtest
 @requires_version('sphinx_gallery')
 def test_qt_scraper(raw, pg_backend, tmp_path):
     """Test sphinx-gallery scraping of the browser."""
