@@ -9,20 +9,20 @@
 
 from copy import deepcopy
 from functools import partial
-from itertools import cycle
 
 import numpy as np
 
 from ..io.pick import channel_type, pick_types
-from ..utils import _clean_names, warn, _check_option, Bunch
+from ..utils import _clean_names, _check_option, Bunch, fill_doc, _to_rgb
 from ..channels.layout import _merge_ch_data, _pair_grad_sensors, find_layout
 from ..defaults import _handle_default
-from .utils import (_check_delayed_ssp, _get_color_list, _draw_proj_checkbox,
+from .utils import (_check_delayed_ssp, _draw_proj_checkbox,
                     add_background_image, plt_show, _setup_vmin_vmax,
                     DraggableColorbar, _setup_ax_spines,
                     _check_cov, _plot_masked_image)
 
 
+@fill_doc
 def iter_topography(info, layout=None, on_pick=None, fig=None,
                     fig_facecolor='k', axis_facecolor='k',
                     axis_spinecolor='k', layout_scale=None, legend=False):
@@ -36,8 +36,7 @@ def iter_topography(info, layout=None, on_pick=None, fig=None,
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement info.
+    %(info_not_none)s
     layout : instance of mne.channels.Layout | None
         The layout to use. If None, layout will be guessed.
     on_pick : callable | None
@@ -81,8 +80,18 @@ def _legend_axis(pos):
     """Add a legend axis to the bottom right."""
     import matplotlib.pyplot as plt
     left, bottom = pos[:, 0].max(), pos[:, 1].min()
+    # check if legend axis overlaps a data axis
+    overlaps = False
+    for _pos in pos:
+        h_overlap = (_pos[0] <= left <= (_pos[0] + _pos[2]))
+        v_overlap = (_pos[1] <= bottom <= (_pos[1] + _pos[3]))
+        if h_overlap and v_overlap:
+            overlaps = True
+            break
+    if overlaps:
+        left += 1.2 * _pos[2]
     wid, hei = pos[-1, 2:]
-    return plt.axes([left, bottom + .05, wid, hei])
+    return plt.axes([left, bottom, wid, hei])
 
 
 def _iter_topography(info, layout, on_pick, fig, fig_facecolor='k',
@@ -305,7 +314,7 @@ def _imshow_tfr(ax, ch_idx, tmin, tmax, vmin, vmax, onselect, ylim=None,
                 tfr=None, freq=None, x_label=None, y_label=None,
                 colorbar=False, cmap=('RdBu_r', True), yscale='auto',
                 mask=None, mask_style="both", mask_cmap="Greys",
-                mask_alpha=0.1, is_jointplot=False):
+                mask_alpha=0.1, is_jointplot=False, cnorm=None):
     """Show time-frequency map as two-dimensional image."""
     from matplotlib import pyplot as plt
     from matplotlib.widgets import RectangleSelector
@@ -318,7 +327,7 @@ def _imshow_tfr(ax, ch_idx, tmin, tmax, vmin, vmax, onselect, ylim=None,
     img, t_end = _plot_masked_image(
         ax, tfr[ch_idx], times, mask, yvals=freq, cmap=cmap,
         vmin=vmin, vmax=vmax, mask_style=mask_style, mask_alpha=mask_alpha,
-        mask_cmap=mask_cmap, yscale=yscale)
+        mask_cmap=mask_cmap, yscale=yscale, cnorm=cnorm)
 
     if x_label is not None:
         ax.set_xlabel(x_label)
@@ -357,7 +366,6 @@ def _plot_timeseries(ax, ch_idx, tmin, tmax, vmin, vmax, ylim, data, color,
                      labels=None):
     """Show time series on topo split across multiple axes."""
     import matplotlib.pyplot as plt
-    from matplotlib.colors import colorConverter
     picker_flag = False
     for data_, color_, times_ in zip(data, color, times):
         if not picker_flag:
@@ -421,11 +429,8 @@ def _plot_timeseries(ax, ch_idx, tmin, tmax, vmin, vmax, ylim, data, color,
 
     ax._cursorline = None
     # choose cursor color based on perceived brightness of background
-    try:
-        facecol = colorConverter.to_rgb(ax.get_facecolor())
-    except AttributeError:  # older MPL
-        facecol = colorConverter.to_rgb(ax.get_axis_bgcolor())
-    face_brightness = np.dot(facecol, np.array([299, 587, 114]))
+    facecol = _to_rgb(ax.get_facecolor())
+    face_brightness = np.dot(facecol, [299, 587, 114])
     ax._cursorcolor = 'white' if face_brightness < 150 else 'black'
 
     plt.connect('motion_notify_event', _cursor_vline)
@@ -656,21 +661,6 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945,
     if not type(evoked) in (tuple, list):
         evoked = [evoked]
 
-    if type(color) in (tuple, list):
-        if len(color) != len(evoked):
-            raise ValueError('Lists of evoked objects and colors'
-                             ' must have the same length')
-    elif color is None:
-        colors = ['w'] + _get_color_list
-        stop = (slice(len(evoked)) if len(evoked) < len(colors)
-                else slice(len(colors)))
-        color = cycle(colors[stop])
-        if len(evoked) > len(colors):
-            warn('More evoked objects than colors available. You should pass '
-                 'a list of unique colors.')
-    else:
-        color = cycle([color])
-
     noise_cov = _check_cov(noise_cov, evoked[0].info)
     if noise_cov is not None:
         evoked = [whiten_evoked(e, noise_cov) for e in evoked]
@@ -689,10 +679,9 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945,
             ch = info['chs'][pick]
             ch['ch_name'] = ch['ch_name'][:-1] + 'X'
             chs.append(ch)
-        info['chs'] = chs
-        info['bads'] = list()   # Bads handled by pair_grad_sensors
-        info._update_redundant()
-        info._check_consistency()
+        with info._unlock(update_redundant=True, check_after=True):
+            info['chs'] = chs
+            info['bads'] = list()   # Bads handled by pair_grad_sensors
         new_picks = list()
         for e in evoked:
             data, _ = _merge_ch_data(e.data[picks], 'grad', [])
@@ -805,8 +794,12 @@ def _plot_evoked_topo(evoked, layout=None, layout_scale=0.945,
     if legend is not False:
         legend_loc = 0 if legend is True else legend
         labels = [e.comment if e.comment else 'Unknown' for e in evoked]
-        legend = plt.legend(labels, loc=legend_loc,
-                            prop={'size': 10})
+        handles = fig.axes[0].lines[:len(evoked)]
+        legend = plt.legend(
+            labels=labels,
+            handles=handles,
+            loc=legend_loc,
+            prop={'size': 10})
         legend.get_frame().set_facecolor(axis_facecolor)
         txts = legend.get_texts()
         for txt, col in zip(txts, color):
