@@ -13,7 +13,7 @@ including how to load, query, subset, export, and plot data from an
 object from (possibly simulated) data in a :class:`NumPy array
 <numpy.ndarray>`, see :ref:`tut-creating-data-structures`.
 
-As usual, we'll start by importing the modules we need:
+As usual, we start by importing the modules we need:
 """
 
 # %%
@@ -258,7 +258,7 @@ evokeds_list[0].plot(picks='eeg')
 # :ref:`tut-section-subselect-epochs` for more information):
 
 left_right_aud = epochs['auditory'].average()
-print(left_right_aud)
+left_right_aud
 
 # %%
 # This approach will weight each epoch equally and create a single
