@@ -2,6 +2,7 @@
 """
 .. _tut-artifact-ica:
 
+============================
 Repairing artifacts with ICA
 ============================
 
@@ -18,18 +19,22 @@ repeatedly typing ``mne.preprocessing`` we'll directly import a few functions
 and classes from that submodule:
 """
 
+# %%
+
 import os
 import mne
-from mne.preprocessing import (ICA, create_eog_epochs, create_ecg_epochs,
-                               corrmap)
+from mne.preprocessing import (ICA, corrmap, create_ecg_epochs,
+                               create_eog_epochs)
 
 sample_data_folder = mne.datasets.sample.data_path()
 sample_data_raw_file = os.path.join(sample_data_folder, 'MEG', 'sample',
-                                    'sample_audvis_raw.fif')
+                                    'sample_audvis_filt-0-40_raw.fif')
 raw = mne.io.read_raw_fif(sample_data_raw_file)
-raw.crop(tmax=60.)
+# Here we'll crop to 60 seconds and drop gradiometer channels for speed
+raw.crop(tmax=60.).pick_types(meg='mag', eeg=True, stim=True, eog=True)
+raw.load_data()
 
-###############################################################################
+# %%
 # .. note::
 #     Before applying ICA (or any artifact repair strategy), be sure to observe
 #     the artifacts in your data to make sure you choose the right repair tool.
@@ -62,7 +67,8 @@ raw.crop(tmax=60.)
 # ICA in MNE-Python
 # ~~~~~~~~~~~~~~~~~
 #
-# .. sidebar:: ICA and dimensionality reduction
+# .. admonition:: ICA and dimensionality reduction
+#     :class: sidebar hint
 #
 #     If you want to perform ICA with *no* dimensionality reduction (other than
 #     the number of Independent Components (ICs) given in ``n_components``, and
@@ -169,7 +175,7 @@ artifact_picks = mne.pick_channels_regexp(raw.ch_names, regexp=regexp)
 raw.plot(order=artifact_picks, n_channels=len(artifact_picks),
          show_scrollbars=False)
 
-###############################################################################
+# %%
 # We can get a summary of how the ocular artifact manifests across each channel
 # type using `~mne.preprocessing.create_eog_epochs` like we did in the
 # :ref:`tut-artifact-overview` tutorial:
@@ -178,7 +184,7 @@ eog_evoked = create_eog_epochs(raw).average()
 eog_evoked.apply_baseline(baseline=(None, -0.2))
 eog_evoked.plot_joint()
 
-###############################################################################
+# %%
 # Now we'll do the same for the heartbeat artifacts, using
 # `~mne.preprocessing.create_ecg_epochs`:
 
@@ -186,7 +192,7 @@ ecg_evoked = create_ecg_epochs(raw).average()
 ecg_evoked.apply_baseline(baseline=(None, -0.2))
 ecg_evoked.plot_joint()
 
-###############################################################################
+# %%
 # Filtering to remove slow drifts
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -204,14 +210,14 @@ ecg_evoked.plot_joint()
 # `~mne.io.Raw` object around so we can apply the ICA solution to it
 # later.
 
-filt_raw = raw.copy()
-filt_raw.load_data().filter(l_freq=1., h_freq=None)
+filt_raw = raw.copy().filter(l_freq=1., h_freq=None)
 
-###############################################################################
-# Fitting and plotting the ICA solution
-# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+# %%
+# Fitting ICA
+# ~~~~~~~~~~~
 #
-# .. sidebar:: Ignoring the time domain
+# .. admonition:: Ignoring the time domain
+#     :class: sidebar hint
 #
 #     The ICA algorithms implemented in MNE-Python find patterns across
 #     channels, but ignore the time domain. This means you can compute ICA on
@@ -219,12 +225,6 @@ filt_raw.load_data().filter(l_freq=1., h_freq=None)
 #     just continuous `~mne.io.Raw` objects), or only use every Nth
 #     sample by passing the ``decim`` parameter to ``ICA.fit()``.
 #
-#     .. note:: `~mne.Epochs` used for fitting ICA should not be
-#               baseline-corrected. Because cleaning the data via ICA may
-#               introduce DC offsets, we suggest to baseline correct your data
-#               **after** cleaning (and not before), should you require
-#               baseline correction.
-#
 # Now we're ready to set up and fit the ICA. Since we know (from observing our
 # raw data) that the EOG and ECG artifacts are fairly strong, we would expect
 # those artifacts to be captured in the first few dimensions of the PCA
@@ -242,27 +242,74 @@ filt_raw.load_data().filter(l_freq=1., h_freq=None)
 # flip on different runs, or may not always be returned in the same order), so
 # we'll also specify a `random seed`_ so that we get identical results each
 # time this tutorial is built by our web servers.
+#
+# .. warning::
+#
+#     `~mne.Epochs` used for fitting ICA should not be
+#     baseline-corrected. Because cleaning the data via ICA may
+#     introduce DC offsets, we suggest to baseline correct your data
+#     **after** cleaning (and not before), should you require
+#     baseline correction.
 
 ica = ICA(n_components=15, max_iter='auto', random_state=97)
 ica.fit(filt_raw)
+ica
 
-###############################################################################
+# %%
 # Some optional parameters that we could have passed to the
 # `~mne.preprocessing.ICA.fit` method include ``decim`` (to use only
 # every Nth sample in computing the ICs, which can yield a considerable
 # speed-up) and ``reject`` (for providing a rejection dictionary for maximum
 # acceptable peak-to-peak amplitudes for each channel type, just like we used
 # when creating epoched data in the :ref:`tut-overview` tutorial).
-#
+
+# %%
+# Looking at the ICA solution
+# ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 # Now we can examine the ICs to see what they captured.
+#
+# Using :meth:`~mne.preprocessing.ICA.get_explained_variance_ratio`, we can
+# retrieve the fraction of variance in the original data that is explained by
+# our ICA components in the form of a dictionary:
+
+explained_var_ratio = ica.get_explained_variance_ratio(filt_raw)
+for channel_type, ratio in explained_var_ratio.items():
+    print(
+        f'Fraction of {channel_type} variance explained by all components: '
+        f'{ratio}'
+    )
+
+# %%
+# The values were calculated for all ICA components jointly, but separately for
+# each channel type (here: magnetometers and EEG).
+#
+# We can also explicitly request for which component(s) and channel type(s) to
+# perform the computation:
+explained_var_ratio = ica.get_explained_variance_ratio(
+    filt_raw,
+    components=[0],
+    ch_type='eeg'
+)
+# This time, print as percentage.
+ratio_percent = round(100 * explained_var_ratio['eeg'])
+print(
+    f'Fraction of variance in EEG signal explained by first component: '
+    f'{ratio_percent}%'
+)
+
+# %%
 # `~mne.preprocessing.ICA.plot_sources` will show the time series of the
 # ICs. Note that in our call to `~mne.preprocessing.ICA.plot_sources` we
-# can use the original, unfiltered `~mne.io.Raw` object:
+# can use the original, unfiltered `~mne.io.Raw` object. A helpful tip is that
+# right clicking (or control + click with a trackpad) on the name of the
+# component will bring up a plot of its properties. In this plot, you can
+# also toggle the channel type in the topoplot (if you have multiple channel
+# types) with 't' and whether the spectrum is log-scaled or not with 'l'.
 
 raw.load_data()
 ica.plot_sources(raw, show_scrollbars=False)
 
-###############################################################################
+# %%
 # Here we can pretty clearly see that the first component (``ICA000``) captures
 # the EOG signal quite well, and the second component (``ICA001``) looks a lot
 # like `a heartbeat <qrs_>`_ (for more info on visually identifying Independent
@@ -274,7 +321,7 @@ ica.plot_sources(raw, show_scrollbars=False)
 # sphinx_gallery_thumbnail_number = 9
 ica.plot_components()
 
-###############################################################################
+# %%
 # .. note::
 #
 #     `~mne.preprocessing.ICA.plot_components` (which plots the scalp
@@ -295,13 +342,13 @@ ica.plot_overlay(raw, exclude=[0], picks='eeg')
 # heartbeats
 ica.plot_overlay(raw, exclude=[1], picks='mag')
 
-###############################################################################
+# %%
 # We can also plot some diagnostics of each IC using
 # `~mne.preprocessing.ICA.plot_properties`:
 
 ica.plot_properties(raw, picks=[0, 1])
 
-###############################################################################
+# %%
 # In the remaining sections, we'll look at different ways of choosing which ICs
 # to exclude prior to reconstructing the sensor signals.
 #
@@ -321,7 +368,7 @@ ica.plot_properties(raw, picks=[0, 1])
 
 ica.exclude = [0, 1]  # indices chosen based on various plots above
 
-###############################################################################
+# %%
 # Now that the exclusions have been set, we can reconstruct the sensor signals
 # with artifacts removed using the `~mne.preprocessing.ICA.apply` method
 # (remember, we're applying the ICA solution from the *filtered* data to the
@@ -338,7 +385,7 @@ reconst_raw.plot(order=artifact_picks, n_channels=len(artifact_picks),
                  show_scrollbars=False)
 del reconst_raw
 
-###############################################################################
+# %%
 # Using an EOG channel to select ICA components
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -370,7 +417,7 @@ ica.plot_sources(raw, show_scrollbars=False)
 # plot ICs applied to the averaged EOG epochs, with EOG matches highlighted
 ica.plot_sources(eog_evoked)
 
-###############################################################################
+# %%
 # Note that above we used `~mne.preprocessing.ICA.plot_sources` on both
 # the original `~mne.io.Raw` instance and also on an
 # `~mne.Evoked` instance of the extracted EOG artifacts. This can be
@@ -416,7 +463,7 @@ ica.plot_sources(raw, show_scrollbars=False)
 # plot ICs applied to the averaged ECG epochs, with ECG matches highlighted
 ica.plot_sources(ecg_evoked)
 
-###############################################################################
+# %%
 # The last of these plots is especially useful: it shows us that the heartbeat
 # artifact is coming through on *two* ICs, and we've only caught one of them.
 # In fact, if we look closely at the output of
@@ -447,17 +494,22 @@ new_ica.plot_sources(raw, show_scrollbars=False)
 # plot ICs applied to the averaged ECG epochs, with ECG matches highlighted
 new_ica.plot_sources(ecg_evoked)
 
-###############################################################################
+# %%
 # Much better! Now we've captured both ICs that are reflecting the heartbeat
 # artifact (and as a result, we got two diagnostic plots: one for each IC that
 # reflects the heartbeat). This demonstrates the value of checking the results
 # of automated approaches like `~mne.preprocessing.ICA.find_bads_ecg`
 # before accepting them.
 
+# %%
+# For EEG, activation of muscles for postural control of the head and neck
+# contaminate the signal as well. This is usually not detected by MEG. For
+# an example showing how to remove these components, see :ref:`ex-muscle-ica`.
+
 # clean up memory before moving on
 del raw, ica, new_ica
 
-###############################################################################
+# %%
 # Selecting ICA components using template matching
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -465,8 +517,8 @@ del raw, ica, new_ica
 # an IC for exclusion on one subject, and then use that component as a
 # *template* for selecting which ICs to exclude from other subjects' data,
 # using `mne.preprocessing.corrmap` :footcite:`CamposViolaEtAl2009`.
-# The idea behind
-# `~mne.preprocessing.corrmap` is that the artifact patterns are similar
+# The idea behind `~mne.preprocessing.corrmap` is that the artifact patterns
+# are similar
 # enough across subjects that corresponding ICs can be identified by
 # correlating the ICs from each ICA solution with a common template, and
 # picking the ICs with the highest correlation strength.
@@ -480,41 +532,25 @@ del raw, ica, new_ica
 # dataset has 109 subjects, we'll just download one run (a left/right hand
 # movement task) from each of the first 4 subjects:
 
-mapping = {
-    'Fc5.': 'FC5', 'Fc3.': 'FC3', 'Fc1.': 'FC1', 'Fcz.': 'FCz', 'Fc2.': 'FC2',
-    'Fc4.': 'FC4', 'Fc6.': 'FC6', 'C5..': 'C5', 'C3..': 'C3', 'C1..': 'C1',
-    'Cz..': 'Cz', 'C2..': 'C2', 'C4..': 'C4', 'C6..': 'C6', 'Cp5.': 'CP5',
-    'Cp3.': 'CP3', 'Cp1.': 'CP1', 'Cpz.': 'CPz', 'Cp2.': 'CP2', 'Cp4.': 'CP4',
-    'Cp6.': 'CP6', 'Fp1.': 'Fp1', 'Fpz.': 'Fpz', 'Fp2.': 'Fp2', 'Af7.': 'AF7',
-    'Af3.': 'AF3', 'Afz.': 'AFz', 'Af4.': 'AF4', 'Af8.': 'AF8', 'F7..': 'F7',
-    'F5..': 'F5', 'F3..': 'F3', 'F1..': 'F1', 'Fz..': 'Fz', 'F2..': 'F2',
-    'F4..': 'F4', 'F6..': 'F6', 'F8..': 'F8', 'Ft7.': 'FT7', 'Ft8.': 'FT8',
-    'T7..': 'T7', 'T8..': 'T8', 'T9..': 'T9', 'T10.': 'T10', 'Tp7.': 'TP7',
-    'Tp8.': 'TP8', 'P7..': 'P7', 'P5..': 'P5', 'P3..': 'P3', 'P1..': 'P1',
-    'Pz..': 'Pz', 'P2..': 'P2', 'P4..': 'P4', 'P6..': 'P6', 'P8..': 'P8',
-    'Po7.': 'PO7', 'Po3.': 'PO3', 'Poz.': 'POz', 'Po4.': 'PO4', 'Po8.': 'PO8',
-    'O1..': 'O1', 'Oz..': 'Oz', 'O2..': 'O2', 'Iz..': 'Iz'
-}
-
 raws = list()
 icas = list()
 
 for subj in range(4):
     # EEGBCI subjects are 1-indexed; run 3 is a left/right hand movement task
     fname = mne.datasets.eegbci.load_data(subj + 1, runs=[3])[0]
-    raw = mne.io.read_raw_edf(fname)
+    raw = mne.io.read_raw_edf(fname).load_data().resample(50)
     # remove trailing `.` from channel names so we can set montage
-    raw.rename_channels(mapping)
+    mne.datasets.eegbci.standardize(raw)
     raw.set_montage('standard_1005')
     # high-pass filter
     raw_filt = raw.copy().load_data().filter(l_freq=1., h_freq=None)
-    # fit ICA
-    ica = ICA(n_components=30, max_iter='auto', random_state=97)
-    ica.fit(raw_filt)
+    # fit ICA, using low max_iter for speed
+    ica = ICA(n_components=30, max_iter=100, random_state=97)
+    ica.fit(raw_filt, verbose='error')
     raws.append(raw)
     icas.append(ica)
 
-###############################################################################
+# %%
 # Now let's run `~mne.preprocessing.corrmap`:
 
 # use the first subject as template; use Fpz as proxy for EOG
@@ -523,32 +559,28 @@ ica = icas[0]
 eog_inds, eog_scores = ica.find_bads_eog(raw, ch_name='Fpz')
 corrmap(icas, template=(0, eog_inds[0]))
 
-###############################################################################
+# %%
 # The first figure shows the template map, while the second figure shows all
 # the maps that were considered a "match" for the template (including the
-# template itself). There were only three matches from the four subjects;
-# notice the output message ``No maps selected for subject(s) 1, consider a
-# more liberal threshold``.  By default the threshold is set automatically by
-# trying several values; here it may have chosen a threshold that is too high.
-# Let's take a look at the ICA sources for each subject:
+# template itself). There is one match for each subject, but it's a good idea
+# to also double-check the ICA sources for each subject:
 
 for index, (ica, raw) in enumerate(zip(icas, raws)):
-    fig = ica.plot_sources(raw, show_scrollbars=False)
+    with mne.viz.use_browser_backend('matplotlib'):
+        fig = ica.plot_sources(raw, show_scrollbars=False)
     fig.subplots_adjust(top=0.9)  # make space for title
     fig.suptitle('Subject {}'.format(index))
 
-###############################################################################
-# Notice that subject 1 *does* seem to have an IC that looks like it reflects
-# blink artifacts (component ``ICA000``). Notice also that subject 3 appears to
-# have *two* components that are reflecting ocular artifacts (``ICA000`` and
-# ``ICA002``), but only one was caught by `~mne.preprocessing.corrmap`.
-# Let's try setting the threshold manually:
+# %%
+# Notice that subjects 2 and 3 each seem to have *two* ICs that reflect ocular
+# activity (components ``ICA000`` and ``ICA002``), but only one was caught by
+# `~mne.preprocessing.corrmap`. Let's try setting the threshold manually:
 
 corrmap(icas, template=(0, eog_inds[0]), threshold=0.9)
 
-###############################################################################
-# Now we get the message ``At least 1 IC detected for each subject`` (which is
-# good). At this point we'll re-run `~mne.preprocessing.corrmap` with
+# %%
+# This time it found 2 ICs for each of subjects 2 and 3 (which is good).
+# At this point we'll re-run `~mne.preprocessing.corrmap` with
 # parameters ``label='blink', plot=False`` to *label* the ICs from each subject
 # that capture the blink artifacts (without plotting them again).
 
@@ -556,22 +588,22 @@ corrmap(icas, template=(0, eog_inds[0]), threshold=0.9, label='blink',
         plot=False)
 print([ica.labels_ for ica in icas])
 
-###############################################################################
+# %%
 # Notice that the first subject has 3 different labels for the IC at index 0:
 # "eog/0/Fpz", "eog", and "blink". The first two were added by
-# `~mne.preprocessing.ICA.find_bads_eog`; the "blink" label was added by
-# the last call to `~mne.preprocessing.corrmap`. Notice also that each
-# subject has at least one IC index labelled "blink", and subject 3 has two
+# `~mne.preprocessing.ICA.find_bads_eog`; the "blink" label was added by the
+# last call to `~mne.preprocessing.corrmap`. Notice also that each subject has
+# at least one IC index labelled "blink", and subjects 2 and 3 each have two
 # components (0 and 2) labelled "blink" (consistent with the plot of IC sources
-# above). The ``labels_`` attribute of `~mne.preprocessing.ICA` objects
-# can also be manually edited to annotate the ICs with custom labels. They also
+# above). The ``labels_`` attribute of `~mne.preprocessing.ICA` objects can
+# also be manually edited to annotate the ICs with custom labels. They also
 # come in handy when plotting:
 
 icas[3].plot_components(picks=icas[3].labels_['blink'])
 icas[3].exclude = icas[3].labels_['blink']
 icas[3].plot_sources(raws[3], show_scrollbars=False)
 
-###############################################################################
+# %%
 # As a final note, it is possible to extract ICs numerically using the
 # `~mne.preprocessing.ICA.get_components` method of
 # `~mne.preprocessing.ICA` objects. This will return a :class:`NumPy
@@ -584,7 +616,7 @@ template_eog_component = icas[0].get_components()[:, eog_inds[0]]
 corrmap(icas, template=template_eog_component, threshold=0.9)
 print(template_eog_component)
 
-###############################################################################
+# %%
 # An advantage of using this numerical representation of an IC to capture a
 # particular artifact pattern is that it can be saved and used as a template
 # for future template-matching tasks using `~mne.preprocessing.corrmap`
@@ -606,7 +638,7 @@ print(template_eog_component)
 # .. _`this EEGLAB tutorial`: https://labeling.ucsd.edu/tutorial/labels
 
 
-###############################################################################
+# %%
 # Compute ICA components on Epochs
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -618,7 +650,7 @@ print(template_eog_component)
 #     This example is computationally intensive, so it might take a few minutes
 #     to complete.
 #
-# Read and preprocess the data. Preprocessing consists of:
+# After reading the data, preprocessing consists of:
 #
 # - MEG channel selection
 # - 1-30 Hz band-pass filter
@@ -634,14 +666,14 @@ filt_raw.pick_types(meg=True, eeg=False, exclude='bads', stim=True).load_data()
 filt_raw.filter(1, 30, fir_design='firwin')
 
 # peak-to-peak amplitude rejection parameters
-reject = dict(grad=4000e-13, mag=4e-12)
+reject = dict(mag=4e-12)
 # create longer and more epochs for more artifact exposure
 events = mne.find_events(filt_raw, stim_channel='STI 014')
 # don't baseline correct epochs
 epochs = mne.Epochs(filt_raw, events, event_id=None, tmin=-0.2, tmax=0.5,
                     reject=reject, baseline=None)
 
-###############################################################################
+# %%
 # Fit ICA model using the FastICA algorithm, detect and plot components
 # explaining ECG artifacts.
 
@@ -652,15 +684,15 @@ ecg_inds, scores = ica.find_bads_ecg(ecg_epochs, threshold='auto')
 
 ica.plot_components(ecg_inds)
 
-###############################################################################
+# %%
 # Plot the properties of the ECG components:
 ica.plot_properties(epochs, picks=ecg_inds)
 
-###############################################################################
+# %%
 # Plot the estimated sources of detected ECG related components:
 ica.plot_sources(filt_raw, picks=ecg_inds)
 
-###############################################################################
+# %%
 # References
 # ^^^^^^^^^^
 # .. footbibliography::
