@@ -487,22 +487,17 @@ for title in ('Without', 'With'):
 
 evoked_eeg = epochs.average().pick('eeg')
 evoked_eeg.del_proj().add_proj(ecg_projs).add_proj(eog_projs)
-fig, axes = plt.subplots(1, 3, figsize=(8, 3), squeeze=False)
-for ii in range(axes.shape[0]):
-    axes[ii, 0].get_shared_y_axes().join(*axes[ii])
+fig, axes = plt.subplots(1, 3, figsize=(8, 3), sharex=True, sharey=True)
 for pi, proj in enumerate((False, True, 'reconstruct')):
-    evoked_eeg.plot(proj=proj, axes=axes[:, pi], spatial_colors=True)
-    if pi == 0:
-        for ax in axes[:, pi]:
-            parts = ax.get_title().split('(')
-            ax.set(ylabel=f'{parts[0]} ({ax.get_ylabel()})\n'
-                          f'{parts[1].replace(")", "")}')
-    axes[0, pi].set(title=f'proj={proj}')
-    for text in list(axes[0, pi].texts):
+    ax = axes[pi]
+    evoked_eeg.plot(proj=proj, axes=ax, spatial_colors=True)
+    parts = ax.get_title().split('(')
+    ylabel = (f'{parts[0]} ({ax.get_ylabel()})\n{parts[1].replace(")", "")}'
+              if pi == 0 else '')
+    ax.set(ylabel=ylabel, title=f'proj={proj}')
+    ax.yaxis.set_tick_params(labelbottom=True)
+    for text in list(ax.texts):
         text.remove()
-plt.setp(axes[1:, :].ravel(), title='')
-plt.setp(axes[:, 1:].ravel(), ylabel='')
-plt.setp(axes[:-1, :].ravel(), xlabel='')
 mne.viz.tight_layout()
 
 # %%
