@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 .. _tut-ssvep:
 
@@ -34,21 +35,20 @@ and statistically separate 12 Hz and 15 Hz responses in the different trials.
 Since the evoked response is mainly generated in early visual areas of the
 brain the statistical analysis will be carried out on an occipital
 ROI.
-
-.. contents:: Outline
-   :depth: 2
 """  # noqa: E501
 # Authors: Dominik Welke <dominik.welke@web.de>
 #          Evgenii Kalenkovich <e.kalenkovich@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
+
+# %%
 
 import matplotlib.pyplot as plt
 import mne
 import numpy as np
 from scipy.stats import ttest_rel
 
-###############################################################################
+# %%
 # Data preprocessing
 # ------------------
 # Due to a generally high SNR in SSVEP/vSSR, typical preprocessing steps
@@ -65,7 +65,8 @@ from scipy.stats import ttest_rel
 
 # Load raw data
 data_path = mne.datasets.ssvep.data_path()
-bids_fname = data_path + '/sub-02/ses-01/eeg/sub-02_ses-01_task-ssvep_eeg.vhdr'
+bids_fname = (data_path / 'sub-02' / 'ses-01' / 'eeg' /
+              'sub-02_ses-01_task-ssvep_eeg.vhdr')
 
 raw = mne.io.read_raw_brainvision(bids_fname, preload=True, verbose=False)
 raw.info['line_freq'] = 50.
@@ -86,7 +87,6 @@ event_id = {
     '15hz': 155
 }
 events, _ = mne.events_from_annotations(raw, verbose=False)
-raw.info["events"] = events
 tmin, tmax = -1., 20.  # in s
 baseline = None
 epochs = mne.Epochs(
@@ -94,7 +94,7 @@ epochs = mne.Epochs(
     event_id=[event_id['12hz'], event_id['15hz']], tmin=tmin,
     tmax=tmax, baseline=baseline, verbose=False)
 
-###############################################################################
+# %%
 # Frequency analysis
 # ------------------
 # Now we compute the frequency spectrum of the EEG data.
@@ -138,17 +138,18 @@ fmin = 1.
 fmax = 90.
 sfreq = epochs.info['sfreq']
 
-psds, freqs = mne.time_frequency.psd_welch(
-    epochs,
+spectrum = epochs.compute_psd(
+    'welch',
     n_fft=int(sfreq * (tmax - tmin)),
     n_overlap=0, n_per_seg=None,
     tmin=tmin, tmax=tmax,
     fmin=fmin, fmax=fmax,
     window='boxcar',
     verbose=False)
+psds, freqs = spectrum.get_data(return_freqs=True)
 
 
-###############################################################################
+# %%
 # Calculate signal to noise ratio (SNR)
 # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 #
@@ -215,7 +216,7 @@ def snr_spectrum(psd, noise_n_neighbor_freqs=1, noise_skip_neighbor_freqs=1):
     return psd / mean_noise
 
 
-###############################################################################
+# %%
 # Now we call the function to compute our SNR spectrum.
 #
 # As described above, we have to define two parameters.
@@ -277,7 +278,7 @@ axes[1].set(
     ylabel='SNR', ylim=[-2, 30], xlim=[fmin, fmax])
 fig.show()
 
-###############################################################################
+# %%
 # You can see that the peaks at the stimulation frequencies (12 Hz, 15 Hz)
 # and their harmonics are visible in both plots (just as the line noise at
 # 50 Hz).
@@ -297,8 +298,8 @@ fig.show()
 # Extract SNR values at the stimulation frequency
 # -----------------------------------------------
 #
-# Our processing yielded a large array of many SNR values for each trial x
-# channel x frequency-bin of the PSD array.
+# Our processing yielded a large array of many SNR values for each trial ×
+# channel × frequency-bin of the PSD array.
 #
 # For statistical analysis we obviously need to define specific subsets of this
 # array. First of all, we are only interested in SNR at the stimulation
@@ -317,7 +318,7 @@ fig.show()
 # define stimulation frequency
 stim_freq = 12.
 
-###############################################################################
+# %%
 # Get index for the stimulation frequency (12 Hz)
 # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 # Ideally, there would be a bin with the stimulation frequency exactly in its
@@ -338,14 +339,14 @@ i_bin_15hz = np.argmin(abs(freqs - 15))
 i_bin_30hz = np.argmin(abs(freqs - 30))
 i_bin_45hz = np.argmin(abs(freqs - 45))
 
-###############################################################################
+# %%
 # Get indices for the different trial types
 # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
 i_trial_12hz = np.where(epochs.events[:, 2] == event_id['12hz'])[0]
 i_trial_15hz = np.where(epochs.events[:, 2] == event_id['15hz'])[0]
 
-###############################################################################
+# %%
 # Get indices of EEG channels forming the ROI
 # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
@@ -357,7 +358,7 @@ roi_vis = ['POz', 'Oz', 'O1', 'O2', 'PO3', 'PO4', 'PO7',
 picks_roi_vis = mne.pick_types(epochs.info, eeg=True, stim=False,
                                exclude='bads', selection=roi_vis)
 
-###############################################################################
+# %%
 # Apply the subset, and check the result
 # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 # Now we simply need to apply our selection and yield a result. Therefore,
@@ -394,7 +395,7 @@ snrs_12hz_chaverage = snrs_12hz.mean(axis=0)
 
 # plot SNR topography
 fig, ax = plt.subplots(1)
-mne.viz.plot_topomap(snrs_12hz_chaverage, epochs.info, vmin=1., axes=ax)
+mne.viz.plot_topomap(snrs_12hz_chaverage, epochs.info, vlim=(1, None), axes=ax)
 
 print("sub 2, 12 Hz trials, SNR at 12 Hz")
 print("average SNR (all channels): %f" % snrs_12hz_chaverage.mean())
@@ -483,7 +484,7 @@ ax.set_ylim([0, 70])
 ax.axhline(1, ls='--', c='r')
 fig.show()
 
-###############################################################################
+# %%
 # As you can easily see there are striking differences between the trials.
 # Let's verify this using a series of two-tailed paired T-Tests.
 #
@@ -558,13 +559,14 @@ window_lengths = [i for i in range(2, 21, 2)]
 window_snrs = [[]] * len(window_lengths)
 for i_win, win in enumerate(window_lengths):
     # compute spectrogram
-    windowed_psd, windowed_freqs = mne.time_frequency.psd_welch(
-        epochs[str(event_id['12hz'])],
+    this_spectrum = epochs[str(event_id['12hz'])].compute_psd(
+        'welch',
         n_fft=int(sfreq * win),
         n_overlap=0, n_per_seg=None,
         tmin=0, tmax=win,
         window='boxcar',
         fmin=fmin, fmax=fmax, verbose=False)
+    windowed_psd, windowed_freqs = this_spectrum.get_data(return_freqs=True)
     # define a bandwidth of 1 Hz around stimfreq for SNR computation
     bin_width = windowed_freqs[1] - windowed_freqs[0]
     skip_neighbor_freqs = \
@@ -630,14 +632,15 @@ window_snrs = [[]] * len(window_starts)
 
 for i_win, win in enumerate(window_starts):
     # compute spectrogram
-    windowed_psd, windowed_freqs = mne.time_frequency.psd_welch(
-        epochs[str(event_id['12hz'])],
+    this_spectrum = epochs[str(event_id['12hz'])].compute_psd(
+        'welch',
         n_fft=int(sfreq * window_length) - 1,
         n_overlap=0, n_per_seg=None,
         window='boxcar',
         tmin=win, tmax=win + window_length,
         fmin=fmin, fmax=fmax,
         verbose=False)
+    windowed_psd, windowed_freqs = this_spectrum.get_data(return_freqs=True)
     # define a bandwidth of 1 Hz around stimfreq for SNR computation
     bin_width = windowed_freqs[1] - windowed_freqs[0]
     skip_neighbor_freqs = \
@@ -657,7 +660,7 @@ for i_win, win in enumerate(window_starts):
             abs(windowed_freqs - 12.))].mean(axis=1)
 
 fig, ax = plt.subplots(1)
-colors = plt.get_cmap('Greys')(np.linspace(0, 1, 10))
+colors = plt.colormaps['Greys'](np.linspace(0, 1, 10))
 for i in range(10):
     ax.plot(window_starts, np.array(window_snrs)[:, i], color=colors[i])
 ax.set(title='Time resolved 12 Hz SNR - %is sliding window' % window_length,
